module adds
        use ModGestListe
        use utils
        implicit none
!!********************************************************************
!!     Module adds
!!
!!         This module contains all global lists
!!
!>  \ingroup  globalVariables
!!  \brief Homotopic path
    type(TLISTE), save :: pathG
!>  \ingroup  globalVariables
!!  \brief Tangents vectors at bifurcations
    type(TLISTE), save :: biftocG
!>  \ingroup  globalVariables
!!  \brief Points at bifurcations
    type(TLISTE), save :: bifcG
!>  \ingroup  globalVariables
!!  \brief States-costates from t0 to tf
    type(TLISTE), save :: etatsG
!>  \ingroup  globalVariables
!!  \brief Jacobian fields from t0 to tf
    type(TLISTE), save :: etatsdhvG

    contains
!!********************************************************************
!!
!!     Subroutine initListe
!!
!>     \ingroup commonLists
!!     \brief initialize the right lists depending on the input 
!!            parameter
!!
!!     if n = 1 then hampath   initialization
!!     if n = 2 then exphvfun  initialization
!!     if n = 3 then expdhvfun initialization
!!
!!     \param[in] n between 1 and 3
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    subroutine initListe(n)
        implicit none
        integer, intent(in) :: n

        select case (n)
        case(1)
            call CREER_STRUCTURE(pathG)
            call CREER_STRUCTURE(bifcG)
            call CREER_STRUCTURE(biftocG)
        case(2)
            call CREER_STRUCTURE(etatsG)
        case(3)
            call CREER_STRUCTURE(etatsdhvG)
        case default
            CALL printandstop('Error: initListe')
        end select

    end subroutine initListe


!!********************************************************************
!!
!!     Subroutine addpath
!!
!>     \ingroup commonLists
!!     \brief   adds to the list pathG
!!
!!     Detailed description :
!!     The input variables are concatenated and added to the list pathG. This
!!     list contains information about the path of zeros computed by hampath()
!!
!!        \param[in]    s      arc length where c, det, norms and pds are computed
!!        \param[in]    n1     size of c
!!        \param[in]    scaletab    scale
!!        \param[in]    c      vector containing x, p and lambda at s
!!        \param[out]   cscalediv     scaled vector containing x, p and lambda at s
!!        \param[in]    toc    tangent vector at c
!!        \param[in]    det    det(S'(c(s)),c'(s))
!!        \param[in]    norms  |S(c(s))|
!!        \param[in]    pds    <c'(s_old)|c'(s_new)> (scalar product)
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine addpath(s,n1,scaletab,c,cscalediv,toc,det,norms,pds)
        implicit none

        integer, intent(in)                          :: n1
        double precision, intent(in)                 :: s
        double precision, intent(in)                 :: det
        double precision, intent(in)                 :: norms
        double precision, intent(in)                 :: pds
        double precision, intent(in), dimension(n1)  :: c
        double precision, intent(in), dimension(n1)  :: toc
        double precision, intent(in), dimension(n1)  :: scaletab
        double precision, intent(out), dimension(n1) :: cscalediv

        !local variables
        double precision, dimension(3+2*n1) :: aux

        call scalediv(n1,c,scaletab,cscalediv)

        aux(1:n1) = cscalediv
        aux(n1+1:2*n1) = toc
        aux(2*n1+1) = det
        aux(2*n1+2) = norms
        aux(2*n1+3) = pds

       call INSERER(s,3+2*n1,aux,pathG)

    end subroutine addpath


!!********************************************************************
!!
!!     Subroutine addbiftoc
!!
!>     \ingroup commonLists
!!     \brief   adds to the list biftocG
!!
!!     Detailed description :
!!     The parameter val corresponds to the two tangent vectors at a bifurcation
!!     point. This method adds val to the list biftocG which contains
!!     all tangent vectors at each bifurcation
!!
!!        \param[in]    s     arc length where val is computed
!!        \param[in]    n     size of val
!!        \param[in]    val   vector containing the concatenation of two
!!                            tagent vectors
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   Subroutine addbiftoc(n,s,val)
       implicit none

       integer, intent(in)                    :: n
       double precision, intent(in)               :: s
       double precision, intent(in), dimension(n) :: val

       call INSERER(s,n,val,biftocG)

     end subroutine addbiftoc

!!********************************************************************
!!
!!     Subroutine addbifc
!!
!>     \ingroup commonLists
!!     \brief   adds to the list bifcG
!!
!!     Detailed description :
!!     This method adds c, which is a bifurcation point, to the list
!!     bifcG
!!
!!        \param[in]    s     arc length where val is computed
!!        \param[in]    n     size of val
!!        \param[in]    c     vector containing x, p and lambda at s
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
     Subroutine addbifc(n,s,c)
       implicit none

       integer, intent(in)                    :: n
       double precision, intent(in)               :: s
       double precision, intent(in), dimension(n) :: c

       call INSERER(s,n,c,bifcG)

     end subroutine addbifc

!!********************************************************************
!!
!!     Subroutine addstates
!!
!>     \ingroup commonLists
!!     \brief   adds to the list etatsG
!!
!!     Detailed description :
!!     This method adds z to the list etatsG which contains the output 
!!     for exphvfun() 
!!
!!        \param[in]    nz     size of z
!!        \param[in]    scaletab    scale
!!        \param[in]    time   time where z is computed
!!        \param[in]    z      vector containing x and p (flow at time)
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine addstateshv(nz,scaletab,time,z)
        implicit none

        integer, intent(in)                             :: nz
        double precision, intent(in)                    :: time
        double precision, intent(in), dimension(nz)     :: z
        double precision, intent(in), dimension(nz)     :: scaletab

        !local variables
        double precision, dimension(nz) :: zscalediv

        call scalediv(nz,z,scaletab,zscalediv)

        if(LONGUEUR(etatsG).GE.1)then
            if(time.NE.GET_TIME(etatsG))then !we can not have two elements with the same time
                call INSERER(time,nz,zscalediv,etatsG)
            end if
        else
            call INSERER(time,nz,zscalediv,etatsG)
        end if

    end subroutine addstateshv


!!********************************************************************
!!
!!     Subroutine addstatesdhv
!!
!>     \ingroup commonLists
!!     \brief   adds to the list etatsdhvG
!!
!!     Detailed description :
!!     This method adds dz to the list etatsG which contains the output 
!!     for expdhvfun() 
!!
!!        \param[in]    ndz   size of dz
!!        \param[in]    scaletab    scale
!!        \param[in]    time  time where dz is computed
!!        \param[in]    dz    vector containing the flow the jacobi fields
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine addstatesdhv(ndz,scaletab,time,dz)
        implicit none

        integer, intent(in)                             :: ndz
        double precision, intent(in)                    :: time
        double precision, intent(in), dimension(ndz)    :: dz
        double precision, intent(in), dimension(ndz)    :: scaletab

        !local variables
        double precision :: t
        double precision, dimension(ndz) :: val
        double precision, dimension(ndz) :: dzscalediv

        call scalediv(ndz,dz,scaletab,dzscalediv)

        if(LONGUEUR(etatsdhvG).GE.1)then
            if(time.NE.GET_TIME(etatsdhvG))then !we can not have two elements with the same time
                call INSERER(time,ndz,dzscalediv,etatsdhvG)
            else
                call EXTRAIRE(t,ndz,val,etatsdhvG)
                call INSERER(time,ndz,dzscalediv,etatsdhvG)
            end if
        else
            call INSERER(time,ndz,dzscalediv,etatsdhvG)
        end if

    end subroutine addstatesdhv

end module adds

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!


