module derivatives 
    implicit none
    contains

!!********************************************************************
!!
!!     Subroutine dhvdz
!!
!>     \ingroup commonDerivatives
!!     \brief   Partial derivative of hvfun with respect to z
!!     \param[in]   t time
!!     \param[in]   n State dimension
!!     \param[in]   z State and adjoint state   at t
!!     \param[in]   k   part of the dimension of dz
!!     \param[in]   dz  the derivative is applied along dz, put in vector form
!!     \param[in]   nbarc    Number of arcs
!!     \param[in]   iarc Index of the current arc
!!     \param[in]   lpar Number of optional parameters
!!     \param[in]   par Optional parameters
!!
!!     \param[out]   val (dhv/dz)(t,z).dz
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
      Subroutine dhvdz(t,n,z,k,dz,nbarc,iarc,lpar,par,val)
        implicit none
        integer,            intent(in)                    :: n,nbarc,k
        integer,            intent(in)                    :: lpar
        integer,            intent(in)                    :: iarc
        double precision,   intent(in)                    :: t
        double precision,   intent(in), dimension(2*n)    :: z
        double precision,   intent(in), dimension(2*n*k)  :: dz
        double precision,   intent(in), dimension(lpar)   :: par
        double precision,   intent(out), dimension(2*n*k) :: val

        !Local variables
        double precision, dimension(lpar)   :: l0
        double precision, dimension(2*n)    :: zdd, work !zd
        double precision                    :: H
        double precision                    :: hd,hd0
        integer                             :: i
        integer                             :: j

        work = 0.0D0
        l0 = 0d0
        DO i = 1, n
            DO j = 1, k
                work(i+n) = 1.0D0
                CALL HFUN_D_D(t, 0d0, 0d0, n, z, dz(1+(j-1)*2*n:1+(j-1)*2*n+2*n-1), work, nbarc, iarc, &
                                lpar, par, l0, l0, H, hd, val(1+(j-1)*2*n+i-1))
                work(i+n) = 0.0D0

                work(i  ) =-1.0D0
                CALL HFUN_D_D(t, 0d0, 0d0, n, z, dz(1+(j-1)*2*n:1+(j-1)*2*n+2*n-1), work, nbarc, iarc, &
                                lpar, par, l0, l0, H, hd, val(1+(j-1)*2*n+i+n-1))
                work(i  ) = 0.0D0
            END DO
        END DO

!!        zdd = 0.0d0
!!        DO i = 1, n
!!            DO j = 1, k
!!                work(i+n) = 1.0D0
!!                CALL HFUN_D_D(t, 0d0, 0d0, 0d0, n, z, dz(1+(j-1)*2*n:1+(j-1)*2*n+2*n-1), work, zdd, nbarc, iarc, &
!!                                lpar, par, l0, l0, l0, H, hd0, hd, val(1+(j-1)*2*n+i-1))
!!                work(i+n) = 0.0D0
!!
!!                work(i  ) =-1.0D0
!!                CALL HFUN_D_D(t, 0d0, 0d0, 0d0, n, z, dz(1+(j-1)*2*n:1+(j-1)*2*n+2*n-1), work, zdd, nbarc, iarc, &
!!                                lpar, par, l0, l0, l0, H, hd0, hd, val(1+(j-1)*2*n+i-1))
!!                work(i  ) = 0.0D0
!!            END DO
!!        END DO

     end subroutine dhvdz

!!********************************************************************
!!
!!     Subroutine dhvdlambda
!!
!>     \ingroup commonDerivatives
!!     \brief  Partial derivative of hvfun with respect to par(indLG)
!!     \param[in]   t time
!!     \param[in]   n State dimension
!!     \param[in]   z State and adjoint state   at t
!!     \param[in]   nbarc    Number of arcs
!!     \param[in]   iarc Index of the current arc
!!     \param[in] lambda Value of the current homotopic parameter
!!     \param[in]   lpar Number of optional parameters
!!     \param[in]   par Optional parameters
!!     \param[in]   vswitch Switching function value at (t,z)
!!
!!     \param[out]   value (dhv/dpar)(t,z)
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
      Subroutine dhvdlambda(t,n,z,nbarc,iarc,lambda,lpar,par,value)
        implicit none
        integer,          intent(in)                  :: n,nbarc
        integer,          intent(in)                  :: lpar
        integer,          intent(in)                  :: iarc
        double precision, intent(in)                  :: t,lambda
        double precision, intent(in), dimension(2*n)  :: z
        double precision, intent(in), dimension(lpar) :: par
        double precision, intent(out), dimension(2*n) :: value

        !.. Local declarations ..
        double precision, dimension(2*n)  :: zd
        double precision, dimension(2*n)  :: zd0
        double precision, dimension(lpar) :: pard
        double precision, dimension(lpar) :: pard0
        double precision                  :: H
        double precision                  :: hd,hd0
        integer                           :: i

        call dpardl(lambda,lpar,pard0)

        zd0  = 0d0
        zd = 0d0
        pard = 0d0
        DO i = 1, n
            zd(i+n) = 1.0D0
            CALL HFUN_D_D(t, 0d0, 0d0, n, z, zd0, zd, nbarc, iarc, &
                            lpar, par, pard0, pard, H, hd, value(i))
            zd(i+n) = 0.0D0

            zd(i  ) =-1.0D0
            CALL HFUN_D_D(t, 0d0, 0d0, n, z, zd0, zd, nbarc, iarc, &
                            lpar, par, pard0, pard, H, hd, value(i+n))
            zd(i  ) = 0.0D0
        END DO

!!        DO i = 1, n
!!            zd(i+n) = 1.0D0
!!            CALL HFUN_D_D(t, 0d0, 0d0, 0d0, n, z, zd0, zd, 0d0, nbarc, iarc, &
!!                            lpar, par, pard0, pard, 0d0, H, hd0, hd, value(i))
!!            zd(i+n) = 0.0D0
!!
!!            zd(i  ) =-1.0D0
!!            CALL HFUN_D_D(t, 0d0, 0d0, 0d0, n, z, zd0, zd, 0d0, nbarc, iarc, &
!!                            lpar, par, pard0, pard, 0d0, H, hd0, hd, value(i))
!!            zd(i  ) = 0.0D0
!!        END DO

     end subroutine dhvdlambda

!!********************************************************************
!!
!!     Subroutine dpardl
!!
!>     \ingroup commonDerivatives
!!     \brief Partial derivative of parfun with respect to lambda
!!       \param[in] lambda   Current homotopic parameter
!!       \param[in] lpar dimension of value
!!
!!       \param[out] value dpar/dl
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine dpardl(lambda,lpar,value)
        use defs
        implicit none
        integer,          intent(in)                   :: lpar
        double precision, intent(in)                   :: lambda
        double precision, intent(out), dimension(lpar) :: value

        !local variables
        double precision                  :: lambdad
        double precision, dimension(lpar) :: par

        if(nparhomG.eq.1)then
            value = 0d0
            value(iparhomG(1)) = 1d0 !if there is one homotopic parameter, the derivative is always 1
        else
            lambdad  = 1d0
            call PARFUN_D(lambda, lambdad, lpar, par0G, parfG, par, value)
        end if

    end subroutine dpardl

!!********************************************************************
!!
!!     Subroutine dedlambda
!!
!>     \ingroup commonDerivatives
!!     \brief Partial derivative of efun with respect to par(indLG)
!!       \param[in] nbarc      Number of arcs
!!       \param[in] n      State dimension
!!       \param[in] ti     ti = [t0 t1 .. t_nbarc-1 tf], where t0 is the initial time and tf the final time.
!!       \param[in] zi     State and adjoint states at [t0 t1 .. t_nbarc-1].
!!       \param[in] expzi  expzi = z(t_i+1;zi,ti) => expzi(:,nbarc) = zf.
!!       \param[in] lambda Value of the current homotopic parameter
!!       \param[in] lpar   Number of optional parameters
!!       \param[in] par    Optional parameters
!!       \param[in] ne     Dimension of the shooting function
!!
!!        \param[out] value  de/dlambda
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine dedlambda(nbarc,n,ti,zi,expzi,lambda,lpar,par,ne,value)
        implicit none
        integer, intent(in)                                 :: nbarc,n,lpar,ne
        double precision, intent(in),  dimension(nbarc+1)   :: ti
        double precision, intent(in),  dimension(2*n,nbarc) :: zi
        double precision, intent(in),  dimension(2*n,nbarc) :: expzi
        double precision, intent(in)                        :: lambda
        double precision, intent(in),  dimension(lpar)      :: par
        double precision, intent(out), dimension(ne,lpar)   :: value

        !local variables
        double precision, dimension(ne)         :: E
        double precision, dimension(nbarc+1)    :: tid
        double precision, dimension(2*n, nbarc) :: zid
        double precision, dimension(2*n, nbarc) :: expzid
        double precision, dimension(lpar)       :: pard

        tid = 0d0
        zid = 0d0
        expzid = 0d0
        call dpardl(lambda,lpar,pard)
        call EFUN_D(nbarc,n,ti,tid,zi,zid,expzi,expzid,lpar,par,pard,ne,E,value)

   end subroutine dedlambda

!!********************************************************************
!!
!!     Subroutine dedti
!!
!>     \ingroup commonDerivatives
!!     \brief Partial derivative of efun with respect to ti
!!       \param[in] nbarc      Number of arcs
!!       \param[in] n      State dimension
!!       \param[in] ti     ti = [t0 t1 .. t_nbarc-1 tf], where t0 is the initial time and tf the final time.
!!       \param[in] zi     State and adjoint states at [t0 t1 .. t_nbarc-1].
!!       \param[in] expzi  expzi = z(t_i+1;zi,ti) => expzi(:,nbarc) = zf.
!!       \param[in] lpar   Number of optional parameters
!!       \param[in] par    Optional parameters
!!       \param[in] ne     Dimension of the shooting function
!!
!!        \param[out] value de/dti
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   Subroutine dedti(nbarc,n,ti,zi,expzi,lpar,par,ne,value)
      implicit none
      integer, intent(in)                                   :: nbarc,n,lpar,ne
      double precision, intent(in),  dimension(nbarc+1)     :: ti
      double precision, intent(in),  dimension(2*n,nbarc)   :: zi
      double precision, intent(in),  dimension(2*n,nbarc)   :: expzi
      double precision, intent(in),  dimension(lpar)        :: par
      double precision, intent(out), dimension(ne,nbarc+1)  :: value

      !local variables
      integer                                   :: i
      double precision, dimension(ne)           :: E
      double precision, dimension(nbarc+1)      :: tid
      double precision, dimension(2*n, nbarc)   :: zid
      double precision, dimension(2*n, nbarc)   :: expzid
      double precision, dimension(lpar)         :: pard

      tid = 0d0
      zid = 0d0
      expzid = 0d0
      pard   = 0d0

      do i=1,nbarc+1
          tid(i) = 1d0
      call EFUN_D(nbarc,n,ti,tid,zi,zid,expzi,expzid,lpar,par,pard,ne,E,value(:,i))
          tid(i) = 0d0
      end do

   end subroutine dedti

!!********************************************************************
!!
!!     Subroutine dedzi
!!
!>     \ingroup commonDerivatives
!!     \brief Partial derivative of efun with respect to zi
!!       \param[in] nbarc      Number of arcs
!!       \param[in] n      State dimension
!!       \param[in] ti     ti = [t0 t1 .. t_nbarc-1 tf], where t0 is the initial time and tf the final time.
!!       \param[in] izi    Derivative wrt zi(:,izi)
!!       \param[in] zi     State and adjoint states at [t0 t1 .. t_nbarc-1].
!!       \param[in] expzi  expzi = z(t_i+1;zi,ti) => expzi(:,nbarc) = zf.
!!       \param[in] lpar   Number of optional parameters
!!       \param[in] par    Optional parameters
!!       \param[in] ne     Dimension of the shooting function
!!
!!    \param[out] value de/dzi
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   Subroutine dedzi(nbarc,n,ti,izi,zi,expzi,lpar,par,ne,value)
      implicit none
      integer, intent(in)                                   :: nbarc,n,lpar,izi,ne
      double precision, intent(in),  dimension(nbarc+1)     :: ti
      double precision, intent(in),  dimension(2*n,nbarc)   :: zi
      double precision, intent(in),  dimension(2*n,nbarc)   :: expzi
      double precision, intent(in),  dimension(lpar)        :: par
      double precision, intent(out), dimension(ne,2*n)      :: value

      !local variables
      integer                                   :: i
      double precision, dimension(ne)           :: E
      double precision, dimension(nbarc+1)      :: tid
      double precision, dimension(2*n, nbarc)   :: zid
      double precision, dimension(2*n, nbarc)   :: expzid
      double precision, dimension(lpar)         :: pard

      tid = 0d0
      zid = 0d0
      expzid = 0d0
      pard   = 0d0

      do i=1,2*n
          zid(i,izi) = 1d0
      call EFUN_D(nbarc,n,ti,tid,zi,zid,expzi,expzid,lpar,par,pard,ne,E,value(:,i))
          zid(i,izi) = 0d0
      end do

   end subroutine dedzi

!!********************************************************************
!!
!!     Subroutine dedexpzi
!!
!>     \ingroup commonDerivatives
!!     \brief Partial derivative of efun with respect to zi
!!       \param[in] nbarc      Number of arcs
!!       \param[in] n      State dimension
!!       \param[in] ti     ti = [t0 t1 .. t_nbarc-1 tf], where t0 is the initial time and tf the final time.
!!       \param[in] zi     State and adjoint states at [t0 t1 .. t_nbarc-1].
!!       \param[in] iexpzi Derivative wrt expzi(:,iexpzi)
!!       \param[in] expzi  expzi = z(t_i+1;zi,ti) => expzi(:,nbarc) = zf.
!!       \param[in] lpar   Number of optional parameters
!!       \param[in] par    Optional parameters
!!       \param[in] ne     Dimension of the shooting function
!!
!!    \param[out] value de/dexpzi
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   Subroutine dedexpzi(nbarc,n,ti,zi,iexpzi,expzi,lpar,par,ne,value)
      implicit none
      integer, intent(in)                                   :: nbarc,n,lpar,iexpzi,ne
      double precision, intent(in),  dimension(nbarc+1)     :: ti
      double precision, intent(in),  dimension(2*n,nbarc)   :: zi
      double precision, intent(in),  dimension(2*n,nbarc)   :: expzi
      double precision, intent(in),  dimension(lpar)        :: par
      double precision, intent(out), dimension(ne,2*n)      :: value

      !local variables
      integer                                   :: i
      double precision, dimension(ne)           :: E
      double precision, dimension(nbarc+1)      :: tid
      double precision, dimension(2*n, nbarc)   :: zid
      double precision, dimension(2*n, nbarc)   :: expzid
      double precision, dimension(lpar)         :: pard

      tid = 0d0
      zid = 0d0
      expzid = 0d0
      pard   = 0d0

      do i=1,2*n
          expzid(i,iexpzi) = 1d0
      call EFUN_D(nbarc,n,ti,tid,zi,zid,expzi,expzid,lpar,par,pard,ne,E,value(:,i))
          expzid(i,iexpzi) = 0d0
      end do

   end subroutine dedexpzi

end module derivatives

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!

