!!********************************************************************
!!
!!     Subroutine dopri5Int
!!
!>     \ingroup commonIntegration
!!     \brief   interface for dopri5 integrator
!!     \param[in]       fun         RHS
!!     \param[in]       neq         dimension of the state
!!     \param[in]       y0          initial point
!!     \param[in]       funpar      parameters given to fun during integration
!!     \param[in]       lfunpar     dimension of funpar
!!     \param[in]       ncpas       number of components of y for which we make a step control
!!     \param[in,out]   tin         initial time
!!     \param[in,out]   tout        final time
!!     \param[out]      yf          final point
!!     \param[out]      flag        information on the integration process, should be 1
!!     \param[in]       choixsolout id giving the method which has called dopri5int, see \ref globalVariables
!!     \param[in]       solout      auxiliary function called after accepted steps
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    subroutine dopri5Int(fun,neq,y0,funpar,lfunpar,ncpas,tin,tout,yf,flag,choixsolout,solout)
        use defs
        implicit none
        integer,            intent(in)                           :: lfunpar
        integer,            intent(in)                           :: choixsolout
        integer,            intent(in)                           :: neq
        integer,            intent(in)                           :: ncpas
        double precision,   intent(inout)                        :: tin
        double precision,   intent(inout)                        :: tout
        double precision,   intent(in),  dimension(neq)          :: y0
        double precision,   intent(in),  dimension(lfunpar)      :: funpar
        double precision,   intent(out), dimension(neq)          :: yf
        integer,            intent(out)                          :: flag

        interface
        Subroutine solout(Niter,told,time,y,neq,dpar,ldpar,ipar,lipar,fun,funpar,lfunpar,irtrn)
            integer,            intent(in)                         :: Niter
            double precision,   intent(in)                         :: told
            double precision,   intent(inout)                      :: time
            double precision,   intent(inout), dimension(neq)      :: y
            integer,            intent(in)                         :: neq
            double precision,   intent(in),    dimension(ldpar)     :: dpar
            integer,            intent(in)                         :: ldpar
            integer,            intent(in),    dimension(lipar)    :: ipar
            integer,            intent(in)                         :: lipar
            double precision,   intent(inout), dimension(lfunpar)  :: funpar
            integer,            intent(in)                         :: lfunpar
            integer,            intent(inout)                      :: irtrn
        end Subroutine solout
        end interface

        interface
            Subroutine fun(neq,time,y,value,funpar,lfunpar)
                implicit none
                integer,           intent(in)                      :: neq
                integer,           intent(in)                      :: lfunpar
                double precision,  intent(in)                      :: time
                double precision,  intent(in),  dimension(lfunpar) :: funpar
                double precision,  intent(in),  dimension(neq)     :: y
                double precision,  intent(out), dimension(neq)     :: value
            end Subroutine fun
        end interface

        !local variables
        integer                                     :: iout
        integer                                     :: itol
        integer                                     :: lw
        integer                                     :: liw
        integer                                     :: NRDENS
        double precision                            :: tinin
        double precision                            :: toutout
        integer,      dimension(:), allocatable     :: iwork
        double precision, dimension(:), allocatable :: work
        double precision, dimension(:), allocatable :: atolv
        double precision, dimension(:), allocatable :: rtolv

        NRDENS=0
        select case (choixsolout)
        case(idExpG,idExpdG,idHamG)
            NRDENS = neq
            iout = 2
        case(idSFunG,idSJacG)
            NRDENS = 0
            iout = 0
        end select
        lw = 8*neq + 5*NRDENS + 21
        liw = NRDENS + 21
        allocate(work(lw), iwork(liw))
        work(1:20) = 0d0
        !work(7)=1d-4
        iwork(1:20) = 0
        iwork(3) = -1
        flag = 0
        iwork(5)=NRDENS
        !use vector tolerances to force step control to use only y
        allocate(atolv(neq),rtolv(neq))
        atolv = 0d0
        rtolv = 0d0
        itol = 2
        if (choixsolout.eq.idHamG) then
            atolv(1:ncpas) = atolcontG
            rtolv(1:ncpas) = rtolcontG
            iwork(1) = maxstepscontG
            if(hmaxcontG.gt.0d0)then
                work(6) = hmaxcontG
            end if
        else
            atolv(1:ncpas) = atolG
            rtolv(1:ncpas) = rtolG
            iwork(1) = maxstepsG
            if(hmaxG.gt.0d0)then
                work(6) = hmaxG
            end if
        end if

        yf = y0
        tinin = tin
        toutout = tout

        call dopri5(neq,fun,tinin,yf,toutout,rtolv,atolv,itol,      &
                    solout,iout,work,lw,     &
                    iwork,liw,funpar,lfunpar,flag)

        if (flag.eq.2) then
            flag = 1
        end if

        tout = tinin

!   IWORK(17)  NFCN    NUMBER OF FUNCTION EVALUATIONS
!   IWORK(18)  NSTEP   NUMBER OF COMPUTED STEPS
!   IWORK(19)  NACCPT  NUMBER OF ACCEPTED STEPS
!   IWORK(20)  NREJCT  NUMBER OF REJECTED STEPS (DUE TO ERROR TEST),

        deallocate(work,iwork,atolv,rtolv)

    end subroutine dopri5Int

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!

