   module init
        use defs
        use utils
        implicit none
        contains


!!********************************************************************
!!
!!     Subroutine initTolScaling
!!
!>     \ingroup commonInitialization
!!     \brief   Initialize defs::scaltolg parameter.
!!     \param[in]    tsc      defs::scaltolg value
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   subroutine initTolScaling(tsc)
      implicit none
      double precision, intent(in) :: tsc
      scaltolg = tsc
   end subroutine initTolScaling

!!********************************************************************
!!
!!     Subroutine initDisplay
!!
!>     \ingroup commonInitialization
!!     \brief   Initialize defs::displayg and defs::dispiterg parameters.
!!     \param[in]  displ    defs::displayg value
!!     \param[in]  disp     defs::dispiterg = 1 if disp = 'on' and 0 if 'off'
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    subroutine initDisplay(displ,disp)
      implicit none
      integer, intent(in)       :: displ
      character(32), intent(in) :: disp

      select case (trim(disp))
        case ('on')
            displayg = 1
        case ('off')
            displayg = 0
        case default
            CALL printandstop('invalid options.Display: on or off ?')
      end select
      IF (displ.LT.0) THEN
        CALL printandstop('options.DispIter must be positive.')
      ENDIF
      dispiterg = displ

   end subroutine initDisplay

!!********************************************************************
!!
!!     Subroutine initStopAtTurningPoint
!!
!>     \ingroup commonInitialization
!!     \brief   Initialize defs::stopatturningpointg parameter.
!!     \param[in]  satp    defs::stopatturningpointg = true if sapa = 'on' and false if 'off'
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    subroutine initStopAtTurningPt(satp)
      implicit none
      integer, intent(in) :: satp

      select case (satp)
        case (1)
            stopAtTurningPtg = .true.
        case (0)
            stopAtTurningPtg = .false.
        case default
            CALL printandstop('invalid options.StopAtTurningPoint: 0 or 1 ?')
      end select

   end subroutine initStopAtTurningPt


!!********************************************************************
!!
!!     Subroutine initSavePath
!!
!>     \ingroup commonInitialization
!!     \brief   Initialize defs::dosavepathinfileg arameter.
!!     \param[in]  sapa    defs::dosavepathinFileg = 1 if sapa = 'on' and 0 if 'off'
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    subroutine initSavePath(sapa)
      implicit none
      character(32), intent(in) :: sapa

      select case (trim(sapa))
        case ('on')
            doSavePathInFileg = 1
        case ('off')
            doSavePathInFileg = 0
        case default
            CALL printandstop('invalid options.DoSavePath = ' // trim(sapa) // ': on or off ?')
      end select

   end subroutine initSavePath

!!********************************************************************
!!
!!     Subroutine initDisplaySolve
!!
!>     \ingroup commonInitialization
!!     \brief   Initialize defs::displayg parameter.
!!        \param[in]    disp    defs::displayg = 1 if disp = 'on' and 0 if 'off'
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   subroutine initDisplaySolve(disp)
      implicit none
     character(32), intent(in) :: disp

     select case (trim(disp))
       case ('on')
            displayg = 1
       case ('off')
            displayg = 0
       case default
            CALL printandstop('invalid options.Display = ' // trim(disp) // ': on or off ?')
     end select

   end subroutine initDisplaySolve

!!********************************************************************
!!
!!     Subroutine initTspanTi
!!
!>     \ingroup commonInitialization
!!     \brief   Initialize defs::grilleg parameter.
!!
!!     Detailed description :
!!     We check first if ti is increasing only if there is more than one arc,
!!     then we check if tspan is contained in ti and sorted.
!!     Finbally we add each instant from tspan in the list grilleg for the 
!!     dense output during the call to exphvfun() or expdhvfun().
!!
!!     \param[in] n       dimension of the grid
!!     \param[in] tspan   the grille to transform into a list
!!     \param[in] nbarc   Number of arcs
!!     \param[in] ti      ti = [t0 t1 .. t_nbarc-1 tf], where t0 is the initial time and tf the final time.
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   subroutine initTspanTi(n,tspan,nbarc,ti)
      implicit none
      integer, intent(in)                      :: n
      integer, intent(in)                      :: nbarc
      double precision, intent(in), dimension(n)   :: tspan
      double precision, intent(in), dimension(nbarc+1) :: ti

      !local variables
      integer                    :: i
      double precision, dimension(1) :: val

      !ti must be increasing if there is more than one arc
      if(nbarc.gt.1)then
          i=2
          do while(i.le.nbarc+1)
            if((ti(i)-ti(i-1)).lt.0d0)then
               CALL printandstop('ti must be increasing')
            end if
            i = i + 1
          end do
      end if

      !tspan must be contained in ti and tspan must be sorted
      i=2
      if((ti(nbarc+1)-tspan(1))*(tspan(1)-ti(1)).lt.0d0)then
        CALL printandstop('tspan must be contained in ti')
      end if
      do while(i.lt.n)
        if((tspan(i)-tspan(i-1))*(tspan(i+1)-tspan(i)).lt.0d0)then
           CALL printandstop('tspan must be sorted')
        end if
        if((ti(nbarc+1)-tspan(i))*(tspan(i)-ti(1)).lt.0d0)then
           CALL printandstop('tspan must be contained in ti')
        end if
        i = i + 1
      end do
      if((ti(nbarc+1)-tspan(n))*(tspan(n)-ti(1)).lt.0d0)then
        CALL printandstop('tspan must be contained in ti')
      end if

      !we add each instant from tspan in the list grilleg
      val(1)=0
      call CREER_STRUCTURE(grilleg,1.0*dble(n-3))
      do i=1,n
        call INSERER(tspan(n-i+1),1,val,grilleg)
      end do

   end subroutine initTspanTi


!!********************************************************************
!!
!!     Subroutine initPar
!!
!>     \ingroup commonInitialization
!!     \brief   Initialize defs::par0g and defs::parfg parameters.
!!
!!     Detailed description :
!!     <ul>
!!     <li> We allocate the global parameters which contain
!!          the initial and final homotopic values.</li>
!!     <li> We count how many homotopic parameters we have.
!!          The number corresponds to the total of distinct
!!          initial and final parameters.</li>
!!     <li> We save the indexes of homotopic parameters.
!!          We use two "for" because we need to allocate 
!!          the vector containing the indexes with the number
!!          of homotopic parameters.</li>
!!     <li> Of course we do not free the memory allocated here.</li>
!!     </ul>
!!
!!     \param[in]    lpar    dimension of par1 and par2
!!     \param[in]    par1    initial parameters
!!     \param[in]    par2    final parameters
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   subroutine initPar(lpar,par1,par2)
      implicit none
      integer     , intent(in)                  :: lpar
      double precision, intent(in), dimension(lpar) :: par1,par2

      integer :: i,k

      !we allocate the global parameters which contain
       !the initial and final homotopic values
      allocate(par0g(lpar),parfg(lpar))

      lparg = lpar
      par0g = par1
      parfg = par2

      !we count how many homotopic parameters we have
       !the number corresponds to the total of distinct
       !initial and final parameters
      k = 0
      do i=1,lpar
         if(abs(par0g(i)-parfg(i)).ne.0d0)then
            k = k+1
         end if
      end do

      !we save the indexes of homotopic parameters
       !we use two "for" because we need to allocate 
       !the vector containing the indexes with the number
       !got before
      nparhomg = k
      allocate(iparhomg(nparhomg))
      k = 0
      do i=1,lpar
         if(abs(par0g(i)-parfg(i)).ne.0d0)then
            k = k+1
            iparhomg(k) = i
         end if
      end do
   end subroutine initPar

!!********************************************************************
!!
!!     Subroutine closePar
!!
!>     \ingroup commonInitialization
!!     \brief   Free the memory allocated in initPar().
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   subroutine closePar()
      deallocate(par0g,parfg,iparhomg)
   end subroutine closePar

!!********************************************************************
!!
!!     Subroutine initDerivative
!!
!>     \ingroup commonInitialization
!!     \brief   Initialize defs::derivativeg parameter.
!!     \param[in]    der     defs::derivativeg value
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   subroutine initDerivative(der)
      implicit none
      character(32), intent(in) :: der
      derivativeg = trim(der)
   end subroutine initDerivative


!!********************************************************************
!!
!!     Subroutine initIntegSfun
!!
!>     \ingroup commonInitialization
!!     \brief   Initialize defs::integeqvarg parameter.
!!        \param[in]    integ    defs::integeqvarg value
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   subroutine initIntegSfun(integ)
      implicit none
      character(32), intent(in) :: integ
      integsfung = trim(integ)
   end subroutine initIntegSfun

!!********************************************************************
!!
!!     Subroutine initIntegEqvar
!!
!>     \ingroup commonInitialization
!!     \brief   Initialize defs::integeqvarg parameter.
!!        \param[in]    integ    defs::integeqvarg value
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   subroutine initIntegEqvar(integ)
      implicit none
      character(32), intent(in) :: integ
      integeqvarg = trim(integ)
      integsfung = trim(integ)
   end subroutine initIntegEqvar

!!********************************************************************
!!
!!     Subroutine initIntegExpdhvfun
!!
!>     \ingroup commonInitialization
!!     \brief   Initialize defs::integexpdhvfung parameter.
!!        \param[in]    integ     defs::integexpdhvfung value
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   subroutine initIntegExpdhvfun(integ)
      implicit none
      character(32), intent(in) :: integ
      integexpdhvfung = trim(integ)
   end subroutine initIntegExpdhvfun

!!********************************************************************
!!
!!     Subroutine initIntegExphvfun
!!
!>     \ingroup commonInitialization
!!     \brief   Initialize defs::integexphvfung parameter.
!!        \param[in]    integ    defs::integexphvfung value
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   subroutine initIntegExphvfun(integ)
      implicit none
      character(32), intent(in) :: integ
      integexphvfung = trim(integ)
   end subroutine initIntegExphvfun

!!********************************************************************
!!
!!     Subroutine initIntegHam
!!
!>     \ingroup commonInitialization
!!     \brief   Initialize defs::integhamg parameter.
!!        \param[in]    integ    defs::integhamg value
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   subroutine initIntegHam(integ)
      implicit none
      character(32), intent(in) :: integ
       select case (trim(integ))
            case ('radau5')
                CALL printandstop('ERROR: Integrate >>> can not use radau5 for hampath')
       end select
      integhamg = trim(integ)
   end subroutine initIntegHam

!!********************************************************************
!!
!!     Subroutine initMaxSteps
!!
!>     \ingroup commonInitialization
!!     \brief   Initialize defs::maxstepsg parameter.
!!        \param[in]    ms      defs::maxstepsg value
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   subroutine initMaxSteps(ms)
      implicit none
      integer, intent(in) :: ms
      maxstepsg = ms
   end subroutine initMaxSteps

!!********************************************************************
!!
!!     Subroutine initMaxStepsHam
!!
!>     \ingroup commonInitialization
!!     \brief   Initialize defs::maxstepscontg parameter.
!!        \param[in]    msh      defs::maxstepscontg value
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   subroutine initMaxStepsHam(msh)
      implicit none
      integer, intent(in) :: msh
      maxstepscontg = msh
   end subroutine initMaxStepsHam

!!********************************************************************
!!
!!     Subroutine initMaxStepsSize
!!
!>     \ingroup commonInitialization
!!     \brief   Initialize defs::hmaxg parameter.
!!     \param[in]    mss      defs::hmaxg value
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   subroutine initMaxStepSize(mss)
      implicit none
      double precision, intent(in) :: mss
      hmaxg = mss
   end subroutine initMaxStepSize

!!********************************************************************
!!
!!     Subroutine initMaxStepsSizeHam
!!
!>     \ingroup commonInitialization
!!     \brief   Initialize defs::hmaxcontg parameter.
!!     \param[in]    mssh      defs::hmaxcontg value
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   subroutine initMaxStepSizeHam(mssh)
      implicit none
      double precision, intent(in) :: mssh
      hmaxcontg = mssh
   end subroutine initMaxStepSizeHam

!!********************************************************************
!!
!!     Subroutine initTol
!!
!>     \ingroup commonInitialization
!!     \brief   Initialize defs::atolg and defs::rtolg parameters.
!!        \param[in]    atole    defs::atolg value
!!        \param[in]    rtole    defs::rtolg value
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   subroutine initTol(atole,rtole)
      implicit none
       double precision, intent(in) :: atole
      double precision, intent(in) :: rtole
       atolg = atole
      rtolg = rtole
   end subroutine initTol

!!********************************************************************
!!
!!     Subroutine initTolCont
!!
!>     \ingroup commonInitialization
!!     \brief   Initialize defs::atolcontg, defs::rtolcontg, defs::densetolg and defs::densetolrelg parameters.
!!        \param[in]    atolecont   atolcontg value
!!        \param[in]    rtolecont   rtolcontg value
!!        \param[in]    densetole   densetolg value
!!        \param[in]    dtr         densetolrelg value
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   subroutine initTolCont(atolecont,rtolecont,densetole,dtr)
        implicit none
        double precision, intent(in) :: atolecont
        double precision, intent(in) :: rtolecont
        double precision, intent(in) :: densetole
        double precision, intent(in) :: dtr
        densetolrelg = dtr
        atolcontg = atolecont
        rtolcontg = rtolecont
        densetolg = densetole
   end subroutine initTolCont

!!********************************************************************
!!
!!     Subroutine initTolSolve
!!
!>     \ingroup commonInitialization
!!     \brief   Initialize defs::xtolg parameter.
!!       \param[in]    xtole   defs::xtolg value
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   subroutine initTolSolve(xtole)
      implicit none
       double precision, intent(in) :: xtole
       xtolg = xtole
   end subroutine initTolSolve

!!********************************************************************
!!
!!     Subroutine initNormeSfunMax
!!
!>     \ingroup commonInitialization
!!     \brief   Initialize defs::normesfunmaxg parameter.
!!       \param[in]    nsm   normesfunmaxg value
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
   subroutine initNormeSfunMax(nsm)
      implicit none
       double precision, intent(in) :: nsm
       NormeSfunMaxg = nsm
   end subroutine initNormeSfunMax

end module init

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!

