!!********************************************************************
!!
!!     Subroutine integration
!!
!>     \ingroup commonIntegration
!!     \brief   performs the integration chosen
!!     \param[in]       fun         RHS
!!     \param[in]       neq         dimension of the state
!!     \param[in]       y0          initial point
!!     \param[in]       funpar      parameters given to fun during integration
!!     \param[in]       lfunpar     dimension of funpar
!!     \param[in]       ncpas       number of components of y for which we make a step control
!!     \param[in,out]   tin         initial time
!!     \param[in,out]   tout        final time
!!     \param[out]      yf          final point
!!     \param[out]      flag        information on the integration process, should be 1
!!     \param[in]       choixsolout id giving the method which has called dopri5int, see \ref globalVariables
!!     \param[in]       solout      auxiliary function called after accepted steps
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
Subroutine integration(fun,neq,y0,funpar,lfunpar,ncpas,tin,tout,yf,flag,choixsolout,solout)
    use defs
    use adds
    use utils
    implicit none
    integer,            intent(in)                      :: lfunpar
    integer,            intent(in)                      :: choixsolout
    integer,            intent(in)                      :: neq
    integer,            intent(in)                      :: ncpas
    double precision,   intent(inout)                   :: tin
    double precision,   intent(inout)                   :: tout
    double precision,   intent(in),  dimension(neq)     :: y0
    double precision,   intent(in),  dimension(lfunpar) :: funpar
    integer,            intent(out)                     :: flag
    double precision,   intent(out), dimension(neq)     :: yf

    interface
    Subroutine solout(Niter,told,time,y,neq,dpar,ldpar,ipar,lipar,fun,funpar,lfunpar,irtrn)
        integer,            intent(in)                         :: Niter
        double precision,   intent(in)                         :: told
        double precision,   intent(inout)                      :: time
        double precision,   intent(inout), dimension(neq)      :: y
        integer,            intent(in)                         :: neq
        double precision,   intent(in),    dimension(ldpar)    :: dpar
        integer,            intent(in)                         :: ldpar
        integer,            intent(in),    dimension(lipar)    :: ipar
        integer,            intent(in)                         :: lipar
        double precision,   intent(inout), dimension(lfunpar)  :: funpar
        integer,            intent(in)                         :: lfunpar
        integer,            intent(inout)                      :: irtrn
    end Subroutine solout
    end interface

    interface
    Subroutine fun(neq,time,y,value,funpar,lfunpar)
        implicit none
        integer,           intent(in)                      :: neq
        integer,           intent(in)                      :: lfunpar
        double precision,  intent(in)                      :: time
        double precision,  intent(in),  dimension(lfunpar) :: funpar
        double precision,  intent(in),  dimension(neq)     :: y
        double precision,  intent(out), dimension(neq)     :: value
    end Subroutine fun
    end interface

    !local variables
    character(32) :: integmode
    character(32) :: methodtitle
    character(32) :: integmodetitle
    CHARACTER(len=120) :: LINE

    select case (choixsolout)
        !SFUN
        case (idSFunG)
        integmode = integsfunG
        integmodetitle= 'SFUN'

        !SJAC
        case (idSJacG)
        integmode = integeqvarG
        integmodetitle= 'VAREQ'

        !EXPHVFUN
        case (idExpG)
        integmode = integexphvfunG
        integmodetitle= 'Exphvfun'

        !EXPDHVFUN
        case (idExpdG)
        integmode = integexpdhvfunG
        integmodetitle= 'Expdhvfun'

        !CONTINUATION
        case (idHamG)
        integmode = integhamG
        integmodetitle= 'Continuation'
    end select

    methodtitle = integmode
    select case (integmode)

        case ('dopri5')

            !!Dormand Prince 5th order (dopri5)
            call dopri5Int(fun,neq,y0,funpar,lfunpar,ncpas,tin,tout,yf,flag,choixsolout,solout)

        case default

            CALL printandstop('  ||| ERROR: Integrate -> Unknown integrator' // integmode)

    end select

    if(displayG.eq.1) then

        if(flag.lt.0)then
            WRITE (LINE,'(a)') '';                  call myprint(LINE,.true.)
            WRITE (LINE,'(a)') '  ||| WARNING: ';   call myprint(LINE,.false.)
            WRITE (LINE,'(a)') integmodetitle;      call myprint(LINE,.false.)
            WRITE (LINE,'(a)') ' Integration (';    call myprint(LINE,.false.)
            WRITE (LINE,'(a)') methodtitle;         call myprint(LINE,.false.)
        end if

        select case (flag)
            case (-1)
                WRITE (LINE,'(a)') ')  ->  INPUT IS NOT CONSISTENT '
            case (-2)
                WRITE (LINE,'(a)') ')  ->  LARGER MAXSTEPS IS NEEDED '
            case (-3)
                WRITE (LINE,'(a)') ')  ->  STEP SIZE BECOMES TOO SMALL '
            case (-4)
                WRITE (LINE,'(a)') ')  ->  PROBLEM IS PROBABLY STIFF (INTERRUPTED) '
            case (-5)
                WRITE (LINE,'(a)') ')  ->  |S| BECOME TOO BIG '
            case (-6)
                WRITE (LINE,'(a)') ')  ->  HAMPATH STEP SIZE BECOMES TO SMALL '
            case (-7)
                WRITE (LINE,'(a)') ')  ->  HAMPATH STOPS AFTER A TURNING POINT '
        end select

        if(flag.lt.0)then
            call myprint(LINE,.true.)
        end if

    end if

end Subroutine integration

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!
