module utils
    use defs
    implicit none
    contains

!!********************************************************************
!>     \ingroup commonUtils
!!     \brief   Write an integer in a file, well formatted for matlab.
!!       \param[in] inte          data to print
!!       \param[in] unitfileout   unit file for writing
!!       \param[in] nameint       the name of the variable corresponding to inte
!!
!!  \author Olivier Cots
!!  \date   2009-2014
!!  \copyright Eclipse Public License
    subroutine writeIntegerMatlab(inte,namevec,unitfileout)
        implicit none
        integer         , intent(in) :: inte,unitfileout
        character(len=*), intent(in) :: namevec
       
        !local variables 
        CHARACTER(len=120)  :: LINE
        
        WRITE (LINE,*) trim(namevec),' = '
        WRITE (unit=unitfileout,fmt='(a)'   ,advance='no') trim(LINE)
        WRITE (unit=unitfileout,fmt='(i0.1)',advance='no') inte       
        WRITE (unit=unitfileout,fmt='(a)'   ,advance='yes') ';'

    end subroutine writeIntegerMatlab

!!********************************************************************
!>     \ingroup commonUtils
!!     \brief   Write a row vector in a file, well formatted for matlab.
!!       \param[in] vec           datas to print
!!       \param[in] n             number of elements of vec
!!       \param[in] unitfileout   unit file for writing
!!       \param[in] namevec       the name of the variable corresponding to vec
!!
!!  \author Olivier Cots
!!  \date   2009-2014
!!  \copyright Eclipse Public License
    subroutine writeRowVectorDbleMatlab(vec,n,namevec,unitfileout)
        implicit none
        double precision, dimension(n), intent(in) :: vec
        integer         , intent(in) :: n,unitfileout
        character(len=*), intent(in) :: namevec
       
        !local variables 
        CHARACTER(len=120)  :: LINE
        integer :: i
        
        WRITE (LINE,*) trim(namevec),' = ['
        WRITE (unit=unitfileout,fmt='(a)',advance='no') trim(LINE)
        do i=1,n
            WRITE (unit=unitfileout,fmt='(e23.15)',advance='no') vec(i)
        end do        
        WRITE (unit=unitfileout,fmt='(a)',advance='yes') '];'

    end subroutine writeRowVectorDbleMatlab

!!********************************************************************
!>     \ingroup commonUtils
!!     \brief   Write a column vector in a file, well formatted for matlab.
!!       \param[in] vec           datas to print
!!       \param[in] n             number of elements of vec
!!       \param[in] unitfileout   unit file for writing
!!       \param[in] namevec       the name of the variable corresponding to vec
!!
!!  \author Olivier Cots
!!  \date   2009-2014
!!  \copyright Eclipse Public License
    subroutine writeColVectorDbleMatlab(vec,n,namevec,unitfileout)
        implicit none
        double precision, dimension(n), intent(in) :: vec
        integer         , intent(in) :: n,unitfileout
        character(len=*), intent(in) :: namevec
       
        !local variables 
        CHARACTER(len=120)  :: LINE
        integer :: i
        
        WRITE (LINE,*) trim(namevec),' = ['
        WRITE (unit=unitfileout,fmt='(a)',advance='no') trim(LINE)
        do i=1,n
            if(i.eq.n)THEN
                WRITE (unit=unitfileout,fmt='(e23.15)',advance='no') vec(i)
            ELSE
                WRITE (unit=unitfileout,fmt='(e23.15)',advance='yes') vec(i)
            end if
        end do        
        WRITE (unit=unitfileout,fmt='(a)',advance='yes') '];'

    end subroutine writeColVectorDbleMatlab

!!********************************************************************
!>     \ingroup commonUtils
!!     \brief   Write a matrix in a file, well formatted for matlab.
!!       \param[in] mat           datas to print
!!       \param[in] m             number of rows of mat
!!       \param[in] n             number of column of mat
!!       \param[in] unitfileout   unit file for writing
!!       \param[in] namemat       the name of the variable corresponding to mat
!!
!!  \author Olivier Cots
!!  \date   2009-2014
!!  \copyright Eclipse Public License
    subroutine writeMatrixDbleMatlab(mat,m,n,namemat,unitfileout)
        implicit none
        double precision, dimension(m,n), intent(in) :: mat
        integer         , intent(in) :: m,n,unitfileout
        character(len=*), intent(in) :: namemat
       
        !local variables 
        CHARACTER(len=120)  :: LINE
        integer :: i,j
        
        WRITE (LINE,*) trim(namemat),' = ['
        WRITE (unit=unitfileout,fmt='(a)',advance='no') trim(LINE)
        do i=1,m
            do j=1,n
                if(j.eq.n)THEN
                    if(i.eq.m)THEN
                        WRITE (unit=unitfileout,fmt='(e23.15)',advance='no') mat(i,j)
                    ELSE
                        WRITE (unit=unitfileout,fmt='(e23.15)',advance='yes') mat(i,j)
                    end IF
                ELSE
                    WRITE (unit=unitfileout,fmt='(e23.15)',advance='no') mat(i,j)
                end if
            end do
        end do        
        WRITE (unit=unitfileout,fmt='(a)',advance='yes') '];'

    end subroutine writeMatrixDbleMatlab

!!********************************************************************
!>     \ingroup commonUtils
!!     \brief   Print a message in fortran, matlab or other interface.
!!       \param[in] message     Message to print
!!       \param[in] doprintOpt  In matlab case, drawnow is executed if doprintOpt is true
!!                              In fortran case, advance = no if false and yes if true
!!                              It is an optional parameter which has a true default value
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    subroutine myprint(message,doprintOpt)
        implicit none
        character (LEN=*), intent(in)   :: message
        logical, intent(in), optional   :: doprintOpt

        !local variables
        logical :: doprint

        IF (PRESENT(doprintOpt)) THEN
            doprint = doprintOpt
        ELSE
            doprint = .true.
        END IF

#if defined(InterfaceFortran)
        IF (doprint) THEN
            write(*,'(a)',advance='yes') trim(message)
        ELSE
            write(*,'(a)',advance='no') trim(message)
        END IF
#endif

#if defined(InterfaceMatlab)
        call mexPrintf(trim(message))
        if(doprint)then
            call mexPrintf('\n')
            call mexEvalString("drawnow;")
        end if
#endif

    end subroutine myprint

!!********************************************************************
!>     \ingroup commonUtils
!!     \brief the index of current arc
!!     \param[in] t      time
!!     \param[in] nbarc  Number of arcs
!!     \param[in] ti     ti = [t0 t1 .. t_{nbarc-1} tf], where t0 is the initial time and tf the final time.
!!
!!     \param[out] arc   index of arc corresponding to time t
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    subroutine getArc(t,nbarc,ti,arc)
        implicit none
        integer, intent(in)                                 :: nbarc
        double precision, intent(in)                        :: t
        double precision, intent(in),  dimension(nbarc+1)   :: ti
        integer, intent(out)                                :: arc

        !local variables
        logical :: fini

        fini = .false.
        arc  = 1
        do while(.not.fini .and. arc.lt.nbarc+1)
            if((ti(arc+1)-t)*(t-ti(arc)).ge.0d0)then
                !if(t.eq.ti(arc+1).and.arc.lt.nbarc)then
                !    arc = arc + 1
                !end if
                fini = .true.
            else
                arc = arc + 1
            end if
        end do

    end subroutine getArc

!!********************************************************************
!>     \ingroup commonUtils
!!     \brief Computes deb, fin and inc for exphvfun and expdhvfun
!!
!!     \param[in] t0     Initial time
!!     \param[in] tf     Final time
!!     \param[in] nbarc  Number of arcs
!!     \param[in] ti     ti = [t0 t1 .. t_{nbarc-1} tf], where t0 is the initial time and tf the final time.
!!
!!     \param[out] deb    first arc
!!     \param[out] fin    final arc
!!     \param[out] inc    incremental direction
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    subroutine getArea(t0,tf,nbarc,ti,deb,fin,inc)
        implicit none
        double precision,   intent(in)                      :: t0,tf
        integer,            intent(in)                      :: nbarc
        double precision,   intent(in), dimension(nbarc+1)  :: ti
        integer,            intent(out)                     :: deb,fin,inc

        !local variables
        integer :: j

        call getArc(t0,nbarc,ti,deb) !get the index of first arc
        call getArc(tf,nbarc,ti,fin) !get the index of last arc

        if(tf-t0.ge.0d0)then !if tf is greater than t0 then inc is positive
            inc = 1
            !we check if t0 is equal to ti(deb+1)
            !if it is the case, then the initial arc is one after
            if(deb.le.nbarc)then
                if(t0.eq.ti(deb+1))then
                    deb = deb + 1
                    if(abs(tf-t0).eq.0d0)then
                        fin = fin + 1
                    end if
                end if
            end if
        else                !else inc is negative
            inc = -1
            j = fin
            !we check if tf is equal to ti(fin)
            !if it is the case, then the final arc is one before
            if(fin.le.nbarc)then
                if(tf.eq.ti(fin+1))then
                    fin = fin + 1
                end if
            end if
        end if

    end subroutine getArea

!!********************************************************************
!>     \ingroup commonUtils
!!     \brief the tspan for exphvfun and expdhvfun
!!     \param[in] t0     Initial time
!!     \param[in] tf     Final time
!!     \param[in] nbarc      Number of arcs
!!     \param[in] ti     ti = [t0 t1 .. t_{nbarc-1} tf], where t0 is the initial time and tf the final time.
!!     \param[in] deb    first arc
!!     \param[in] fin    final arc
!!     \param[in] inc    incrementation direction
!!
!!     \param[out] tspan the integration grid which contains only values of ti
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    subroutine getTspan(t0,tf,nbarc,ti,deb,fin,inc,tspan)
        implicit none
        double precision, intent(in)                                :: t0,tf
        integer,          intent(in)                                :: deb,fin,inc,nbarc
        double precision, intent(in) , dimension(nbarc+1)           :: ti
        double precision, intent(out), dimension(inc*(fin-deb)+2)   :: tspan

        !local variables
        integer :: i,j

        !we add to tspan the values of ti contained in [t0,tf]
        tspan(1) = t0
        j = 2
        do i=deb+(inc+1)/2,fin+(-inc+1)/2,inc
            tspan(j)   = ti(i)
            j = j + 1
        end do
        tspan(inc*(fin-deb)+2) = tf

    end subroutine getTspan

!!********************************************************************
!>     \ingroup commonUtils
!!     \brief   Perform scaling, id defs::doscalingg is true, according to k and scaltol.
!!     \param[in]   nz          dimension of z
!!     \param[in]   z           input datas to be scaled
!!     \param[out]  ztilde      ouptut datas after scaling
!!     \param[out]  scaletab    scale
!!     \param[in]   k           the datas are scaled between 0 and k
!!     \param[in]   scaltol     the datas are scaled if they are greater than scaltol
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    subroutine scaling(nz,z,ztilde,scaletab,k,scaltol)
        integer,          intent(in)                 :: nz,k
        double precision, intent(in)                 :: scaltol
        double precision, intent(in), dimension(nz)  :: z
        double precision, intent(out), dimension(nz) :: ztilde, scaletab

        !local variables
        integer :: i,l
        CHARACTER(len=120) :: LINE

        if(doScalingG)then
            do i=1,nz
                if(abs(z(i)).gt.scaltol)then
                    l = floor(log10(abs(z(i)))) + k
                    if(abs(z(i)).lt.1)then
                        l = l - 1
                    end if
                else
                    l = 0
                end if
                scaletab(i) = 10d0**(-l)
                ztilde(i) = z(i) * scaletab(i)
            end do
        else
            scaletab = 1d0
            ztilde  = z
        end if

!        scaletab = 1d0
!        ztilde  = z

    end subroutine scaling

!!********************************************************************
!>     \ingroup commonUtils
!!     \brief   Do scaling according to scaletab.
!!       \param[in] ny          dimension of y and yscaled
!!       \param[in] y           input datas to be scaled
!!       \param[in] scaletab    scale used as reference
!!       \param[out] yscaled    ouptut datas after scaling
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    subroutine scalemul(ny,y,scaletab,yscaled)
        implicit none
        integer,          intent(in)                    :: ny
        double precision, intent(in),   dimension(ny)   :: y, scaletab
        double precision, intent(out),  dimension(ny)   :: yscaled

        !local variables
        integer :: i

!        yscaled = y
        do i=1,ny
            yscaled(i) = y(i) * scaletab(i)
        end do

    end subroutine scalemul

!!********************************************************************
!>     \ingroup commonUtils
!!     \brief   Undo scaling according to scaletab.
!!       \param[in] ny          dimension of y and yscaled
!!       \param[in] yscaled     input datas to be unscaled
!!       \param[in] scaletab    scale used as reference
!!       \param[out] y          output datas after unscaling
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    subroutine scalediv(ny,yscaled,scaletab,y)
        implicit none
        integer,          intent(in)                    :: ny
        double precision, intent(in),   dimension(ny)   :: yscaled, scaletab
        double precision, intent(out),  dimension(ny)   :: y

        !local variables
        integer :: i

!        y = yscaled
        do i=1,ny
            y(i) = yscaled(i) / scaletab(i)
        end do

    end subroutine scalediv

!!********************************************************************
!>     \ingroup commonUtils
!!     \brief make parameters vector wrt lambda
!!       \param[in] lpar   Parameters vector dimension
!!       \param[in] lambda Homotopic parameter
!!
!!       \param[out] par   Optional parameters at lambda
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    subroutine getPar(lambda,lpar,par)
      implicit none
      integer,          intent(in)                    :: lpar
      double precision, intent(in)                    :: lambda
      double precision, intent(out), dimension(lpar)  :: par

      if(nparhomG.eq.1)then !the homotopic parameter is equals to lambda if there is only one 
          par = par0G
          par(iparhomG(1)) = lambda
      else !else it is a relation between par0G and parfG computed by parfun
          call parfun(lambda,lpar,par0G,parfG,par)
      end if

    end subroutine getPar

!!********************************************************************
!>     \ingroup commonUtils
!!     \brief make the unknown x given to ssolve
!!       \param[in] nbarc      Number of arcs
!!       \param[in] n      State dimension
!!       \param[in] ti     ti = [t0 t1 .. t_{nbarc-1} tf], where t0 is the initial time and tf the final time.
!!       \param[in] zi     State and adjoint states at [t0 t1 .. t_{nbarc-1}].
!!       \param[in] ne     Dimension of the shooting function
!!       \param[in] ntf    If 1, then the final time is free and tf is an independent variable
!!
!!        \param[out] x    Unknown for ssolve
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    subroutine dox(nbarc,n,ti,zi,ne,ntf,x)
      implicit none
      integer,          intent(in)                          :: nbarc,n,ne,ntf
      double precision, intent(in),  dimension(nbarc+1)     :: ti
      double precision, intent(in),  dimension(2*n,nbarc)   :: zi
      double precision, intent(out), dimension(ne)          :: x

      !local variables
      integer :: i

      x(1:2*n) = zi(:,1) !z0
      do i=2,nbarc
        x((i-1)*2*n+1:i*2*n) = zi(:,i) !zi
        x(nbarc*2*n+i-1)     = ti(i)   !ti
      end do
      if(ntf.eq.1)then
        x(nbarc*2*n+nbarc-1+ntf)    = ti(nbarc+1) !tf
      end if

      !x = (z0,z1,..,z_{nbarc-1},t1,..,t_{nbarc-1},(tf))

    end subroutine dox

!!********************************************************************
!>     \ingroup commonUtils
!!     \brief unmake the unknown x given to ssolve
!!       \param[in] t0      initial time
!!       \param[in] tf      final time
!!       \param[in] nbarc      Number of arcs
!!       \param[in] n      State dimension
!!       \param[in] x    Unknown for ssolve
!!       \param[in] ne     Dimension of the shooting function
!!       \param[in] ntf    If 1, then the final time is free and tf is an independent variable
!!
!!       \param[out] ti     ti = [t0 t1 .. t_{nbarc-1} tf], where t0 is the initial time and tf the final time.
!!       \param[out] zi     State and adjoint states at [t0 t1 .. t_{nbarc-1}].
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    subroutine undox(t0,tf,nbarc,n,ti,zi,ne,ntf,x)
      implicit none
      integer,          intent(in)                          :: nbarc,n,ne,ntf
      double precision, intent(in)                          :: t0,tf
      double precision, intent(out),  dimension(nbarc+1)    :: ti
      double precision, intent(out),  dimension(2*n,nbarc)  :: zi
      double precision, intent(in) , dimension(ne)          :: x

      !local variables
      integer :: i

      !x = (z0,z1,..,z_{nbarc-1},t1,..,t_{nbarc-1},(tf))

      ti(1)   = t0       !t0 is not in x
      zi(:,1) = x(1:2*n) !z0
      do i=2,nbarc
         zi(:,i) = x((i-1)*2*n+1:i*2*n) !zi
         ti(i)   = x(nbarc*2*n+i-1)     !ti
      end do
      if(ntf.eq.1)then
         ti(nbarc+1) = x(nbarc*2*n+nbarc-1+ntf) !tf if it is free
      else
         ti(nbarc+1) = tf !tf if it is fixed
      end if

    end subroutine undox

!!********************************************************************
!>     \ingroup commonUtils
!!     \brief make the vector c given to hampath
!!       \param[in] nbarc      Number of arcs
!!       \param[in] n      State dimension
!!       \param[in] ti     ti = [t0 t1 .. t_{nbarc-1} tf], where t0 is the initial time and tf the final time.
!!       \param[in] zi     State and adjoint states at [t0 t1 .. t_{nbarc-1}].
!!       \param[in] lambda Current homotopic parameter
!!       \param[in] ne     Dimension of the shooting function
!!       \param[in] ntf    If 1, then the final time is free and tf is an independent variable
!!
!!       \param[out] c    Vector of zeros path for hampath
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    subroutine doc(nbarc,n,ti,zi,lambda,ne,ntf,c)
      implicit none
      integer,          intent(in)                          :: nbarc,n,ne,ntf
      double precision, intent(in)                          :: lambda
      double precision, intent(in),  dimension(nbarc+1)     :: ti
      double precision, intent(in),  dimension(2*n,nbarc)   :: zi
      double precision, intent(out), dimension(ne+1)        :: c

      !local variables
      integer :: i

      c(1:2*n) = zi(:,1) !z0
      do i=2,nbarc
        c((i-1)*2*n+1:i*2*n) = zi(:,i) !zi
        c(nbarc*2*n+i-1)     = ti(i)   !ti
      end do
      if(ntf.eq.1)then
        c(nbarc*2*n+nbarc-1+ntf)    = ti(nbarc+1) !tf
      end if
      c(nbarc*2*n+nbarc-1+ntf+1)    = lambda !lambda

      ! c = (z0,z1,..,z_{nbarc-1},t1,..,t_{nbarc-1},(tf),lambda)

    end subroutine doc

!!********************************************************************
!>     \ingroup commonUtils
!!     \brief unmake the unknown c given to hampath
!!       \param[in] t0      initial time
!!       \param[in] tf      final time
!!       \param[in] nbarc      Number of arcs
!!       \param[in] n      State dimension
!!       \param[in] c    Unknown for hampath
!!       \param[in] ne     Dimension of the shooting function
!!       \param[in] ntf    If 1, then the final time is free and tf is an independent variable
!!
!!       \param[out] ti     ti = [t0 t1 .. t_{nbarc-1} tf], where t0 is the initial time and tf the final time.
!!       \param[out] zi     State and adjoint states at [t0 t1 .. t_{nbarc-1}].
!!       \param[out] lambda Homotopic parameter
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    subroutine undoc(t0,tf,nbarc,n,ti,zi,lambda,ne,ntf,c)
      implicit none
      integer,          intent(in)                          :: nbarc,n,ne,ntf
      double precision, intent(in)                          :: t0,tf
      double precision, intent(out),  dimension(nbarc+1)    :: ti
      double precision, intent(out),  dimension(2*n,nbarc)  :: zi
      double precision, intent(out)                         :: lambda
      double precision, intent(in) , dimension(ne+1)        :: c

      !local variables
      integer :: i

      ! c = (z0,z1,..,z_{nbarc-1},t1,..,t_{nbarc-1},(tf),lambda)

      ti(1)   = t0       !t0 is not in c
      zi(:,1) = c(1:2*n) !z0
      do i=2,nbarc
         zi(:,i) = c((i-1)*2*n+1:i*2*n) !zi
         ti(i)   = c(nbarc*2*n+i-1)     !ti
      end do
      if(ntf.eq.1)then
         ti(nbarc+1) = c(nbarc*2*n+nbarc-1+ntf) !tf if it is free
      else
         ti(nbarc+1) = tf !tf if it is fixed
      end if
      lambda = c(nbarc*2*n+nbarc-1+ntf+1) !lambda

    end subroutine undoc

end module utils

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!


