!!********************************************************************
!!
!!     Subroutine dhvfunint
!!
!>     @ingroup dhvfunPackage
!!     @brief  Interface to the functions hvfun and dhvdhz, which gives the Vector
!!        field associated to H (second member of the IVP) and its linearized.
!!
!!      Detailed description :
!!
!!      Initialize parameters for dhvfun subroutine
!!      For each time t(i)
!!      <ul>
!!      <li> call getArc() -> get the arc index at t(i) </li>
!!      <li> update parameters for dhvfun : the index and number of arcs are given through the parameters </li>
!!      <li> call hvfun()   -> get the hamiltonian vector field value </li>
!!      <li> call dhvdz()   -> get the hamiltonian matrix value </li>
!!      </ul>
!!
!!        @param[in]    t        time
!!        @param[in]    n        State dimension
!!        @param[in]    k        Number of jacobi fields
!!        @param[in]    z        State and adjoint state at t
!!        @param[in]    dz       Jacobi fields at t
!!        @param[in]    nbarc    Number of arcs
!!        @param[in]    ti       ti = [t0 t1 .. t_{nbarc-1} tf], where t0 is the initial time and tf the final time.
!!        @param[in]    lpar     Number of optional parameters
!!        @param[in]    par      Optional parameters
!!        @param[in]    nt       Dimension of t and number of columns of z and value
!!
!!        @param[out]    hv      The second member of the BVP
!!        @param[out]   dhv      The second member of the Variationnal equations
!!
!!  \author Olivier Cots
!!  \date   2014
!!  \copyright Eclipse Public License
      Subroutine dhvfunint(t,n,k,z,dz,nbarc,ti,lpar,par,nt,hv,dhv)
       use utils
       use derivatives
       implicit none
       integer,      intent(in)                             :: n,nbarc
       integer,      intent(in)                             :: lpar
       integer,      intent(in)                             :: nt
       integer,      intent(in)                             :: k
       double precision, intent(in),  dimension(nt)         :: t
       double precision, intent(in),  dimension(nbarc+1)    :: ti
       double precision, intent(in),  dimension(2*n,nt)     :: z
       double precision, intent(in),  dimension(2*n,nt*k)   :: dz
       double precision, intent(in),  dimension(lpar)       :: par
       double precision, intent(out), dimension(2*n,nt)     :: hv
       double precision, intent(out), dimension(2*n,nt*k)   :: dhv

       !local variables
       integer :: i,iarc
       double precision, dimension(lpar+2)       :: paraux
       double precision, dimension(2*n*k)        :: dzaux, dhvaux

       !initialize parameters for hvfun subroutine
       paraux(1:lpar) = par
       paraux(lpar+2) = dble(nbarc)

       !For each time t(i), get the index of the arc associated
       !and compute the value of the hamiltonian vector field
       do i=1,nt

            call getArc(t(i),nbarc,ti,iarc)  !get the arc index

            paraux(lpar+1) = iarc            !the index is given through paraux
            call hvfun(2*n,t(i),z(:,i),hv(:,i),paraux,lpar+2) !get hvfun value

            !Compute value(...) = A * Y(...) for dz/dz0 : must be put in column vector
            dzaux = reshape(dz(:,(i-1)*k+1:i*k),(/2*n*k/))
            call dhvdz(t(i),n,z(:,i),k,dzaux,nbarc,iarc,lpar,par,dhvaux)
            dhv(:,(i-1)*k+1:i*k) = reshape(dhvaux,(/2*n,k/))

       end do

      end subroutine dhvfunint

!!******************************************************************************************!!
! Written on 2014                                                                           !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!


