!!********************************************************************
!!
!!     Subroutine expdhvfun
!!
!>     @ingroup expdhvfunPackage
!!     @brief   Computes the (right) chronological exponential of the
!!              variational system associated to hv.
!!     @param[in]   n     State dimension
!!     @param[in]   k     Number of jacobi fields
!!     @param[in]   z0    initial flow
!!     @param[in]   dz0   initial Jacobi fields
!!     @param[in]   lpar  Number of optional parameters
!!     @param[in]   par  Optional parameters
!!     @param[in]   t0    initial time
!!     @param[in]   tf    final time
!!       @param[in] nbarc      Number of arcs
!!       @param[in] ti     ti = [t0 t1 .. t_nbarc-1 tf], where t0 is the initial time and tf the final time.
!!
!!     @param[out]  flag   Integration output (should be 1)
!!     @param[out]  dimeT  Number of steps calculated
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine expdhvfun(n,k,z0,dz0,lpar,par,t0,tf,nbarc,ti,flag,dimeT)
        use utils
        use defs
        use adds
        implicit none
        integer,            intent(in)                     :: n,nbarc
        integer,            intent(in)                     :: k
        integer,            intent(in)                     :: lpar
        double precision,   intent(in)                     :: t0
        double precision,   intent(in)                     :: tf
        double precision,   intent(in), dimension(nbarc+1) :: ti
        double precision,   intent(in), dimension(lpar)    :: par
        double precision,   intent(in), dimension(2*n)     :: z0
        double precision,   intent(in), dimension(2*n,k)   :: dz0
        integer,            intent(out)                    :: flag
        integer,            intent(out)                    :: dimeT

        external expdhvrhs

        !local variables
        integer                                         :: neq, lparaux
        double precision                                :: tin
        double precision                                :: tfin
        double precision, dimension(lpar+4+2*n*(k+1))   :: paraux
        double precision, dimension(2*n*k,1)            :: aux
        double precision, dimension(2*n*(k+1),nbarc+1)  :: dzi
        double precision, dimension(2*n*(k+1))          :: dzin,scaletab,dzscalediv
        integer                                         :: arc,i,j,inc,flagaux,deb,fin
        double precision, dimension(:), allocatable     :: tspan

        call getArea(t0,tf,nbarc,ti,deb,fin,inc)

        !expdhvfun can be called only on one arc because we do not deal with jump on the jacobi equations
        if(fin.ne.deb)then
            CALL printandstop('Error: you can call expdhvfun only along one arc.')
        end if

        allocate(tspan(inc*(fin-deb)+2))
        call getTspan(t0,tf,nbarc,ti,deb,fin,inc,tspan)

        neq = 2*n*k+2*n
        dzi  = 0d0
        dzin = 0d0
        !dz0 is converted into a vector and put at the first element of dzi
        !dzi will contain dz for each instant of tspan
        dzi(1:2*n,1) = z0
        aux = reshape(dz0,(/2*n*k,1/))
        dzi(2*n+1:neq,1) = aux(:,1)

        !scaling
        call scaling(neq,dzi(:,1),dzin,scaletab,kscalG,scaltolG)
        dzi(:,1) = dzin

        lparaux        = lpar+4+neq
        paraux(1:lpar) = par
        paraux(lpar+2) = dble(nbarc)
        paraux(lpar+3) = dble(n)
        paraux(lpar+4) = dble(k)
        paraux(lpar+5:lparaux) = scaletab

        flag = 1
        j = 1
        !it is written like this but actually deb=fin, so there is only one step
        do i=deb,fin,inc
            arc = i
            paraux(lpar+1) = dble(arc)
            !integration
            tin  = tspan(j)
            tfin = tspan(j+1)
            dzin = dzi(:,j)

            call odeexpdhv(expdhvrhs,neq,dzin,paraux,lparaux,2*n,tin,tfin,dzi(:,j+1),flagaux)
            if(flagaux.ne.1)then
            if(flag.eq.1)then
                flag = flagaux
            end if
            end if
            j = j + 1
        end do
        dimeT = LONGUEUR(etatsdhvG)

        deallocate(tspan)

    end subroutine expdhvfun

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!
