!!********************************************************************
!!
!!     Subroutine expdhvfunint
!!
!>     @ingroup expdhvfunPackage
!!     @brief  Interface of expdhvfun.
!!
!!      Detailed description :
!!      <ul>
!!      <li> Initialize global parameters
!!      <ul>
!!       <li> call initTspanTi()        :    Initialization of the grid for dense output wrt ti </li>
!!       <li> call initTol()            :    Initialization of integrator tolerances </li>
!!       <li> call initIntegExpdhvfun() :    Initialization of integrator choice </li>
!!       <li> call initMaxSteps()       :    Initialization of maximum steps </li>
!!       <li> call initMaxStepsSize()   :    Initialization of maximum steps size </li>
!!       <li> call initListe()          :    Initialization of the output list (allocation) </li>
!!       <li> call initTolScaling()     :    Initialization of tolerance for auto scaling </li>
!!      </ul>
!!      </li>
!!      <li> call expdhvfun()           :    call the mere function </li>
!!      <li> The output is get calling getstatesdhv() </li>
!!      </ul>
!!
!!        @param[in]    n          State dimension
!!        @param[in]    k          Number of jacobi fields
!!        @param[in]    z0         initial flow
!!        @param[in]    dz0        initial Jacobi fields
!!        @param[in]    lpar       Number of optional parameters
!!        @param[in]    par        Optional parameters
!!        @param[in]    nt         Dimension of tspan
!!        @param[in]    tspan      Grid time : [t0 t1 ... tf]
!!        @param[in]    nbarc      Number of arcs
!!        @param[in]    ti         Contains t0, tf and the intermediate times if any. 
!!                                 ti = [t0 t1 .. t_{nbarc-1} tf]
!!        @param[in]    at         Absolute tolerance for integration
!!        @param[in]    rt         Relative tolerance for integration
!!        @param[in]    inte       Name of the integrator
!!        @param[in]    ms         Maximum of integration steps
!!        @param[in]    mss        Maximum size of integration steps
!!        @param[in]    tsc        Tolerance for automatic scaling
!!
!!        @param[out]    flag      Integration output (should be 1)
!!        @param[out]    dimeT     Number of steps calculated
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine expdhvfunint(n,k,z0,dz0,lpar,par,nt,tspan,nbarc,ti,flag,dimeT,at,rt,inte,ms,mss,tsc)
      use adds
      use init
      implicit none
      integer,          intent(in)                     :: n,nbarc
      integer,          intent(in)                     :: k
      integer,          intent(in)                     :: lpar
      integer,          intent(in)                     :: nt
      double precision, intent(in), dimension(lpar)    :: par
      double precision, intent(in), dimension(2*n)     :: z0
      double precision, intent(in), dimension(2*n,k)   :: dz0
      double precision, intent(in), dimension(nt)      :: tspan
      double precision, intent(in), dimension(nbarc+1) :: ti
      integer,          intent(out)                    :: flag
      integer,          intent(out)                    :: dimeT
      integer,          intent(in)                     :: ms
      double precision, intent(in)                     :: at
      double precision, intent(in)                     :: rt
      double precision, intent(in)                     :: mss
      double precision, intent(in)                     :: tsc
      character(32),    intent(in)                     :: inte

      !Initialize global parameters
      call initTspanTi(nt,tspan,nbarc,ti)   ! Initialization of the grid for dense output wrt ti
      call initTol(at,rt)                   ! Initialization of integrator tolerances 
      call initIntegExpdhvfun(inte)         ! Initialization of integrator choice
      call initMaxSteps(ms)                 ! Initialization of maximum steps
      call initMaxStepSize(mss)             ! Initialization of maximum steps size
      call initListe(3)                     ! Initialization of the output list (allocation)
      call initTolScaling(tsc)              ! Initialization of tolerance for auto scaling

      call expdhvfun(n,k,z0,dz0,lpar,par,tspan(1),tspan(nt),nbarc,ti,flag,dimeT)

   end subroutine expdhvfunint

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!
