!!********************************************************************
!!
!!     Subroutine exphvfun
!!
!>     @ingroup exphvfunPackage
!!     @brief   Computes the (right) chronological exponential of the
!!              Hamiltonian vector field hv defined by h.
!!     @param[in]   n      State dimension
!!     @param[in]   z0     initial flow
!!     @param[in]   lpar   Number of optional parameters
!!     @param[in]   par   Optional parameters
!!     @param[in]   t0     initial time
!!     @param[in]   tf     final time
!!       @param[in] nbarc      Number of arcs
!!       @param[in] ti     ti = [t0 t1 .. t_nbarc-1 tf], where t0 is the initial time and tf the final time.
!!
!!     @param[out]  flag   Integration output (should be 1)
!!     @param[out]  dimeT  Number of steps calculated
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine exphvfun(n,z0,lpar,par,t0,tf,nbarc,ti,flag,dimeT)
        use utils
        use defs
        use adds
        implicit none
        integer,            intent(in)                      :: n,nbarc
        integer,            intent(in)                      :: lpar
        double precision,   intent(in)                      :: t0
        double precision,   intent(in)                      :: tf
        double precision,   intent(in), dimension(nbarc+1)  :: ti
        double precision,   intent(in), dimension(2*n)      :: z0
        double precision,   intent(in), dimension(lpar)     :: par
        integer,            intent(out)                     :: flag
        integer,            intent(out)                     :: dimeT

        external exphvrhs

        !local variables
        double precision                            :: tin
        double precision                            :: tfin
        double precision, dimension(2*n)            :: zin,scaletab,zscalediv
        double precision, dimension(lpar+2+2*n)     :: paraux
        double precision, dimension(:), allocatable :: tspan
        integer                                     :: arc,i,j,flagaux,deb,fin,inc,lparaux
        double precision, dimension(2*n,nbarc+1)    :: zi

        !get first and last arc
        call getArea(t0,tf,nbarc,ti,deb,fin,inc)

        allocate(tspan(inc*(fin-deb)+2))
        !tspan contains all instants in ti between t0 and tf
        call getTspan(t0,tf,nbarc,ti,deb,fin,inc,tspan)

        !scaling
        call scaling(2*n,z0,zin,scaletab,kscalG,scaltolG)

        lparaux                 = lpar+2+2*n
        zi(:,1)                 = zin
        paraux(1:lpar)          = par
        paraux(lpar+2)          = dble(nbarc)
        paraux(lpar+3:lparaux)  = scaletab
        flag = 1
        j = 1
        do i=deb,fin,inc
            arc = i
            paraux(lpar+1) = dble(arc) !we indicate in which arc we are
            !integration
            tin  = tspan(j)
            tfin = tspan(j+1)
            zin  = zi(:,j)

            call odeexphv(exphvrhs,2*n,zin,paraux,lparaux,2*n,tin,tfin,zi(:,j+1),flagaux)
            if(flagaux.ne.1)then
                if(flag.eq.1)then
                    flag = flagaux !if there is even a single integration which went wrong, it is indicated to flag
                end if
            end if
            j = j + 1
        end do
        dimeT = LONGUEUR(etatsG)

        deallocate(tspan)

    end subroutine exphvfun

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!
