!!********************************************************************
!!
!!     Subroutine getpath
!!
!>     @ingroup hampathPackage
!!     @brief  Transforms the list pathG into arrays for matlab dealing.
!!
!!     We extract all information and then we remove the list pathG and
!!     deallocate the initial and final paramters par0G and parfG,
!!     calling closePar(). The parameters at each step are retrieved by
!!     the method getPar() and zi and ti with the method undoc().
!!
!!        \param[in]    t0        Initial time
!!        \param[in]    tf        Final time
!!        @param[in]    m         Number of arcs
!!        @param[in]    n         State dimension
!!        @param[in]    ne        Number of unknown for the shooting method
!!        \param[in]    ntf       If 1, then the final time is free and tf is an independent variable
!!        @param[in]    dimepath  Number of steps from hampath
!!        @param[in]    lpar      Number of parameters
!!
!!        @param[out]    parout  Parameters at each integration step from hampath
!!        @param[out]    tis     ti at each integration step
!!        @param[out]    zis     zi at each integration step
!!        @param[out]    tocis   Tangent vector at each integration step
!!        @param[out]    ss      Arc length at each integration step
!!        @param[out]    det     Det(S'(c(s)),c'(s)) at each integration step
!!        @param[out]    norms   |S(c(s))| at each integration step
!!        @param[out]    ps      <c'(s_old)|c'(s_new)> at each integration step
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine getpath(t0,tf,m,n,ne,ntf,dimepath,lpar,parout,tis,zis,tocis,ss,det,norms,ps)
        use ModGestListe
        use adds
        use utils
        use init
        implicit none
        integer,                                     intent(in)  :: m,n,ne,lpar,ntf
        integer,                                     intent(in)  :: dimepath
        double precision,                            intent(in)  :: t0,tf
        double precision, dimension(lpar,dimepath),  intent(out) :: parout
        double precision, dimension((m+1)*dimepath), intent(out) :: tis
        double precision, dimension(2*n,m*dimepath), intent(out) :: zis
        double precision, dimension(ne+1,dimepath) , intent(out) :: tocis
        double precision, dimension(dimepath),       intent(out) :: ss
        double precision, dimension(dimepath),       intent(out) :: det
        double precision, dimension(dimepath),       intent(out) :: norms
        double precision, dimension(dimepath),       intent(out) :: ps

        !local variables
        integer                                     :: i,j
        integer                                     :: dime
        double precision                            :: time
        double precision, dimension(:), allocatable :: valpath
        double precision, dimension(ne+1)           :: c
        double precision, dimension(m+1)            :: ti
        double precision, dimension(2*n,m)          :: zi
        double precision, dimension(lpar)           :: par
        double precision                            :: lambda

        dime = GET_VALEUR_DIM(pathG)
        allocate(valpath(dime))

        do i=1,dimepath
            call EXTRAIRE(time,dime,valpath,pathG)

            tocis(:,dimepath-i+1) = valpath(ne+1+1:2*(ne+1))
            det(dimepath-i+1)    = valpath(dime-2)
            norms(dimepath-i+1)  = valpath(dime-1)
            ps(dimepath-i+1)     = valpath(dime-0)

            ss(dimepath-i+1) = time
            c = valpath(1:ne+1)
            call undoc(t0,tf,m,n,ti,zi,lambda,ne,ntf,c)
            call getPar(lambda,lpar,par)
            parout(:,dimepath-i+1) = par
            tis((dimepath-i)*(m+1)+1:(dimepath-i+1)*(m+1)) = ti
            do j=1,m
                zis(:,(dimepath-i)*m+j) = zi(:,j)
            end do
        end do

        call DETRUIRE(pathG)
        deallocate(valpath)
        call closePar()

    end subroutine getpath

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!

