!!********************************************************************
!!
!!     Subroutine hampath
!!
!>     @ingroup hampathPackage
!!     @brief   Performs the homotopy from (z00,lambda0) to (zf,lambdaf),
!!        then find possible bifurcations. Returns the number of
!!        steps during continuation, the number of bifurcations and
!!        flags telling if all went good.
!!        @param[in]    nbarc      Number of arcs
!!        @param[in]    ti         Contains t0, tf and the intermediate times if any.
!!                                 ti = [t0 t1 .. t_{nbarc-1} tf]
!!        @param[in]    n          State dimension
!!        @param[in]    zi         Initial guess : state and adjoint state at ti
!!        \param[in]    ne         Dimension of the shooting function
!!        \param[in]    ntf        If 1, then the final time is free and tf is an independent variable
!!        \param[in]    msf        Maximum arc length
!!        \param[out]    tif        Final ti
!!        \param[out]    zif        Final zi
!!        @param[in]    lpar       Number of optional parameters
!!        @param[out]    parf       Final optional parameters
!!
!!        @param[out]   dimepath   Number of steps
!!        @param[out]   dimebif    Number of bifurcations
!!        @param[out]   sfreached  Equals 1 if sfmax has been reached
!!        @param[out]   flag       Integration output (should be 1)
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine hampath(nbarc,ti,n,zi,ne,ntf,msf,tif,zif,lpar,parf,sfreached,dimepath,dimebif,flag)
        use utils
        use defs
        use adds
        use bifurcations
        implicit none
        integer,          intent(in)                               :: n,nbarc,lpar,ne,ntf
        double precision, intent(in)                               :: msf
        double precision, intent(in), dimension(nbarc+1)           :: ti
        double precision, intent(in), dimension(2*n,nbarc)         :: zi
        double precision, intent(out), dimension(nbarc+1)          :: tif
        double precision, intent(out), dimension(2*n,nbarc)        :: zif
        double precision, intent(out), dimension(lpar)             :: parf
        integer,          intent(out)                              :: sfreached,dimepath,dimebif,flag

        external tangentVector

        !local variables
        double precision, dimension(2*n,nbarc)  :: expzi
        double precision, dimension(ne)         :: shootf
        integer                                 :: flagsfun
        double precision, dimension(ne+1)       :: scaletab,cscalediv,c0scaled,cfscaled,cf
        double precision, dimension(6+ne+1)     :: paraux
        integer                                 :: lparaux,ip
        double precision                        :: lambda,s0,sf,t0,tf,lambdaf
        double precision, dimension(ne+1)       :: c0

        !allocate needed memory for some global parameters
        allocate(tocoldG(ne+1))                     !previous tangent vector
        allocate(tocoldbifG(ne+1))                  !previous tangent vector used in bifurcation method
        allocate(tocG(ne+1))                        !current tangent vector

        tocoldG = 0d0; sdetG   = 0d0 ; hinitG  = 0  !used in tangentvector and soloutCont

        s0      = 0d0                               !initial arc length
        sf      = msf                               !final arc length equals to maximum arc length

        !initialization of first homotopic parameter
        if(nparhomG.eq.1)then
            lambda = par0G(iparhomG(1))        !here we have an homotopy on a scalar parameter
        else
            lambda  = 0d0                      !here we have f(lambda,par0,parf) with lambda = 0 to 1 and par is a vector
        end if

        !making the initial vector : c(0) = (zi,ti,lambda) in the generic case
        call doc(nbarc,n,ti,zi,lambda,ne,ntf,c0)

        !init print
        call printinit

        !init print in file
        call printinitinfile

        !initialization of the parameters for hampath
        lparaux   = 6+ne+1
        t0        = ti(1)
        tf        = ti(nbarc+1)
        paraux(1) = dble(nbarc)
        paraux(2) = dble(n)
        paraux(3) = dble(ne)
        paraux(4) = dble(ntf)
        paraux(5) = t0
        paraux(6) = tf

        !scaling
        call scaling(ne+1,c0,c0scaled,scaletab,kscalG,scaltolG)
        paraux(6+1:6+ne+1) = scaletab

        ip=1
        call odecont(tangentvector,ne+1,c0scaled,paraux,lparaux,ne+1,s0,sf,cfscaled,flag)
        call scalediv(ne+1,cfscaled,scaletab,cf)

        if (sf >= msf) then
            sfreached = 1
            flag = 0
        end if

        dimepath = LONGUEUR(pathG)

        !find bifurcations
        dimebif = 0
        !call findbif(ne+1,dimebif,dimepath,2,paraux(1:2))

        !final display
        call undoc(t0,tf,nbarc,n,tif,zif,lambdaf,ne,ntf,cf)
        call getPar(lambdaf,lpar,parf)
        call sfun(nbarc,tif,n,zif,lpar,parf,expzi,ne,shootf,flagsfun)
        call printfinal(nbarc,tif,n,zif,lpar,parf,ne,ntf,sf,msf,lambdaf,shootf,dimepath,flag)

        !deallocate memory created at the beginning
        deallocate(tocoldG)
        deallocate(tocoldbifG)
        deallocate(tocG)

    end subroutine hampath

!------------------------------------------------------------------------------------------------------------------------------------------------!
!------------------------------------------------------------------------------------------------------------------------------------------------!
!------------------------------------------------------------------------------------------------------------------------------------------------!
!>  @ingroup hampathPackage
!!  @brief   Intial priniting for hampath() method.
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    subroutine printinit()
        use utils
        use defs
        implicit none

        CHARACTER(len=120) :: LINE

        if((dispiterG.gt.0).AND.((displayG.EQ.-1).OR.(displayG.EQ.1))) then
!            WRITE (LINE,'(a19, $)') '-----------------';    call myprint(LINE,.false.)
!            WRITE (LINE,'(a16, $)') '--------------';       call myprint(LINE,.false.)
!            WRITE (LINE,'(a16, $)') '--------------';       call myprint(LINE,.false.)
!            WRITE (LINE,'(a16, $)') '--------------';       call myprint(LINE,.false.)
!            WRITE (LINE,'(a16, $)') '--------------';       call myprint(LINE,.false.)
!            WRITE (LINE,'(a19, $)') '-----------------';    call myprint(LINE,.false.)
!            WRITE (LINE,'(a19, $)') '-----------------';    call myprint(LINE,.false.)
!            WRITE (LINE,'(a)') '';                  call myprint(LINE,.true.)
            WRITE (LINE,'(a)') '';                  call myprint(LINE,.true.)
            WRITE (LINE,'(a)') '  Homotopic param.   Arclength s     det(s)';   call myprint(LINE,.false.)
            WRITE (LINE,'(a)') '          |S(z)|          Inner product   |x(0)|             |p(0)|'
            call myprint(LINE,.true.)
            !WRITE (LINE,'(a)') '';                  call myprint(LINE,.true.)
        end if

    end subroutine printinit

!------------------------------------------------------------------------------------------------------------------------------------------------!
!------------------------------------------------------------------------------------------------------------------------------------------------!
!------------------------------------------------------------------------------------------------------------------------------------------------!
!>  @ingroup hampathPackage
!!  @brief   Final printing for hampath() method.
!!        @param[in]    nbarc      Number of arcs
!!        \param[out]   tif        Final ti
!!        @param[in]    n          State dimension
!!        \param[out]   zif        Final zi
!!        @param[in]    lpar       Number of optional parameters
!!        @param[out]   parf       Final optional parameters
!!        \param[in]    ne         Dimension of the shooting function
!!        \param[in]    ntf        If 1, then the final time is free and tf is an independent variable
!!        \param[in]    sf         Final arc length
!!        \param[in]    msf        Maximum arc length
!!        \param[in]    lambdaf    Final homotopic parameter
!!        \param[in]    shootf     Final sfun() value at final point of the path of zeros
!!        @param[in]    dimepath   Number of steps
!!        @param[in]    flag       Integration output (should be 1)
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    subroutine printfinal(nbarc,tif,n,zif,lpar,parf,ne,ntf,sf,msf,lambdaf,shootf,dimepath,flag)
        use utils
        implicit none
        integer,          intent(in)                       :: n,nbarc,lpar,ne,ntf
        double precision, intent(in), dimension(nbarc+1)   :: tif
        double precision, intent(in), dimension(2*n,nbarc) :: zif
        double precision, intent(in), dimension(lpar)      :: parf
        double precision, intent(in)                       :: sf,msf,lambdaf
        double precision, intent(in), dimension(ne)        :: shootf
        integer,          intent(in)                       :: dimepath,flag

        !local variables
        integer             :: i,j
        CHARACTER(len=120)  :: LINE
        double precision    :: DNRM2

        if((displayG.eq.1).OR.(displayG.EQ.-3)) then

            if (sf >= msf) then
                WRITE (LINE,'(a)') '';  call myprint(LINE,.true.)
                WRITE (LINE,'(a)')  ' ||| WARNING: Continuation integration ->'
                call myprint(LINE,.false.)
                WRITE (LINE,'(a)')  ' PROBLEM NEEDS A LARGER SF '
                call myprint(LINE,.true.)
            end if

            WRITE (LINE,'(a)') '';  call myprint(LINE,.true.)
            WRITE (LINE,'(a)') ' Results of the homotopy:'; call myprint(LINE,.true.)
            if(flag.eq.1) then
                WRITE (LINE,'(a)') ''; call myprint(LINE,.true.)
                WRITE (LINE,'(a)') ' Homotopy successfully completed !'
                call myprint(LINE,.true.)
                WRITE (LINE,'(a)') ''; call myprint(LINE,.true.)
            end if
            WRITE (LINE,'(a)') ' Remark: during the homotopy, the column corresponding to the norm'
            call myprint(LINE,.false.)
            WRITE (LINE,'(a)') ' of the shooting function is an indication of the real value.'
            call myprint(LINE,.true.)
            WRITE (LINE,'(a)') ' The real value is given by sfun which is called at the end of the '
            call myprint(LINE,.false.)
            WRITE (LINE,'(a)') ' homotopy and given below.'
            call myprint(LINE,.true.)
            WRITE (LINE,'(a)') ''; call myprint(LINE,.true.)

            !!
            WRITE (LINE,'(" steps              = ",i0.1)')  dimepath;    call myprint(LINE,.true.)
            WRITE (LINE,'(" flag               = ",i0.1)')      flag;    call myprint(LINE,.true.)
            WRITE (LINE,'(" sf                 =",e23.15)')      sf;    call myprint(LINE,.true.)
            WRITE (LINE,'(" lambdaf            =",e23.15)') lambdaf;    call myprint(LINE,.true.)
            if(ntf.eq.1) then
                WRITE (LINE,'(" |S(zf,lambdaf,tf)| =",e23.15)') DNRM2(ne, shootf, 1);    call myprint(LINE,.true.)
            else
                WRITE (LINE,'(" |S(zf,lambdaf)|    =",e23.15)') DNRM2(ne, shootf, 1);    call myprint(LINE,.true.)
            end if
            !!

            WRITE (LINE,'(a)') ''; call myprint(LINE,.true.)
            WRITE (LINE,'(a)') ''; call myprint(LINE,.true.)
            WRITE (LINE,'(a)') ' ti_final  = ['; call myprint(LINE,.false.)
            DO i = 1, nbarc+1
                WRITE (LINE,'(e23.15)') tif(i)
                call myprint(LINE,.false.)
            END DO
            WRITE (LINE,'(a)') ']'
            call myprint(LINE,.true.)

            WRITE (LINE,'(a)') ''; call myprint(LINE,.true.)
            WRITE (LINE,'(a)') ' zi_final  = ['
            call myprint(LINE,.false.)
            DO i = 1, 2*n
                if(nbarc.eq.1)then
                    if(i.eq.1)then
                        WRITE (LINE,'(e23.15)') zif(i,1)
                        call myprint(LINE,.true.)
                    elseif(i.eq.2*n)then
                        WRITE (LINE,'("              ",e23.15)') zif(i,1)
                        call myprint(LINE,.false.)
                    else
                        WRITE (LINE,'("              ",e23.15)') zif(i,1)
                        call myprint(LINE,.true.)
                    end if
                else
                    DO j=1,nbarc
                        if(j.eq.1)then
                            if(i.eq.1)then
                                WRITE (LINE,'(e23.15)') zif(i,j)
                            else
                                WRITE (LINE,'("              ",e23.15)') zif(i,j)
                            end if
                            call myprint(LINE,.false.)
                        elseif(j.eq.nbarc)then
                            if(i.eq.2*n)then
                                WRITE (LINE,'(e23.15)') zif(i,j)
                                call myprint(LINE,.false.)
                            else
                                WRITE (LINE,'(e23.15)') zif(i,j)
                                call myprint(LINE,.true.)
                            end if
                        else
                            WRITE (LINE,'(e23.15)') zif(i,j)
                            call myprint(LINE,.false.)
                        end if
                    END DO
                end if
            END DO
            WRITE (LINE,'(a)') ']'
            call myprint(LINE,.true.)

            WRITE (LINE,'(a)') ''; call myprint(LINE,.true.)
            WRITE (LINE,'(a)') ' par_final = ['
            call myprint(LINE,.false.)
            DO i = 1, lpar
                WRITE (LINE,'(e23.15)') parf(i)
                call myprint(LINE,.false.)
            END DO
            WRITE (LINE,'(a)') ']'''
            call myprint(LINE,.true.)
            !WRITE (LINE,'(a)') ''; call myprint(LINE,.true.)

        end if

    end subroutine printfinal

!------------------------------------------------------------------------------------------------------------------------------------------------!
!------------------------------------------------------------------------------------------------------------------------------------------------!
!------------------------------------------------------------------------------------------------------------------------------------------------!
!>  @ingroup hampathPackage
!!  @brief   Initial printing in file for the save of homotopic path.
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    subroutine printinitinfile()
        use utils
        use defs
        implicit none

        CHARACTER(len=120)  :: LINE
        CHARACTER(len = 8)  :: frmt ! format
        CHARACTER(len = 8)  :: DateINFO ! ccyymmdd
        CHARACTER(len = 10) :: TimeINFO ! hhmmss.sss
        LOGICAL             :: file_exists
        integer             :: i,j,unitfile

        ! creating file for save
        ! ouvre le fichier pour y ecrire
        if(doSavePathInFileG.eq.1)then
            i = 0
            WRITE (filenamepathG,'(a)') 'savedPath.m'
            INQUIRE(FILE=filenamepathG, EXIST=file_exists)    ! file_exists will be TRUE if the file
                                                            ! exists and FALSE otherwise
            j=1
            do while(file_exists)
                i=i+1
                j=ceiling(log10(dble(i)+0.1))
                WRITE (frmt,'(a,i1,a)') '(a,i',j,',a)'
                WRITE (filenamepathG,frmt) 'savedPath',i,'.m'
                INQUIRE(FILE=filenamepathG, EXIST=file_exists)
            end do
            CALL DATE_AND_TIME(DateINFO, TimeINFO) !The current time, in the form hhmmss.sss, where hh is the hour, 
                                                    !mm minutes, and ss.sss seconds and milliseconds. 
            !WRITE (filenamepathG,*) 'hampath_saved_path_',DateINFO,'_',TimeINFO,'.m'
            unitfile=20
            open (unit=unitfile,file=filenamepathG,action="write",status="new")
            ![lout,tiout,ziout,sout,viout,bifs,dets,normS,ps,flag] = hampath(parspan,ti,zi,options)
            WRITE (LINE,*) '%Created by Hampath on ', DateINFO(1:4),'/',DateINFO(5:6),'/',DateINFO(7:8), &
                                                ' at ',TimeINFO(1:2),'h ',TimeINFO(3:4),'min ',TimeINFO(5:6),'s.'
            WRITE (unit=unitfile,fmt='(a)') 'function [lout,tiout,ziout,sout,viout,dets,normS,ps] = savedPath()'
            WRITE (unit=unitfile,fmt='(a)') '%Usage [lout,tiout,ziout,sout,viout,dets,normS,ps] = savedPath()'
            WRITE (unit=unitfile,fmt='(a)') ' '
            WRITE (unit=unitfile,fmt='(a)') LINE
            WRITE (unit=unitfile,fmt='(a)') 'lout  = [];'
            WRITE (unit=unitfile,fmt='(a)') 'tiout = [];'
            WRITE (unit=unitfile,fmt='(a)') 'ziout = [];'
            WRITE (unit=unitfile,fmt='(a)') 'sout  = [];'
            WRITE (unit=unitfile,fmt='(a)') 'viout = [];'
            WRITE (unit=unitfile,fmt='(a)') 'dets  = [];'
            WRITE (unit=unitfile,fmt='(a)') 'normS = [];'
            WRITE (unit=unitfile,fmt='(a)') 'ps    = [];'
            close(unit=unitfile)
        end if

    end subroutine printinitinfile

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!
