!!********************************************************************
!!
!!     Subroutine hampathint
!!
!>     @ingroup hampathPackage
!!     @brief  Interface for the continuation method.
!!
!!      Detailed description :
!!        <ul>
!!        <li> Initialize global parameters
!!      <ul>
!!        <li>      call initDerivative()     ->    method to derivate </li>
!!        <li>      call initDisplay()        ->    display choice and steps to display </li>
!!        <li>      call initSavePath()       ->    choice to save path in file or not </li>
!!        <li>      call initIntegEqvar()     ->    integrator choice for variationnal equations </li>
!!        <li>      call initIntegHam()       ->    integrator choice for path following </li>
!!        <li>      call initMaxSteps()       ->    maximum integrator steps for var eq </li>
!!        <li>      call initMaxStepsHam()    ->    maximum integrator steps for path </li>
!!        <li>      call initStopAtTurningPt()->    stop or not after encountered a turning point </li>
!!        <li>      call initNormeSfunMax()   ->    maximum shooting function norm allowed during continuation </li>
!!        <li>      call initMaxStepSize()    ->    maximum steps size </li>
!!        <li>      call initMaxStepSizeHam() ->    maximum steps size for hampath </li>
!!        <li>      call initTolCont()        ->    path following tolerances : absolute, relative and for the dense output </li>
!!        <li>      call initTol()            ->    variationnal equations tolerances </li>
!!        <li>      call initTolScaling()     ->    Initialization of tolerance for auto scaling </li>
!!        <li>      call initListe()          ->    initialization of the lists for output </li>
!!        <li>      call initPar()            ->    initialization of homotopic and constant user parameters </li>
!!        </ul></li>
!!        <li> Initialization of parameters </li>
!!        <li> Call the mere function : call hampath() </li>
!!        <li> The output is recovered  calling getpath() and getbif() </li>
!!        </ul>
!!
!!        @param[in]    nbarc     Number of arcs
!!        @param[in]    ti        Contains t0, tf and the intermediate times if any.
!!                                ti = [t0 t1 .. t_{nbarc-1} tf]
!!        @param[in]    n         State dimension
!!        @param[in]    zi        Initial guess : state and adjoint state at ti
!!        @param[in]    nparspan  Size of homotopic parameters grid
!!        @param[in]    lparspan  Number of optional parameters
!!        @param[in]    parspan   Parameters grid
!!        \param[in]    ne         Dimension of the shooting function
!!        \param[in]    ntf        If 1, then the final time is free and tf is an independent variable
!!        \param[in]    iwork     Integer parameters
!!        \param[in]    dwork     Double precision parameters
!!        @param[in]    der       Derivative method
!!        @param[in]    disp      Display or not the results of the homotopy
!!        @param[in]    inte      Name of the integrator
!!        @param[in]    inth      Name of the integrator of the homotopy
!!        @param[in]    sapa      Save path in file or not
!!
!!        @param[out]    dimepath    Number of steps
!!        @param[out]    dimebif     Number of bifurcations
!!        @param[out]    flag        Integration output (should be 1)
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine hampathint(  nbarc,ti,n,zi,lparspan,nparspan,parspan,ne,ntf, &
                            dimepath,dimebif,flag,                          &
                            iwork,dwork,der,disp,inte,inth,sapa)
        use adds
        use init
        use utils
        implicit none
        integer,          intent(in)                               :: n,nbarc,nparspan,lparspan,ne,ntf
        double precision, intent(in), dimension(nbarc+1)           :: ti
        double precision, intent(in), dimension(2*n,nbarc)         :: zi
        double precision, intent(in), dimension(lparspan,nparspan) :: parspan
        integer,          intent(out)                              :: dimepath,dimebif,flag
        integer,          intent(in), dimension(4)                 :: iwork
        double precision, intent(in), dimension(11)                :: dwork
        character(32),    intent(in)                               :: der,disp,inte,inth,sapa

!        character(32)       :: Derivative           = 'eqvar'
!        character(32)       :: Display              = 'on'
!        character(32)       :: DoSavePath           = 'off'
!        character(32)       :: FreeFinalTime        = 'false'
!        character(32)       :: ODE                  = 'dopri5'
!        character(32)       :: ODEHam               = 'dopri5'
!
!        integer             :: Dispiter             = 1
!        integer             :: MaxStepsOde          = 1d5
!        integer             :: MaxStepsOdeHam       = 1d5
!        integer             :: StopAtTurningPoint   = 0
!
!        double precision    :: MaxSf                = 1d5
!        double precision    :: MaxSfunNorm          = 1d-1
!        double precision    :: MaxStepSizeOde       = 0d0
!        double precision    :: MaxStepSizeOdeHam    = 0d0
!        double precision    :: TolOdeAbs            = 1d-14
!        double precision    :: TolOdeRel            = 1d-8
!        double precision    :: TolOdeHamAbs         = 1d-14
!        double precision    :: TolOdeHamRel         = 1d-8
!        double precision    :: TolDenseHamEnd       = 1d-8
!        double precision    :: TolDenseHamX         = 1d-8
!        double precision    :: TolScaling           = 1d-5

        !local variables
        double precision, dimension(nbarc+1)   :: tif
        double precision, dimension(2*n,nbarc) :: zif
        double precision, dimension(lparspan)  :: parf
        integer                             :: di,mso,msoh,satp,sfreached
        double precision                    :: msf,msn,msso,mssoh,toa,tor,toha,tohr,tdhe,tdhx,ts

        if(nparspan.ne.2)then
            call printandstop('Error in hampath call : parspan must have two columns!')
        end if

        di   = iwork(1); mso  = iwork(2) ; msoh = iwork(3) ; satp  = iwork(4);
        msf  = dwork(1); msn  = dwork(2) ; msso = dwork(3) ; mssoh = dwork(4);
        toa  = dwork(5); tor  = dwork(6) ; toha = dwork(7) ; tohr  = dwork(8);
        tdhe = dwork(9); tdhx = dwork(10); ts   = dwork(11);

        !Initialize global parameters
        call initDerivative(der)                !method to derivate
        call initDisplay(di,disp)               !display choice and steps to display
        call initSavePath(sapa)                 !choice to save path in file or not
        call initIntegEqvar(inte)               !integrator choice for variationnal equations
        call initIntegHam(inth)                 !integrator choice for path following

        call initMaxSteps(mso)                  !maximum integrator steps for var eq
        call initMaxStepsHam(msoh)              !maximum integrator steps for hampath
        call initStopAtTurningPt(satp)          !stop or not after a turning point

        call initNormeSfunMax(msn)              !maximum shooting function norm allowed during continuation
        call initMaxStepSize(msso)              !Initialization of maximum steps size
        call initMaxStepSizeHam(mssoh)          !Initialization of maximum steps size for hampath
        call initTolCont(toha,tohr,tdhe,tdhx)   !path following tolerances
        call initTol(toa,tor)                   !variationnal equations tolerances
        call initTolScaling(ts)                 !tolerance for auto scaling

        call initListe(1)                                 !initialization of the lists for output
        call initPar(lparspan,parspan(:,1),parspan(:,2))  !initialization of homotopic and constant user parameters

        !call the mere function
        call hampath(nbarc,ti,n,zi,ne,ntf,msf,tif,zif,lparspan,parf,sfreached,dimepath,dimebif,flag)

    end subroutine hampathint

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!
