!!********************************************************************
!!
!!     Subroutine soloutCont
!!
!>     @ingroup hampathPackage
!!     @brief   implementation of solout
!!     @param[in]   Niter       Number of iterations
!!     @param[in]   told        Previous time
!!     @param[in]   neq         Dimension of y
!!     @param[in]   dpar        Double precision parameters
!!     @param[in]   ldpar       Dimension of dpar
!!     @param[in]   ipar        Integer parameters
!!     @param[in]   lipar       Dimension of ipar
!!     @param[in]   fun         RHS
!!     @param[in]   lfunpar     Dimension of funpar
!!     @param[in,out]   time    Current time
!!     @param[in,out]   y       Current point, can be changed by dense output
!!     @param[in,out]   funpar  Parameters given to fun
!!     @param[in,out]   irtrn   Flag
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine soloutCont(Niter,told,time,y,neq,dpar,ldpar,ipar,lipar,fun,funpar,lfunpar,irtrn)
        use defs
        use adds
        implicit none
        integer,            intent(in)                         :: Niter
        double precision,   intent(in)                         :: told
        double precision,   intent(inout)                      :: time
        double precision,   intent(inout), dimension(neq)      :: y
        integer,            intent(in)                         :: neq
        double precision,   intent(in),    dimension(ldpar)    :: dpar
        integer,            intent(in)                         :: ldpar
        integer,            intent(in),    dimension(lipar)    :: ipar
        integer,            intent(in)                         :: lipar
        double precision,   intent(inout), dimension(lfunpar)  :: funpar
        integer,            intent(in)                         :: lfunpar
        integer,            intent(inout)                      :: irtrn

        interface
            Subroutine fun(neq,time,y,value,funpar,lfunpar)
                implicit none
                integer,          intent(in)                        :: neq
                integer,          intent(in)                        :: lfunpar
                double precision, intent(in)                        :: time
                double precision, intent(in),  dimension(lfunpar)   :: funpar
                double precision, intent(in),  dimension(neq)       :: y
                double precision, intent(out), dimension(neq)       :: value
            end Subroutine fun
        end interface

        !local variables
        double precision                 :: hstep,senslambda,lambdaf,y1,y2,t,t1,t2,ps,den,ecart,t0,tf
        double precision, dimension(neq) :: scaletab,toc,yscalediv
        integer                          :: i,n,dime,ne,ntf,fini,nbarc,ip
        type(TNOEUD), pointer            :: P
        double precision                 :: DNRM2

!        double precision, dimension(neq) :: yg,ydeb
!        integer                          :: dime,n,k,lpar,nbarc,iarc,nl,niarc
!        integer                          :: fini
!        double precision, dimension(1)   :: val
!        double precision, dimension(neq) :: yold
!        double precision                 :: tdeb
!        double precision                 :: vswitchR,lambda

        character(32)                    :: nameinte
        CHARACTER(len=120) :: LINE

        nameinte = integhamG
        scaletab = 1d0
        hstep    = time - told

!---------------------------------------------------------------------------------------------------------------
!---------------------------------------------------------------------------------------------------------------
        !get n,lpar,par,nbar,iarc
        nbarc    = nint(funpar(1))
        n        = nint(funpar(2))
        ne       = nint(funpar(3))
        ntf      = nint(funpar(4))
        t0       =      funpar(5)
        tf       =      funpar(6)

        scaletab = funpar(6+1:6+neq)

        ip = 1
!---------------------------------------------------------------------------------------------------------------
!---------------------------------------------------------------------------------------------------------------
        if(Niter.gt.1) then
            if(nparhomG.eq.1)then
                senslambda  = par0G(iparhomG(1)) - parfG(iparhomG(1))
                lambdaf     = parfG(iparhomG(1))
            else
                senslambda  = -1d0
                lambdaf     = 1d0
            end if

            !get the value at told and time
            call denseOutput(nameinte,neq,told,y1,told,hstep,lipar,ipar,ldpar,dpar)
            call denseOutput(nameinte,neq,time,y2,told,hstep,lipar,ipar,ldpar,dpar)

            !unscaling
            y1 = y1 / scaletab(neq)
            y2 = y2 / scaletab(neq)

            !detect turning point
            if ((sign(1d0,(y1-y2)*senslambda).lt.0d0).and.stopAtTurningPtG) then
                irtrn = -7
            end if
        end if

        !test if lambdaf is reached
        if(((lambdaf-y1)*(lambdaf-y2).LE.0) .and. (Niter.gt.1)) then !lambdaf reached

            t1 = told
            t2 = time
            t  = t1 + (t2-t1)/2
            call denseOutput(nameinte,neq,t,ps,told,hstep,lipar,ipar,ldpar,dpar)

            !unscaling
            ps = ps / scaletab(neq)

            if (ps/=0) then
                den = abs(ps)
                den = 1d0
            else
                den = 1d0
            end if

            if(sensLambda<0) then
                ecart =  (lambdaf-ps)/den
            else
                ecart = (-lambdaf+ps)/den
            end if

            !find the arc length t where lambda(t) = lambdaf
            do while ((ecart>densetolG) .or. (ecart<0d0))
                if((ps-lambdaf)*(y1-lambdaf)<0) then
                    t2 = t
                    y2 = ps
                else
                    t1 = t
                    y1 = ps
                end if
                t = t1 + (t2-t1)/2
                call denseOutput(nameinte,neq,t,ps,told,hstep,lipar,ipar,ldpar,dpar)
                !unscaling
                ps = ps / scaletab(neq)
                if (ps/=0) then
                    den = abs(ps)
                    den = 1d0
                else
                    den = 1d0
                end if
               if(sensLambda<0) then
                   ecart = (lambdaf-ps)/den
               else
                   ecart = (-lambdaf+ps)/den
               end if
            end do !end do while

            !compute the value of the vector at lambdaf
            do i=1,neq
                call denseOutput(nameinte,i,t,y(i),told,hstep,lipar,ipar,ldpar,dpar)
            end do

            time = t
            irtrn = -1

            !this call permits to get the current global values required
            call tangentvector(neq,time,y,toc,funpar,lfunpar)

        end if !end lambdaf reached

        tocoldG = tocG
        if(sdetG.ne.sign(1d0,deterG)) then
        !we change the current sign of the determinant if we found a bifurcation
            sdetG   = -1d0*sdetG
        end if

        !we add the information to the path of zeros
        call addpath(time,neq,scaletab,y,yscalediv,tocG,deterG,normsG,pdsG)

        dime = GET_VALEUR_DIM(pathG)

        !we check if the norm of the shooting function is too big
        if(GET_VALEUR(pathG,dime-1,dime) .gt. NormeSfunMaxG)then
          irtrn = -5
        end if

        !if lambdaf is not reached and we have made at least 20 steps
        !we check if we are progressing or not
        if(irtrn.ge.0 .and. Niter.ge.20) then
            fini = 0
            i = 0
            call GET_NOEUD_SUIVANT(P,pathG)
            y1 = GET_VALEUR(P,neq,dime)
            !hampath stops if the last 10 steps are too close two y  wrt lambda
            do while((i.LE.10).AND.(fini.EQ.0))
                call GET_NOEUD_SUIVANT(P)
                y2 = GET_VALEUR(P,neq,dime)

                ecart=abs((y2-y1)/y1)
                if(ecart.gt.densetolrelG)then
                    fini = 1
                end if

                y1 = y2
                i = i + 1
            end do
            if(fini.eq.0)then
                irtrn = -6
            end if
        end if

        !we display information at each step
        if((displayG.EQ.1) .and. (dispiterG.gt.0) .and. ((mod(Niter,dispiterG).eq.0) .or. (Niter.eq.1))) then
            WRITE (LINE,'(e19.11)') yscalediv(neq)
            call myprint(LINE,.false.)
            WRITE (LINE,'(e16.8)') time
            call myprint(LINE,.false.)
            WRITE (LINE,'(e16.8)') GET_VALEUR(pathG,dime-2,dime)
            call myprint(LINE,.false.)
            WRITE (LINE,'(e16.8)') GET_VALEUR(pathG,dime-1,dime)
            call myprint(LINE,.false.)
            WRITE (LINE,'(e16.8)') GET_VALEUR(pathG,dime,dime)
            call myprint(LINE,.false.)
            WRITE (LINE,'(e19.11)') DNRM2(n, yscalediv(1:n), 1)
            call myprint(LINE,.false.)
            WRITE (LINE,'(e19.11)') DNRM2(n, yscalediv(1+n:2*n), 1)
            call myprint(LINE,.true.)
        end if

        if(doSavePathInFileG.eq.1)then
            call savePathToFile(nbarc,n,ne,ntf,t0,tf,lparG,time,dime,GET_VALEUR(pathG,dime))
        end if

  end subroutine soloutCont

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!
