!!********************************************************************
!!
!!     Subroutine tangentvector
!!
!>     @ingroup hampathPackage
!!     @brief   RHS of homotopic integration : tangent vector.
!!        @param[in]    n1          State, costate and lambda dimension
!!        @param[in]    t           Time
!!        @param[in]    cscaled     cscaled = (z,lambda)
!!        @param[in]    lparaux     Number of optional parameters
!!        @param[in]    paraux      Optional parameters
!!
!!        @param[out]   toc         Tangent vector
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine tangentvector(n1,t,cscaled,toc,paraux,lparaux)
        use defs
        use utils
        implicit none
        integer,      intent(in)                            :: n1
        integer,      intent(in)                            :: lparaux
        double precision, intent(in)                        :: t
        double precision, intent(in),  dimension(n1)        :: cscaled
        double precision, intent(in),  dimension(lparaux)   :: paraux
        double precision, intent(out), dimension(n1)        :: toc

        !local variables
        integer                                         :: INFO,n,nbarc,i,lpar,ne,ntf
        double precision, dimension(:), allocatable     :: ti
        double precision, dimension(:,:), allocatable   :: zi
        double precision, dimension(:,:), allocatable   :: expzi
        double precision                                :: lambda,sensLambda,t0,tf
        double precision                                :: detR,detQ,deter,pds,norms
        double precision                                :: DDOT,DNRM2
        double precision, dimension(lparG)              :: par
        double precision, dimension(n1-1,n1)            :: A
        double precision, dimension(n1,n1-1)            :: AT
        double precision, dimension(n1,1)               :: e
        double precision, dimension(n1-1)               :: TAU,WORK,s
        double precision, dimension(n1)                 :: scaletab,c,tocin
        double precision, dimension(1)                  :: WORK2

        lpar     = lparG
        nbarc    = nint(paraux(1))
        n        = nint(paraux(2))
        ne       = nint(paraux(3))
        ntf      = nint(paraux(4))
        t0       =      paraux(5)
        tf       =      paraux(6)

        allocate(ti(nbarc+1),zi(2*n,nbarc),expzi(2*n,nbarc))
        scaletab  = paraux(7:6+n1)
        call scalediv(n1,cscaled,scaletab,c)

        if(nparhomG.eq.1)then
            senslambda = par0G(iparhomG(1)) - parfG(iparhomG(1))
        else
            senslambda = -1d0
        end if

        call undoc(t0,tf,nbarc,n,ti,zi,lambda,ne,ntf,c)
        call getPar(lambda,lpar,par)

        !AT = the transpose of the jacobian of S(y,lambda)
        call sjac(nbarc,ti,n,zi,lpar,par,expzi,1,lambda,ne,ntf,A)
        !call conditionner(A,n1-1,n1)

        AT = transpose(A)
        toc = 0

        !computation of AT=QR
        call DGEQRF( n1, n1-1, AT, n1, TAU, WORK, n1-1, INFO )

        !computation of toc
        e = 0d0
        e(n1,1) = 1d0
        call DORMQR( 'L', 'N', n1, 1, n1-1, AT, n1, TAU, e, n1, WORK2, 1, INFO )
        toc = e(:,1)

        !computation of det(Q) and det(R) where AT=QR
        detQ = 0d0
        detR = 1d0
        do i=1,n1-1
            if(TAU(i)>0d0) then
                detQ = detQ + 1
            end if
            detR = detR * AT(i,i)
        end do
        detQ = -1**detQ
        deter = detQ*detR

        !check if we scan the curve in the right direction
        if(sdetG.eq.0d0) then
            pds = 0d0
            if(sign(1d0,toc(n1)).eq.sign(1d0,sensLambda)) then
                !not the right direction
                toc    = -1d0*toc
                sdetG  = -1d0*sign(1d0,deter)
                deter  = -1d0*deter
            else
                sdetG  =      sign(1d0,deter)
            end if
        else
            if(sdetG.ne.sign(1d0,deter)) then
                toc   = -1d0*toc
                deter = -1d0*deter
            end if
            pds = DDOT(n1,toc,1,tocoldG,1)
            if(pds.lt.0d0) then
                pds       = -1d0*pds
                toc       = -1d0*toc
                deter     = -1d0*deter
            end if
        end if

        !computation of |S(z,lambda)|
        call efun(nbarc,n,ti,zi,expzi,lpar,par,ne,s)
        norms = DNRM2(ne, s, 1)

        if(hinitG.ne.1) then !this is because of dopri5 which calls tangentVector twice for initialization
        !save current global values
            tocG = toc
            pdsG = pds
            normsG = norms
            deterG = deter
        end if

        !scaling
        tocin = toc
        call scalemul(n1,tocin,scaletab,toc)

        hinitG = hinitG + 1
        deallocate(ti,zi,expzi)

    end subroutine tangentvector

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!

