!!********************************************************************
!!
!!     Subroutine hvfunint
!!
!>     @ingroup hvfunPackage
!!     @brief  Interface to the function hvfun, which gives the Vector
!!        field associated to H (second member of the IVP).
!!
!!      Detailed description :
!!
!!      Initialize parameters for hvfun subroutine
!!      For each time t(i)
!!      <ul>
!!      <li> call getArc() -> get the arc index at t(i) </li>
!!      <li> update parameters for hvfun : the index and number of arcs are given through the parameters </li>
!!      <li> call hvfun()   -> get the hamiltonian vector field value </li>
!!      </ul>
!!
!!        @param[in]    t        time
!!        @param[in]    n        State dimension
!!        @param[in]    z        State and adjoint state at t
!!        @param[in]    nbarc    Number of arcs
!!        @param[in]    ti       ti = [t0 t1 .. t_{nbarc-1} tf], where t0 is the initial time and tf the final time.
!!        @param[in]    lpar     Number of optional parameters
!!        @param[in]    par      Optional parameters
!!        @param[in]    lvalue   Dimension of t and number of columns of z and value
!!
!!        @param[out]    value   The second member of the BVP
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
      Subroutine hvfunint(t,n,z,nbarc,ti,lpar,par,lvalue,value)
       use utils
       implicit none
       integer,      intent(in)                         :: n,nbarc
       integer,      intent(in)                         :: lpar
       integer,      intent(in)                         :: lvalue
       double precision, intent(in),  dimension(lvalue)     :: t
       double precision, intent(in),  dimension(nbarc+1)    :: ti
       double precision, intent(in),  dimension(2*n,lvalue) :: z
       double precision, intent(in),  dimension(lpar)       :: par
       double precision, intent(out), dimension(2*n,lvalue) :: value

       !local variables
       integer :: i,arc
       double precision, dimension(lpar+2)       :: paraux

       !initialize parameters for hvfun subroutine
       paraux(1:lpar) = par
       paraux(lpar+2) = dble(nbarc)

       !For each time t(i), get the index of the arc associated
       !and compute the value of the hamiltonian vector field
       do i=1,lvalue
            call getArc(t(i),nbarc,ti,arc)  !get the arc index
            paraux(lpar+1) = arc            !the index is given through paraux
            call hvfun(2*n,t(i),z(:,i),value(:,i),paraux,lpar+2) !get hvfun value
       end do

    end subroutine hvfunint

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!

