!!********************************************************************
!!
!!     Subroutine sfun
!!
!>     @ingroup sfunPackage
!!     @brief   Computes the shooting function.
!!        @param[in]    nbarc     Number of arcs
!!        @param[in]    ti    ti = [t0 t1 .. tm-1 tf]
!!        @param[in]    n     State dimension
!!        @param[in]    zi    State and adjoint state at ti
!!        @param[in]    lpar  Number of optional parameters
!!        @param[in]    par  Optional parameters
!!        \param[in] ne      Dimension of the shooting function
!!
!!        @param[out]   expzi  State and adjoint state at [t1 .. tf]
!!        @param[out]   s     Value of the shooting function
!!        @param[out]   flag  Integration output (should be 1)
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine sfun(nbarc,ti,n,zi,lpar,par,expzi,ne,s,flag)
        use utils
        use adds
        implicit none
        integer,            intent(in)                        :: nbarc,n,ne
        integer,            intent(in)                        :: lpar
        double precision,   intent(in),  dimension(1+nbarc)   :: ti
        double precision,   intent(in),  dimension(lpar)      :: par
        double precision,   intent(in),  dimension(2*n,nbarc) :: zi
        double precision,   intent(out), dimension(2*n,nbarc) :: expzi
        double precision,   intent(out), dimension(ne)        :: s
        integer,            intent(out)                       :: flag

        external exphvrhs

        !Local variables
        double precision, dimension(2*n)        :: zin,scaletab,zscalediv
        double precision                        :: tin
        double precision                        :: tfin
        double precision, dimension(lpar+2+2*n) :: paraux
        integer                                 :: arc,i,lparaux

        !out init
        s = 0d0

        lparaux        = lpar+2+2*n
        paraux(1:lpar) = par
        paraux(lpar+2) = dble(nbarc)
        do i=1,nbarc
            arc = i
            paraux(lpar+1) = dble(arc) !we indicate on which arc we are
            !integration
            tin  = ti(i)
            tfin = ti(i+1)
            !scaling
            call scaling(2*n,zi(:,i),zin,scaletab,kscalG,scaltolG)
            paraux(lpar+3:lparaux) = scaletab
            !zin  = zi(:,i)

            call odesfun(exphvrhs,2*n,zin,paraux,lparaux,2*n,tin,tfin,expzi(:,i),flag)
            zin = expzi(:,i)
            call scalediv(2*n,zin,scaletab,zscalediv)
            expzi(:,i) = zscalediv
        end do

        call efun(nbarc,n,ti,zi,expzi,lpar,par,ne,s)

    end subroutine sfun

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!

