!!********************************************************************
!!
!!     Subroutine sfunint
!!
!>     @ingroup sfunPackage
!!     @brief  Interface of sfun
!!
!!      Detailed description :
!!        <ul>
!!        <li> Initialize global parameters
!!      <ul>
!!        <li>      call initTol()            ->    variationnal equations tolerances </li>
!!        <li>      call initIntegSfun()      ->    integrator choice for variationnal equations </li>
!!        <li>      call initMaxSteps()       ->    maximum integrator steps for both var eq and path </li>
!!       <li>       call initMaxStepsSize()   ->    Initialization of maximum steps size </li>
!!        <li>      call initTolScaling()     ->    Initialization of tolerance for auto scaling </li>
!!        </ul></li>
!!        <li> Call the mere function : call sfun() </li>
!!        </ul>
!!
!!        @param[in]    nbarc     Number of arcs
!!        @param[in]    ti        Contains t0, tf and the intermediate times if any.
!!                                  ti = [t0 t1 .. t_{nbarc-1} tf]
!!        @param[in]    n         State dimension
!!        @param[in]    zi        State and adjoint state at ti
!!        @param[in]    lpar      Number of optional parameters
!!        @param[in]    par       Optional parameters
!!        \param[in]    ne        Dimension of the shooting function
!!        @param[in]    at        Absolute tolerance for integration
!!        @param[in]    rt        Relative tolerance for integration
!!        @param[in]    inte      Name of the integrator
!!        @param[in]    ms        Maximum of integration steps
!!        @param[in]    mss       Maximum size of integration steps
!!        @param[in]    tsc        Tolerance for automatic scaling
!!
!!        @param[out]    s        Value of the shooting function
!!        @param[out]    flag     Integration output (should be 1)
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine sfunint(nbarc,ti,n,zi,lpar,par,ne,s,flag,at,rt,inte,ms,mss,tsc)
        use utils
        use init
        implicit none
        integer,            intent(in)                          :: n,nbarc,ne
        integer,            intent(in)                          :: lpar
        double precision,   intent(in),  dimension(lpar)        :: par
        double precision,   intent(in),  dimension(2*n,nbarc)   :: zi
        double precision,   intent(in),  dimension(nbarc+1)     :: ti
        double precision,   intent(out), dimension(ne)          :: s
        integer,            intent(out)                         :: flag
        integer,            intent(in)                          :: ms
        double precision,   intent(in)                          :: at
        double precision,   intent(in)                          :: rt
        double precision,   intent(in)                          :: mss
        double precision,   intent(in)                          :: tsc
        character(32),      intent(in)                          :: inte

        !local variables
        double precision, dimension(2*n,nbarc) :: expzi

        call initTol(at,rt)          ! variationnal equations tolerances
        call initIntegSfun(inte)     ! integrator choice for sfun
        call initMaxSteps(ms)        ! maximum integrator steps for var eq
        call initMaxStepSize(mss)    ! Initialization of maximum steps size
        call initTolScaling(tsc)     ! Initialization of tolerance for auto scaling

        !call the mere function
        call  sfun(nbarc,ti,n,zi,lpar,par,expzi,ne,s,flag)

   end subroutine sfunint

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!

