!!********************************************************************
!!
!!     Subroutine dexpzi
!!
!>     @ingroup sjacPackage
!!     @brief   Solution of variational equations
!!     @param[in]  t0      initial time
!!     @param[in]  tf      final time
!!     @param[in]  n       State dimension
!!     @param[in]  z0      State and adjoint state at t0
!!     @param[in]  nbarc       Number of arcs
!!     @param[in]  iarc    Index of the current arc
!!     @param[in]  biarc   If biarc==true then we compute dexpzi/dti
!!     @param[in]  bl      If bl==true then we compute dexpzi/dl
!!     @param[in]  lambda  Current homotopic parameter
!!     @param[in]  lpar    Number of optional parameters
!!     @param[in]  par    Optional parameters
!!     @param[in]  ncolv   Number of columns of value
!!
!!     @param[out] zfinal  State and adjoint state at tf
!!     @param[out] value   dzi = dexpzi/dzi
!!                         if biarc==true then dti = dexpzi/dti else dti = [0]
!!                         if bl==true then dl  = dexpzi/dl  else dl  = [0]
!!                         value = [dzi dti dl]
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine dexpzi(t0,tf,n,z0,nbarc,iarc,biarc,zfinal,bl,lambda,lpar,par,ncolv,value)
        use defs
        use utils
        implicit none
        integer,          intent(in)                        :: n,iarc,ncolv,nbarc
        logical,          intent(in)                        :: bl,biarc
        integer,          intent(in)                        :: lpar
        double precision, intent(in)                        :: t0
        double precision, intent(in)                        :: tf
        double precision, intent(in)                        :: lambda
        double precision, intent(in),  dimension(2*n)       :: z0
        double precision, intent(in),  dimension(lpar)      :: par
        double precision, intent(out), dimension(2*n)       :: zfinal
        double precision, intent(out), dimension(2*n,ncolv) :: value

        !local variables
        double precision, dimension(:),   allocatable   :: Yinit,paraux
        double precision, dimension(:),   allocatable   :: Yf,scaletab,Yscalediv,Yscaled
        double precision                                :: tin
        double precision                                :: tfin
        integer                                         :: i,niarc,nl
        integer                                         :: neq
        integer                                         :: flag,cpas
        integer                                         :: ny
        double precision, dimension(2*n)                :: hvz0

        external dexpzirhs

        ny = 2*n
        !initialization
        if(biarc) then
            niarc = 1
        else
            niarc = 0
        end if
        if(bl) then
            nl = 1
        else
            nl = 0
        end if
        tin = t0
        tfin = tf

        neq = ny + ny*ny + niarc*ny + nl*ny
        allocate(Yinit(neq),Yf(neq),scaletab(neq),Yscaled(neq),Yscalediv(neq),paraux(lpar+6+neq))

        paraux(1:lpar) = par
        paraux(lpar+1) = dble(iarc)
        paraux(lpar+2) = dble(nbarc)
        !Initialization of vswitch for switching
        paraux(lpar+3) = dble(ny)
        paraux(lpar+4) = dble(niarc)
        paraux(lpar+5) = dble(nl)
        paraux(lpar+6) = lambda

        !Yinit = z0 for z(t)
        !Yinit = I  for dz/dz0
        !Yinit = -Hv(z0) for dz/dt0
        !Yinit = 0 for dz/dl
        Yinit = 0d0
        Yinit(1:ny) = z0
        do i=1,ny
            Yinit(i*(ny+1)) = 1
        end do
        if(niarc.eq.1) then
            call hvfun(ny,t0,z0,hvz0,paraux(1:lpar+2),lpar+2)
            Yinit(ny+ny*ny+1:ny+ny*ny+ny) = -1d0*hvz0
        end if
        if(nl.eq.1) then
            cpas = neq !step control on all variables if we compute the jacobian of the homotopic function
            ! cpas = ny
        else
            cpas = ny !step control only on z if we compute he jacobian of the shooting function
        end if

        !scaling
        call scaling(neq,Yinit,Yscaled,scaletab,kscalG,scaltolG)
        paraux(lpar+6+1:lpar+6+neq) = scaletab

        !integrate variational equations
        call odesjac(dexpzirhs,neq,Yscaled,paraux,lpar+6+neq,cpas,tin,tfin,Yf,flag)

        !unscaling
        call scalediv(neq,Yf,scaletab,Yscalediv)

        !value = [dzi dti dl]
        value(:,1:ny) = reshape(Yscalediv(ny+1:ny+ny*ny),(/ny,ny/))
        if(niarc.eq.1) then
            value(:,ny+1)    = Yscalediv(ny+ny*ny+1:2*ny+ny*ny)
        else
            value(:,ny+1)    = 0d0
        end if
        if(nl.eq.1) then
            value(:,ny+1+1)  = Yscalediv(neq-ny+1:neq)
        end if

        zfinal = Yscalediv(1:ny)

        deallocate(Yinit,Yf,scaletab,Yscaled,Yscalediv,paraux)

    end subroutine dexpzi

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!
