!!********************************************************************
!!
!!     Subroutine sjac
!!
!>     @ingroup sjacPackage
!!     @brief   Computes the Jacobian of the shooting function.
!!        @param[in]    nbarc     Number of arcs
!!        @param[in]    ti    ti = [t0 t1 .. tm-1 tf]
!!        @param[in]    n     State dimension
!!        @param[in]    zi    State and adjoint state at ti
!!        @param[in]    lpar  Number of optional parameters
!!        @param[in]    par  Optional parameters
!!        @param[in]    nl     if nl=1 then fjac contains the derivatives wrt lambda
!!        @param[in]    lambda  Homotopic parameter lambda
!!       \param[in] ne     Dimension of the shooting function
!!       \param[in] ntf    If 1, then the final time is free and tf is an independent variable
!!
!!        @param[out]   expzi  State and adjoint state at [t1 .. tf]
!!        @param[out]   fjac   Jacobian of the shooting function
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine sjac(nbarc,ti,n,zi,lpar,par,expzi,nl,lambda,ne,ntf,fjac)
        use defs
        use utils
        use derivatives
        implicit none
        integer,          intent(in)                          :: n,nbarc,nl,ne,ntf
        integer,          intent(in)                          :: lpar
        double precision, intent(in)                          :: lambda
        double precision, intent(in),  dimension(nbarc+1)     :: ti
        double precision, intent(in),  dimension(lpar)        :: par
        double precision, intent(in),  dimension(2*n,nbarc)   :: zi
        double precision, intent(out), dimension(2*n,nbarc)   :: expzi
        double precision, intent(out), dimension(ne,ne+nl)    :: fjac

        !local variables
        integer                                           :: i,j,flag,lparaux
        logical                                           :: bl
        double precision, dimension(2*n,nbarc*(2*n+1+nl)) :: vdexpzi
        double precision, dimension(ne,nbarc*2*n)         :: vdedexpzi
        double precision, dimension(ne,lpar)              :: MneGlpar
        double precision, dimension(lpar+2)               :: paraux
        double precision, dimension(ne,nbarc+1)           :: vdedti
        double precision, dimension(2*n)                  :: vdexpzim1dti
        double precision                                  :: eps,h,zero,vswitch,vswitchf
        double precision, dimension(2*n,nbarc)            :: zitemp
        double precision, dimension(1+nbarc)              :: titemp
        double precision, dimension(lpar)                 :: rpartemp
        double precision, dimension(ne)                   :: fvec1,fvec2

        !fjac  = [dS/dz0 dS/dz1 .. dS/dz_nbarc-1 dS/t1 .. dS/t_nbarc-1 dS/dtf dS/dl]
        !ne = 2*n*nbarc+nbarc-1+ntf
        MneGlpar    = 0d0
        lparaux     = lpar+2

        select case (derivativeG)
            case ('finite') !finite differences

                zero = 0.0d0
                eps  = epsilon(1d0)
                eps  = sqrt(eps)

                titemp=ti
                zitemp=zi
                rpartemp=par

                !dS/dzi
                do j=1,nbarc
                    do i=1,2*n
                        h = eps*abs(zi(i,j))
                        if (h .eq. zero) h = eps
                        zitemp(i,j) = zi(i,j) + h
                        call sfun(nbarc,titemp,n,zitemp,lpar,par,expzi,fvec1,flag)
                        zitemp(i,j) = zi(i,j)
                        call sfun(nbarc,titemp,n,zitemp,lpar,par,expzi,fvec2,flag)
                        !zitemp(i,j) = zi(i,j)
                        fjac(:,(j-1)*2*n+i) = (fvec1 - fvec2)/h
                    end do
                end do

                !dS/dti
                do j=2,nbarc
                    h = eps*abs(ti(j))
                    if (h .eq. zero) h = eps
                    titemp(j) = ti(j) + h
                    call sfun(nbarc,titemp,n,zitemp,lpar,par,expzi,fvec1,flag)
                    titemp(j) = ti(j)
                    call sfun(nbarc,titemp,n,zitemp,lpar,par,expzi,fvec2,flag)
                    !titemp(j) = ti(j)
                    fjac(:,nbarc*2*n+j-1) = (fvec1 - fvec2)/h
                end do

                !dS/tf
                if(ntf.eq.1)then
                    j = nbarc+1
                    h = eps*abs(ti(j))
                    if (h .eq. zero) h = eps
                    titemp(j) = ti(j) + h
                    call sfun(nbarc,titemp,n,zitemp,lpar,par,expzi,fvec1,flag)
                    titemp(j) = ti(j)
                    call sfun(nbarc,titemp,n,zitemp,lpar,par,expzi,fvec2,flag)
                    !titemp(j) = ti(j)
                    fjac(:,nbarc*2*n+j-1) = (fvec1 - fvec2)/h
                end if

                !dS/dl
                if(nl.eq.1)then
                    h = eps*abs(lambda)
                    if (h .eq. zero) h = eps
                    call getPar(lambda+h,lpar,rpartemp)
                    call sfun(nbarc,titemp,n,zitemp,lpar,rpartemp,expzi,fvec1,flag)
                    rpartemp = par
                    call sfun(nbarc,titemp,n,zitemp,lpar,rpartemp,expzi,fvec2,flag)
                    rpartemp = par
                    fjac(:,nbarc*2*n+nbarc-1+ntf+1) = (fvec1 - fvec2)/h
                end if

                !the real value of expzi
                call sfun(nbarc,titemp,n,zitemp,lpar,rpartemp,expzi,fvec1,flag)

            case ('eqvar') !variationnal equations


                if(nl.eq.1)then
                    bl = .true.
                else
                    bl = .false.
                end if

                !!!
                !dexpz0 is apart because t0 is fixed and we have no derivative wrt t0
                call dexpzi(ti(1),ti(2),n,zi(:,1),nbarc,1,.false.,expzi(:,1),bl,lambda, &
                            lpar,par,2*n+1+nl,vdexpzi(:,1:2*n+1+nl))

                !i=1,nbarc-1 : dexpzi
                do i=2,nbarc
                    call dexpzi(ti(i),ti(i+1),n,zi(:,i),nbarc,i,.true.,expzi(:,i),bl,lambda, &
                                lpar,par,2*n+1+nl,vdexpzi(:,(i-1)*(2*n+1+nl)+1:i*(2*n+1+nl)))
                end do
                !!!

                !dS/dz0 = de/dz0 + de/dexpz0*dexpz0/dz0
                !de/dz0
                call dedzi(nbarc,n,ti,1,zi,expzi,lpar,par,ne,fjac(:,1:2*n))
                !de/dz0 + de/dexpz0*dexpz0/dz0
                call dedexpzi(nbarc,n,ti,zi,1,expzi,lpar,par,ne,vdedexpzi(:,1:2*n))
                call dgemm('N','N',ne,2*n,2*n,1d0,vdedexpzi(:,1:2*n),ne,                     &
                            vdexpzi(:,1:2*n),2*n,1d0,fjac(:,1:2*n),ne)

                do i=2,nbarc
                    !dS/dzi = de/dzi + de/dexpzi*dexpzi/dzi
                    !de/dzi
                    call dedzi(nbarc,n,ti,i,zi,expzi,lpar,par,ne,fjac(:,(i-1)*2*n+1:i*2*n))
                    !de/dzi + de/dexpzi*dexpzi/dzi
                    call dedexpzi(nbarc,n,ti,zi,i,expzi,lpar,par,ne,vdedexpzi(:,(i-1)*2*n+1:i*2*n))
                    call dgemm('N','N',ne,2*n,2*n,1d0,vdedexpzi(:,(i-1)*2*n+1:i*2*n),ne,    &
                                vdexpzi(:,(i-1)*(2*n+1+nl)+1:(i-1)*(2*n+1+nl)+2*n),2*n,1d0,    &
                                fjac(:,(i-1)*2*n+1:i*2*n),ne)
                end do

                !dS/dt0
                !nothing
                !t0 = ti(1)

                !dS/dti, i=1..nbarc-1
                !dS/dti = de/dti + de/dexpzi*dexpzi/dti + de/dexpzim1*dexpzim1/dti
                paraux(1:lpar) = par
                paraux(lpar+2) = dble(nbarc)
                call dedti(nbarc,n,ti,zi,expzi,lpar,par,ne,vdedti)
                do i=2,nbarc
                    fjac(:,nbarc*2*n+i-1) = vdedti(:,i)
                    call dgemv('N',ne,2*n,1d0,vdedexpzi(:,(i-1)*2*n+1:i*2*n),ne,          &
                                vdexpzi(:,(i-1)*(2*n+1+nl)+2*n+1),1,1d0,fjac(:,nbarc*2*n+i-1),1)
                    paraux(lpar+1) = dble(i-1)

                    call hvfun(2*n,ti(i),expzi(:,i-1),vdexpzim1dti,paraux,lparaux)
                    call dgemv('N',ne,2*n,1d0,vdedexpzi(:,(i-2)*2*n+1:(i-1)*2*n),ne,      &
                                vdexpzim1dti,1,1d0,fjac(:,nbarc*2*n+i-1),1)
                end do

                if(ntf.eq.1)then
                    !de/dtf
                    fjac(:,nbarc*2*n+nbarc) = vdedti(:,nbarc+1)
                    !dexpzim1/dtf
                    paraux(lpar+1) = dble(nbarc)

                    call hvfun(2*n,ti(nbarc+1),expzi(:,nbarc),vdexpzim1dti,paraux,lparaux)
                    !de/dtf + de/dexpzim1*dexpzim1/dtf
                    call dgemv('N',ne,2*n,1d0,vdedexpzi(:,(nbarc-1)*2*n+1:(nbarc)*2*n),ne, &
                                vdexpzim1dti,1,1d0,fjac(:,nbarc*2*n+nbarc),1)
                end if


                if(bl)then
                    !dS/dl = de/dl + sum(de/dexpzi*dexpzi/dl)
                    !de/dl
                    call dedlambda(nbarc,n,ti,zi,expzi,lambda,lpar,par,ne, &
                                    fjac(:,nbarc*2*n+nbarc-1+ntf+nl))

                    do i=1,nbarc
                        !de/dexpzi = vdedexpzi(:,(i-1)*2*n+1:i*2*n)
                        !dexpzi/dl = vdexpzi(:,(i-1)*(2*n+1+nl)+2*n+1+nl)
                        call dgemv('N',ne,2*n,1d0,vdedexpzi(:,(i-1)*2*n+1:i*2*n),ne, &
                                    vdexpzi(:,(i-1)*(2*n+1+nl)+2*n+1+nl),1,1d0,      &
                                    fjac(:,nbarc*2*n+nbarc-1+ntf+nl),1)
                    end do
                end if

            case default

                CALL printandstop('  ||| ERROR: Derivative choice unknown ...')

        end select

    end subroutine sjac

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!

