!!********************************************************************
!!
!!     Subroutine sfunint
!!
!>     @ingroup sjacPackage
!!     @brief  Interface of sjac
!!
!!      Detailed description :
!!        <ul>
!!        <li> Initialize global parameters
!!      <ul>
!!        <li>      call initTol()            ->    variationnal equations tolerances </li>
!!        <li>      call initIntegEqvar()     ->    integrator choice for variationnal equations </li>
!!        <li>      call initMaxSteps()       ->    maximum integrator steps for both var eq and path </li>
!!        <li>      call initMaxStepsSize()   ->    Initialization of maximum steps size </li>
!!        <li>      call initDerivative()     ->    method to derivate </li>
!!        <li>      call initTolScaling()     ->    Initialization of tolerance for auto scaling </li>
!!        </ul></li>
!!        <li> Calling the mere function with some initialization </li>
!!        <ul>
!!        <li> Either we derivate wrt to parameter and (ti,zi) </li>
!!            <ul>
!!            <li> we initialize variables (allocate memory) to tell from which parameters we derivate </li>
!!            <li> we call sjac() the right way </li>
!!            <li> we free memory allocated before </li>
!!            </ul>
!!        <li> Or we derivate only wrt to (ti,zi) -> we call sjac() without any additionnal parameters</li>
!!        </ul>
!!        <li> We do not deal with the flag, which is always 1 </li>
!!        </ul>
!!
!!        @param[in]    nbarc     Number of arcs
!!        @param[in]    ti        Contains t0, tf and the intermediate times if any.
!!                                  ti = [t0 t1 .. t_{nbarc-1} tf]
!!        @param[in]    n         State dimension
!!        @param[in]    zi        State and adjoint state at ti
!!        @param[in]    lpar      Number of optional parameters
!!        @param[in]    par       Optional parameters
!!        @param[in]    lindexpar Number of index parameters
!!        @param[in]    indexpar  Optional parameters
!!       \param[in] nl      Part of the dimension of fjac. nl is 1 if we derivate the homotopic function and 0 if it is the shooting function.
!!       \param[in] ne     Dimension of the shooting function
!!       \param[in] ntf    If 1, then the final time is free and tf is an independent variable
!!        @param[in]    at        Absolute tolerance for integration
!!        @param[in]    rt        Relative tolerance for integration
!!        @param[in]    inte      Name of the integrator
!!        @param[in]    ms        Maximum of integration steps
!!        @param[in]    der       Derivation method
!!        @param[in]    mss       Maximum size of integration steps
!!        @param[in]    tsc        Tolerance for automatic scaling
!!
!!        @param[out]    fjac     Value of the shooting function
!!        @param[out]    flag     Integration output (should be 1)
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine sjacint(nbarc,ti,n,zi,lpar,par,lindexpar,indexpar,nl,ne,ntf,fjac,flag,at,rt,inte,ms,der,mss,tsc)
        use utils
        use init
        implicit none
        integer,            intent(in)                         :: n,nbarc
        integer,            intent(in)                         :: lpar,nl,ne,ntf
        double precision,   intent(in),  dimension(lpar)       :: par
        integer,            intent(in)                         :: lindexpar
        double precision,   intent(in),  dimension(lindexpar)  :: indexpar
        double precision,   intent(in),  dimension(2*n,nbarc)  :: zi
        double precision,   intent(in),  dimension(nbarc+1)    :: ti
        double precision,   intent(out), dimension(ne,ne+nl)   :: fjac
        integer,            intent(out)                        :: flag
        integer,            intent(in)                         :: ms
        double precision,   intent(in)                         :: mss
        double precision,   intent(in)                         :: at
        double precision,   intent(in)                         :: rt
        character(32),      intent(in)                         :: inte
        character(32),      intent(in)                         :: der
        double precision,   intent(in)                         :: tsc

        !local variables
        double precision, dimension(2*n,nbarc)                :: expzi
        double precision, dimension(lpar)                     :: paraux
        integer, dimension(lindexpar)                         :: indexparint
        integer :: i

        call initTol(at,rt)          ! variationnal equations tolerances
        call initIntegEqvar(inte)    ! integrator choice for variationnal equations
        call initMaxSteps(ms)        ! maximum integrator steps for var eq
        call initMaxStepSize(mss)    ! Initialization of maximum steps size
        call initDerivative(der)     ! method to derivate
        call initTolScaling(tsc)     ! Initialization of tolerance for auto scaling

        do i=1,lindexpar
                indexparint(i) = int(indexpar(i))
        end do

        !call the mere function with initialization
        if(lindexpar.gt.0)then !we derivate wrt to parameter and (ti,zi)
                paraux = par
                paraux(indexparint) = paraux(indexparint) + 1d0 !this permit to tell hampath from which parameters we derivate

                call initpar(lpar,par,paraux)               !we initialize par0G, parfG which permits hampath to be aware 
                                                            !from which parameters index it has to derivate

                if(lindexpar.eq.1)then 
                    call sjac(nbarc,ti,n,zi,lpar,par,expzi,1,par(indexparint(1)),ne,ntf,fjac) !if there is just a scalar parameter, we give to sjac
                else
                    call sjac(nbarc,ti,n,zi,lpar,par,expzi,1,0d0,ne,ntf,fjac) !if there is a vector of parameters, then we derivate at lambda = 0, ie at par0G
                endif
                call closePar() !we free the memory allocated
        else !we derivate only wrt to (ti,zi)
                call sjac(nbarc,ti,n,zi,lpar,par,expzi,0,0d0,ne,ntf,fjac) !no additional parameters
        endif

        !we do not deal with the flag
        flag = 1

   end subroutine sjacint

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!

