!!********************************************************************
!!
!!     Subroutine hybrjint
!!
!>     @ingroup ssolvePackage
!!     @brief  hybrj interface.
!!     @param[in] jacfun    gives function for which one wants to find a zero and its Jacobian
!!    \param[in] nx         dimension of x
!!    \param[in] x0i        initial guess
!!    \param[in] lparaux    number of parameters
!!    \param[in] paraux     parameters given to jacfun
!!    \param[in] xtol       tolerance
!!    \param[in] maxfev     maximum number of evaluations of fcn
!!    \param[in] xsol       solution
!!    \param[in] fxsol      evaluation of fcn at the solution
!!    \param[in] nfev       number of evaluations of the function
!!    \param[in] njac       number of evaluations of the Jacobian
!!    \param[in] flag       should be 1
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine hybrjint(jacfun,nx,x0i,lparaux,paraux,xtol,maxfev,xsol,fxsol,nfev,njac,flag)
        implicit none
        integer,            intent(in)                    :: nx
        integer,            intent(in)                    :: lparaux,maxfev
        double precision,   intent(in)                    :: xtol
        double precision,   intent(in), dimension(lparaux):: paraux
        double precision,   intent(in), dimension(nx)     :: x0i
        double precision,   intent(out), dimension(nx)    :: xsol
        double precision,   intent(out), dimension(nx)    :: fxsol
        integer,            intent(out)                   :: nfev
        integer,            intent(out)                   :: njac
        integer,            intent(out)                   :: flag

        interface
            Subroutine jacfun(nx,x,paraux,lparaux,nfev,fvec,fjac,ldfjac,iflag)
            implicit none
            integer,            intent(in)                        :: nx
            integer,            intent(in)                        :: lparaux
            integer,            intent(in)                        :: ldfjac
            integer,            intent(in)                        :: nfev
            double precision,   intent(in),  dimension(lparaux)   :: paraux
            double precision,   intent(in),  dimension(nx)        :: x
            double precision,   intent(out), dimension(nx)        :: fvec
            double precision,   intent(out), dimension(ldfjac,nx) :: fjac
            integer,            intent(inout)                     :: iflag
            end Subroutine jacfun
        end interface

        !local declarations for HYBRJ
        integer                                     :: mode
        integer                                     :: np
        integer                                     :: l
        integer                                     :: ifail
        double precision                            :: fa
        double precision, dimension(nx)             :: diag
        double precision, dimension(nx)             :: qtf
        double precision, dimension(nx)             :: w1
        double precision, dimension(nx)             :: w2
        double precision, dimension(nx)             :: w3
        double precision, dimension(nx)             :: w4
        double precision, dimension((nx*(nx+1))/2)  :: r
        double precision, dimension(nx,nx)          :: fjac

        !HYBRJ solver
        ifail   = -1
        mode    = 1
        fa      = 100d0
        np      = 0
        l       = (nx*(nx+1))/2
        njac    = 0
        nfev    = 0
        w1      = 0d0
        w2      = 0d0
        w3      = 0d0
        w4      = 0d0
        qtf     = 0d0
        fxsol   = 0d0
        diag    = 1d0
        fjac    = 0d0
        r       = 0d0

        xsol    = x0i

        call hybrj(jacfun,nx,xsol,paraux,lparaux,fxsol,fjac,nx,xtol,maxfev, &
                    diag,mode,fa,np,ifail,nfev,njac,r,l,qtf,w1,w2,w3,w4)

        flag    = ifail

    end subroutine hybrjint

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!
