!!********************************************************************
!!
!!     Subroutine sfunjac
!!
!>     @ingroup ssolvePackage
!!     @brief   Subroutine given to hybrj. Call sfun or sjac,
!!              depending on the value iflag.
!!        @param[in]    nx       State and costate dimension
!!        @param[in]    ldfjac   Leading dimension of fjac
!!        @param[in]    x        tate and costate
!!        @param[in]    lparaux     Number of optional parameters
!!        @param[in]    paraux     Optional parameters
!!        @param[in]    nfev     Number of calls to fvec
!!        @param[in]    ldfjac     leading dimension of fjac
!!        @param[in]   iflag    if iflag = 1 then call sfun else call sjac
!!
!!        @param[out]   fvec     Shooting function value
!!        @param[out]   fjac     Shooting function jacobian
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine sfunjac(nx,x,paraux,lparaux,nfev,fvec,fjac,ldfjac,iflag)
        use utils
        implicit none
        integer,          intent(in)                        :: nx
        integer,          intent(in)                        :: lparaux
        integer,          intent(in)                        :: ldfjac
        integer,          intent(in)                        :: nfev
        double precision, intent(in), dimension(lparaux)    :: paraux
        double precision, intent(in), dimension(nx)         :: x
        double precision, intent(out), dimension(nx)        :: fvec
        double precision, intent(out), dimension(ldfjac,nx) :: fjac
        integer,          intent(inout)                     :: iflag

        !Local variables
        double precision, dimension(lparaux-6)        :: par
        double precision, dimension(:,:), allocatable :: zi
        double precision, dimension(:,:), allocatable :: expzi
        double precision, dimension(:)  , allocatable :: ti
        double precision                              :: DNRM2,t0,tf
        integer                                       :: flag
        integer                                       :: n,nbarc,lpar,ne,ntf

        CHARACTER(len=120) :: LINE

        ! iflag = 1 : function at x and return fvec. do not alter fjac.
        ! iflag = 2 : jacobian at x and return fjac. do not alter fvec.

        lpar  = lparaux - 6
        par   = paraux(1:lpar)
        nbarc = nint(paraux(lpar+1))
        n     = nint(paraux(lpar+2))
        ne    = nint(paraux(lpar+3))
        ntf   = nint(paraux(lpar+4))
        t0    =      paraux(lpar+5)
        tf    =      paraux(lpar+6)

        allocate(zi(2*n,nbarc),ti(nbarc+1),expzi(2*n,nbarc))
        call undox(t0,tf,nbarc,n,ti,zi,ne,ntf,x)

        if (iflag == 1) then

            !Shooting Function evaluation
            fvec = 0d0
            call sfun(nbarc,ti,n,zi,lpar,par,expzi,ne,fvec,flag)

            if(displayG.eq.1) then
        WRITE (LINE,'(    i10, e23.15, e23.15, e23.15)') nfev,DNRM2(nx, fvec, 1),DNRM2(n, x(1:n), 1),DNRM2(n, x(n+1:2*n), 1)   
                call myprint(LINE,.true.)
            end if

        elseif(iflag == 2) then

            !Jacobian evaluation
            fjac = 0d0
            call sjac(nbarc,ti,n,zi,lpar,par,expzi,0,-1d0,ne,ntf,fjac)

        end if

        deallocate(zi,ti,expzi)

    end subroutine sfunjac

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!
