!!********************************************************************
!!
!!     Subroutine ssolve
!!
!>     @ingroup ssolvePackage
!!     @brief   Solves systems of nonlinear equations of several variables.
!!        Uses hybrid Powell method (hybrj). fun(x)=0.
!!        \param[in] nbarc         Number of arcs
!!        @param[in]    ti         Contains t0, tf and the intermediate times if any.
!!                                 ti = [t0 t1 .. t_{nbarc-1} tf]
!!        @param[in]    n          State dimension
!!        @param[in]    zi         Initial guess : state and adjoint state at ti
!!        @param[in]    lpar       Number of optional parameters
!!        @param[in]    par        Optional parameters
!!        \param[in]    ne         Dimension of the shooting function
!!        \param[in]    ntf        If 1, then the final time is free and tf is an independent variable
!!
!!        \param[out]   tisol      Final ti solution
!!        \param[out]   zisol      Final zi solution
!!        @param[out]   ssol       Evaluation of sfun() at the solution
!!        @param[out]   nfev       Number of fun evaluation
!!        @param[out]   njev       Number of jacobian evaluation
!!        @param[out]   flag       Solver output (should be 1)
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine ssolve(nbarc,ti,n,zi,lpar,par,tisol,zisol,ne,ntf,ssol,nfev,njev,flag)
        use defs
        use utils
        implicit none
        integer,            intent(in)                        :: n,nbarc,ne,ntf
        integer,            intent(in)                        :: lpar
        double precision,   intent(in),  dimension(lpar)      :: par
        double precision,   intent(in),  dimension(nbarc+1)   :: ti
        double precision,   intent(in),  dimension(2*n,nbarc) :: zi
        double precision,   intent(out), dimension(nbarc+1)   :: tisol
        double precision,   intent(out), dimension(2*n,nbarc) :: zisol
        double precision,   intent(out), dimension(ne)        :: ssol
        integer,            intent(out)                       :: nfev
        integer,            intent(out)                       :: njev
        integer,            intent(out)                       :: flag

        external sfunjac

        !local variables
        integer                                 :: solvemode = 1
        integer                                 :: lparaux,i,j,nx
        CHARACTER(len=120)                      :: LINE
        double precision,  dimension(ne)        :: x0i,xsol
        double precision,  dimension(lpar+6)    :: paraux
        double precision                        :: t0, tf

        !---------------------------------------!
        !--------------- DISPLAY ---------------!
        !---------------------------------------!
        if(displayG.eq.1) then
!            call myprint('---------------------',.false.)
!            call myprint('---------------------',.false.)
!            call myprint('---------------------',.false.)
!            call myprint('-------------',.true.)
            WRITE (LINE,'(a)') '';  call myprint(LINE,.true.)
            WRITE (LINE,'(a)') '     Calls  |S(z)|                 |x(0)|                 |p(0)|                '
            call myprint(LINE,.true.)
            WRITE (LINE,'(a)') '';  call myprint(LINE,.true.)
        end if
        !---------------------------------------!
        !---------------------------------------!

        select case (solvemode)
            case (1)

                !initialization of the parameters for ssolve
                lparaux = lpar+6
                t0      = ti(1)
                tf      = ti(nbarc+1)
                paraux(1:lpar) = par
                paraux(1+lpar) = dble(nbarc)
                paraux(2+lpar) = dble(n)
                paraux(3+lpar) = dble(ne)
                paraux(4+lpar) = dble(ntf)
                paraux(5+lpar) = t0
                paraux(6+lpar) = tf

                !create the unknown for ssolve
                call dox(nbarc,n,ti,zi,ne,ntf,x0i)
                nx = ne

                !call the mere function
                call hybrjint(sfunjac,nx,x0i,lparaux,paraux,xtolG,maxfevG,xsol,ssol,nfev,njev,flag)

                !get the results
                call undox(t0,tf,nbarc,n,tisol,zisol,ne,ntf,xsol)

            case default
                WRITE (LINE,'(a)') '';  call myprint(LINE,.true.)
                write(LINE,'(a)') '  ||| ERROR: Solving -> Unknown solver...',solvemode
                call myprint(LINE,.true.)
                WRITE (LINE,'(a)') '';  call myprint(LINE,.true.)
                stop
        end select

        !---------------------------------------!
        !--------------- DISPLAY ---------------!
        !---------------------------------------!
        if(displayG.eq.1) then

            WRITE (LINE,'(a)') '';  call myprint(LINE,.true.)
            WRITE (LINE,'(a)') ' Results of the shooting method: '; call myprint(LINE,.true.);

            WRITE (LINE,'(a)') '';  call myprint(LINE,.true.)
            WRITE (LINE,'(a)') ' tisol = ['     ; call myprint(LINE,.false.);
            DO i = 1, nbarc+1
                WRITE (LINE,'(e23.15)') tisol(i); call myprint(LINE,.false.)
            END DO
            WRITE (LINE,'(a)') ']'              ; call myprint(LINE,.true.)

            WRITE (LINE,'(a)') '';  call myprint(LINE,.true.)
            WRITE (LINE,'(a)') ' zisol = ['     ; call myprint(LINE,.false.)
            DO i = 1, 2*n

                if(nbarc.eq.1)then
                    if(i.eq.1)then
                        WRITE (LINE,'(e23.15)') zisol(i,1)
                        call myprint(LINE,.true.)
                    elseif(i.eq.2*n)then
                        WRITE (LINE,'("          ",e23.15)') zisol(i,1)
                        call myprint(LINE,.false.)
                    else
                        WRITE (LINE,'("          ",e23.15)') zisol(i,1)
                        call myprint(LINE,.true.)
                    end if
                else
                    DO j=1,nbarc
                        if(j.eq.1)then
                            if(i.eq.1)then
                                WRITE (LINE,'(e23.15)') zisol(i,j)
                            else
                                WRITE (LINE,'("          ",e23.15)') zisol(i,j)
                            end if
                            call myprint(LINE,.false.)
                        elseif(j.eq.nbarc)then
                            if(i.eq.2*n)then
                                WRITE (LINE,'(e23.15)') zisol(i,j)
                                call myprint(LINE,.false.)
                            else
                                WRITE (LINE,'(e23.15)') zisol(i,j)
                                call myprint(LINE,.true.)
                            end if
                        else
                            WRITE (LINE,'(e23.15)') zisol(i,j)
                            call myprint(LINE,.false.)
                        end if
                    END DO
                end if

            END DO
            WRITE (LINE,'(a)') ']'
            call myprint(LINE,.true.)

            WRITE (LINE,'(a)') '';  call myprint(LINE,.true.)
            WRITE (LINE,'(a)') ' s     = ['
            call myprint(LINE,.false.)
            DO i = 1, nx
                if(i.eq.1)then
                    WRITE (LINE,'(e23.15)') ssol(i)
                    call myprint(LINE,.true.)
                elseif(i.eq.nx)then
                    WRITE (LINE,'("          ",e23.15)') ssol(i)
                    call myprint(LINE,.false.)
                else
                    WRITE (LINE,'("          ",e23.15)') ssol(i)
                    call myprint(LINE,.true.)
                end if
            END DO
            WRITE (LINE,'(a)') ']';                       call myprint(LINE,.true.)

            WRITE (LINE,'(a)') '';  call myprint(LINE,.true.)
            WRITE (LINE,'(" nfev   = ",i0.1)') nfev;    call myprint(LINE,.true.)
            WRITE (LINE,'(a)') '';  call myprint(LINE,.true.)
            WRITE (LINE,'(" njev   = ",i0.1)') njev;    call myprint(LINE,.true.)
            WRITE (LINE,'(a)') '';  call myprint(LINE,.true.)
            WRITE (LINE,'(" flag   = ",i0.1)') flag;    call myprint(LINE,.true.)
            WRITE (LINE,'(a)') '';  call myprint(LINE,.true.)


            if(flag.eq.1)then
                WRITE (LINE,'(a)') '';  call myprint(LINE,.true.)
                write(LINE,'(a)') 'Successfully completed : relative error between two consecutive'
                call myprint(LINE,.true.)
                write(LINE,'(a)') '                         iterates is at most xtol.'
            else
                WRITE (LINE,'(a)') '';  call myprint(LINE,.true.)
                call myprint('  ||| WARNING: Solver (hybrj)  ->  ',.false.)
                select case (flag)
                    case (0)
                        WRITE (LINE,'(a)') 'improper input parameters'
                    case (1)
                    case (2)
                        write(LINE,'(a)') 'number of calls to fcn'
                        call myprint(LINE,.true.)
                        write(LINE,'(a)') 'with iflag = 1 has reached maxfev'
                    case (3)
                        write(LINE,'(a)') 'xtol is too small. no further'
                        call myprint(LINE,.true.)
                        write(LINE,'(a)') 'improvement in the approximate solution x is possible'
                    case (4)
                        write(LINE,'(a)') 'iteration is not making good progress,'
                        call myprint(LINE,.true.)
                        write(LINE,'(a)') 'as measured by the improvement from the last five jacobian evaluations'
                    case (5)
                        write(LINE,'(a)') 'iteration is not making good progress,'
                        call myprint(LINE,.true.)
                        write(LINE,'(a)') 'as measured by the improvement from the last ten iterations'
                    case default

                end select
                WRITE (LINE,'(a)') '';  call myprint(LINE,.true.)
            end if
            call myprint(LINE,.true.)
        end if !display
        !---------------------------------------!
        !---------------------------------------!

    end subroutine ssolve

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!

