!!********************************************************************
!!
!!     Subroutine ssolveint
!!
!>     @ingroup ssolvePackage
!!     @brief  Interface of ssolve.
!!
!!      Detailed description :
!!        <ul>
!!        <li> Initialize global parameters
!!      <ul>
!!        <li>      call initTol()            ->    variationnal equations tolerances </li>
!!        <li>      call initTolSolve()       ->    relative tol between iterates </li>
!!        <li>      call initIntegEqvar()     ->    integrator choice for variationnal equations </li>
!!        <li>      call initDerivative()     ->    method to derivate </li>
!!        <li>      call initMaxSteps()       ->    maximum integrator steps for both var eq and path </li>
!!        <li>      call initMaxStepsSize()   ->    Initialization of maximum steps size </li>
!!        <li>      call initDisplaySolve()   ->    display choice </li>
!!        <li>      call initt0GtfG()         ->    global t0 and tf </li>
!!        <li> call initTolScaling()          ->    Initialization of tolerance for auto scaling </li>
!!        </ul></li>
!!        <li> Initialization of the parameters for ssolve </li>
!!        <li> Call to the mere function: call ssolve() </li>
!!        </ul>
!!
!!        @param[in]    nbarc   Number of arcs
!!        @param[in]    ti      Contains t0, tf and the intermediate times if any.
!!                              ti = [t0 t1 .. t_{nbarc-1} tf]
!!        @param[in]    n       State dimension
!!        @param[in]    zi      Initial guess : state and adjoint state at ti
!!        @param[in]    lpar    Number of optional parameters
!!        @param[in]    par     Optional parameters
!!       \param[in] ne     Dimension of the shooting function
!!       \param[in] ntf    If 1, then the final time is free and tf is an independent variable
!!        @param[in]    at      Absolute tolerance for integration
!!        @param[in]    rt      Relative tolerance for integration
!!        @param[in]    xt      xtol value fo hybrj
!!        @param[in]    inte    Name of the integrator
!!        @param[in]    ms      Maximum of integration steps
!!        @param[in]    disp    Display or not the results of the resolution
!!        @param[in]    der     Derivation method
!!        @param[in]    mss     Maximum size of integration steps
!!        @param[in]    tsc     Tolerance for automatic scaling
!!
!!        @param[out]    tisol  Solution : ti solution
!!        @param[out]    zisol  Solution : state and adjoint state at tisol
!!        @param[out]    ssol   Solution : sfun(tisol,zisol)
!!        @param[out]    nfev   Number of sfun evaluation
!!        @param[out]    njev   Number of jacfun evaluation
!!        @param[out]    flag   solver output (should be 1)
!!
!!  \author Olivier Cots
!!  \date   2009-2013
!!  \copyright Eclipse Public License
    Subroutine ssolveint(nbarc,ti,n,zi,lpar,par,tisol,zisol,ne,ntf,ssol,nfev,njev,  &
                           flag,at,rt,xt,inte,ms,disp,der,mss,tsc)
        use utils
        use defs
        use init
        implicit none
        integer,            intent(in)                        :: n,nbarc,ne,ntf
        integer,            intent(in)                        :: lpar
        double precision,   intent(in),  dimension(lpar)      :: par
        double precision,   intent(in),  dimension(nbarc+1)   :: ti
        double precision,   intent(in),  dimension(2*n,nbarc) :: zi
        double precision,   intent(out), dimension(nbarc+1)   :: tisol
        double precision,   intent(out), dimension(2*n,nbarc) :: zisol
        double precision,   intent(out), dimension(ne)        :: ssol
        integer,            intent(out)                       :: nfev
        integer,            intent(out)                       :: njev
        integer,            intent(out)                       :: flag
        integer,            intent(in)                        :: ms
        double precision,   intent(in)                        :: at
        double precision,   intent(in)                        :: rt
        double precision,   intent(in)                        :: xt
        double precision,   intent(in)                        :: mss
        character(32),      intent(in)                        :: disp
        character(32),      intent(in)                        :: inte
        character(32),      intent(in)                        :: der
        double precision,   intent(in)                        :: tsc

        call initTol(at,rt)                  !variationnal equations tolerances
        call initTolSolve(xt)                !relative tol between iterates
        call initIntegEqvar(inte)            !integrator choice for variationnal equations
        call initDerivative(der)             !method to derivate
        call initMaxSteps(ms)                !maximum integrator steps for var eq
        call initMaxStepSize(mss)            !Initialization of maximum steps size
        call initDisplaySolve(disp)          !display choice
        call initTolScaling(tsc)             !Initialization of tolerance for auto scaling

        !call the mere function
        call ssolve(nbarc,ti,n,zi,lpar,par,tisol,zisol,ne,ntf,ssol,nfev,njev,flag)

   end subroutine ssolveint

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!

