!!   @brief Equations, conditions.
!!   The purpose of efun is to code state and costate conditions
!!   @param[in] nbarc  Number of arcs : nbarc = 1 for simple shooting
!!   @param[in] n      State dimension
!!   @param[in] ti     ti = [t0 t1 .. t_nbarc-1 tf], where t0 is the initial
!!                     time and tf the final time.
!!   @param[in] zi     State and adjoint states at [t0 t1 .. t_nbarc-1].
!!   @param[in] expzi  expzi = z(t_i+1;zi,ti) => expzi(:,nbarc) = zf.
!!   @param[in] lpar   Number of optional parameters
!!   @param[in] par    Optional parameters
!!   @param[in] ne     Number of equations and dimension of E
!!   @param[out] E     Equations of the MPBVP
!!   @attention        The size of E depends on tf. If tf is free,
!!                     you can call hfun for an extra condition.
!!                     The vector par can be used for constant values
!!                     or homotopic parameters.
!!   @see \ref hfun
    Subroutine efun(nbarc,n,ti,zi,expzi,lpar,par,ne,E)
        implicit none

        integer, intent(in)                                    :: nbarc
        integer, intent(in)                                    :: n
        integer, intent(in)                                    :: ne
        integer, intent(in)                                    :: lpar
        double precision,  dimension(nbarc+1),   intent(in)    :: ti
        double precision,  dimension(2*n,nbarc), intent(in)    :: zi
        double precision,  dimension(2*n,nbarc), intent(in)    :: expzi
        double precision,  dimension(lpar),      intent(in)    :: par
        double precision,  dimension(ne),        intent(out)   :: E

        !local variables
        double precision :: x0,v0,xf,vf,pv1,mu,hf
        double precision, dimension(2*n) :: z1,expz1,expzf

        IF (ne.NE.10) THEN
            call printandstop('Error: wrong equations dimension.')
        END IF
        IF (nbarc.NE.2) THEN
            call printandstop('Error: wrong number of arcs.')
        END IF
        IF (n.NE.2) THEN
            call printandstop('Error: wrong state dimension.')
        END IF
        IF (lpar.NE.1) THEN
            call printandstop('Error: wrong par dimension.')
        END IF

         x0    =    zi(1,1      )
         v0    =    zi(2,1      )
         xf    = expzi(1,nbarc  )
         vf    = expzi(2,nbarc  )
         z1    =    zi(:,nbarc  )
         pv1   =    z1(2+n      )
         expz1 = expzi(:,nbarc-1)
         expzf = expzi(:,nbarc  )

         call hfun(ti(nbarc+1),n,expzf,nbarc,nbarc,lpar,par,hf)

         E(1)   = x0 + 1d0
         E(2)   = v0 + 1d0
         E(3)   = xf
         E(4)   = vf
         E(5:8) = z1 - expz1
         E(9)   = pv1
         E(10)  = hf

    end subroutine efun

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!
