        module mod_exphvfun
            use mod_options
            implicit none
            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   exphvfun interface.
            !!  \details It is used by fortran user or called by other language interface.
            !!           It makes the link between the user and the core of the program.
            interface exphvfun
                module procedure exphvfunSimple
                module procedure exphvfunMultiple
            end interface
            contains

            !! exphvfunint(n,z0,lpar,par,nt,tspan,nbarc,ti,flag,dimeT,at,rt,inte,ms,mss,tsc)
            !! getstates(n,dimeT,Ts,Ys)

            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   Link to exphvfunint() in the simple shooting case
            !!  \details This subroutine is called through mod_exphvfun::exphvfun interface.
            !!
            !!  \param[in]  tspan   tspan = [tspan1 tspan2 ... tspanf] must be sorted
            !!  \param[in]  z0      initial flow
            !!  \param[in]  options hampath options
            !!  \param[in]  par     parameters given to hfun
            !!  \param[out] tout    time at each integration step
            !!  \param[out] exphv   flow at time tout
            !!  \param[out] flag    flag should be 1 (ODE integrator output)
            !!
            !!  \author Olivier Cots
            !!  \date   2009-2013
            !!  \copyright Eclipse Public License
            Subroutine exphvfunSimple(tspan, z0, options, par, tout, exphv, flag)
                implicit none
                double precision,  dimension(:),        intent(in)  :: tspan
                double precision,  dimension(:),        intent(in)  :: z0
                type(TOptions),                         intent(in)  :: options
                double precision,  dimension(:),        intent(in)  :: par
                double precision,  dimension(:)  , allocatable,     intent(out) :: tout
                double precision,  dimension(:,:), allocatable,     intent(out) :: exphv
                integer,                                            intent(out) :: flag

                !local variable
                integer          :: n,nbarc,lpar,nt,dimeT,ms
                double precision :: ti(2),at,rt,mss,tsc
                character(32)    :: inte,optName

                nt      = size(tspan)
                n       = size(z0,1)/2
                lpar    = size(par)
                nbarc   = 1
                ti(1)   = tspan(1)
                ti(2)   = tspan(nt)

                call getOption(options,'TolOdeAbs'     ,at)
                call getOption(options,'TolOdeRel'     ,rt)
                call getOption(options,'ODE'           ,inte)
                call getOption(options,'MaxStepsOde'   ,ms)
                call getOption(options,'MaxStepSizeOde',mss)
                call getOption(options,'TolScaling'    ,tsc)

                call exphvfunint(n,z0,lpar,par,nt,tspan,nbarc,ti,flag,dimeT,at,rt,inte,ms,mss,tsc)

                allocate(tout(dimeT),exphv(2*n,dimeT))

                call getstates(n,dimeT,tout,exphv)

            end subroutine exphvfunSimple

            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   Link to exphvfunint() in the multiple shooting case
            !!  \details This subroutine is called through mod_exphvfun::exphvfun interface.
            !!
            !!  \param[in]  tspan   tspan = [tspan1 tspan2 ... tspanf] must be sorted
            !!  \param[in]  z0      initial flow
            !!  \param[in]  ti      ti = [t0 t1 ... t_nbarc-1 tf] must be increasing
            !!  \param[in]  options hampath options
            !!  \param[in]  par     parameters given to hfun
            !!  \param[out] tout    time at each integration step
            !!  \param[out] exphv   flow at time tout
            !!  \param[out] flag    flag should be 1 (ODE integrator output)
            !!
            !!  \author Olivier Cots
            !!  \date   2009-2013
            !!  \copyright Eclipse Public License
            Subroutine exphvfunMultiple(tspan, z0, ti, options, par, tout, exphv, flag)
                implicit none
                double precision,  dimension(:),        intent(in)  :: tspan
                double precision,  dimension(:),        intent(in)  :: z0
                double precision,  dimension(:),        intent(in)  :: ti
                type(TOptions),                         intent(in)  :: options
                double precision,  dimension(:),        intent(in)  :: par
                double precision,  dimension(:)  , allocatable,     intent(out) :: tout
                double precision,  dimension(:,:), allocatable,     intent(out) :: exphv
                integer,                                            intent(out) :: flag

                !local variable
                integer          :: n,nbarc,lpar,nt,dimeT,ms
                double precision :: at,rt,mss,tsc
                character(32)    :: inte

                nt      = size(tspan)
                n       = size(z0,1)/2
                lpar    = size(par)
                nbarc   = size(ti)-1

                call getOption(options,'TolOdeAbs'     ,at)
                call getOption(options,'TolOdeRel'     ,rt)
                call getOption(options,'ODE'           ,inte)
                call getOption(options,'MaxStepsOde'   ,ms)
                call getOption(options,'MaxStepSizeOde',mss)
                call getOption(options,'TolScaling'    ,tsc)

                call exphvfunint(n,z0,lpar,par,nt,tspan,nbarc,ti,flag,dimeT,at,rt,inte,ms,mss,tsc)

                allocate(tout(dimeT),exphv(2*n,dimeT))

                call getstates(n,dimeT,tout,exphv)

            end subroutine exphvfunMultiple

      end module mod_exphvfun

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!

