        module mod_hamfun
            implicit none
            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   hamfun interface.
            !!  \details It is used by fortran user or called by other language interface.
            !!           It makes the link between the user and the core of the program.
            interface hamfun
                module procedure hamfunSimple, hamfunSimpleSingle
                module procedure hamfunMultiple, hamfunMultipleSingle
            end interface
            contains

            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   Link to hfunint() in the simple shooting case with a scalar time
            !!  \details This subroutine is called through mod_hamfun::hamfun interface.
            !!
            !!  \param[in]  t       time
            !!  \param[in]  z       state-costate
            !!  \param[in]  par     parameters
            !!  \param[out] h       Hamiltonian
            !!
            !!  \author Olivier Cots
            !!  \date   2009-2013
            !!  \copyright Eclipse Public License
            Subroutine hamfunSimpleSingle(t,z,par,h)
                implicit none
                double precision,                   intent(in)   :: t
                double precision,  dimension(:),    intent(in)   :: z
                double precision,  dimension(:),    intent(in)   :: par
                double precision,                   intent(out)  :: h

                !local variable
                integer          :: n,nbarc,lpar,lvalue
                double precision :: ti(2)
                double precision :: tint(1), zint(size(z),1), hint(1)

                lvalue  = 1
                n       = size(z)/2
                lpar    = size(par)
                nbarc   = 1
                ti(1)   = t
                ti(2)   = t+1d0

                tint(1)     = t
                zint(:,1)   = z

                call hfunint(tint,n,zint,nbarc,ti,lpar,par,lvalue,hint)

                h = hint(1)

            end subroutine hamfunSimpleSingle

            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   Link to hfunint() in the simple shooting case with a vector of times
            !!  \details This subroutine is called through mod_hamfun::hamfun interface.
            !!
            !!  \param[in]  t       time
            !!  \param[in]  z       state-costate
            !!  \param[in]  par     parameters
            !!  \param[out] h       Hamiltonian
            !!
            !!  \author Olivier Cots
            !!  \date   2009-2013
            !!  \copyright Eclipse Public License
            Subroutine hamfunSimple(t,z,par,h)
                implicit none
                double precision,  dimension(:),   intent(in)   :: t
                double precision,  dimension(:,:), intent(in)   :: z
                double precision,  dimension(:),   intent(in)   :: par
                double precision,  dimension(:),   intent(out)  :: h

                !local variable
                integer          :: n,nbarc,lpar,lvalue
                double precision :: ti(2)

                lvalue  = size(t)
                n       = size(z,1)/2
                lpar    = size(par)
                nbarc   = 1
                ti(1)   = t(1)
                ti(2)   = t(lvalue)

                call hfunint(t,n,z,nbarc,ti,lpar,par,lvalue,h)

            end subroutine hamfunSimple

            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   Link to hfunint() in the multiple shooting case with a scalar time
            !!  \details This subroutine is called through mod_hamfun::hamfun interface.
            !!
            !!  \param[in]  t       time
            !!  \param[in]  z       state-costate
            !!  \param[in]  ti      ti = [t0 t1 ... t_nbarc-1 tf]
            !!  \param[in]  par     parameters
            !!  \param[out] h       Hamiltonian
            !!
            !!  \author Olivier Cots
            !!  \date   2009-2013
            !!  \copyright Eclipse Public License
            Subroutine hamfunMultipleSingle(t,z,ti,par,h)
                implicit none
                double precision,                   intent(in)  :: t
                double precision,  dimension(:),    intent(in)  :: z
                double precision,  dimension(:),    intent(in)  :: ti
                double precision,  dimension(:),    intent(in)  :: par
                double precision,                   intent(out) :: h

                !local variable
                integer          :: n,nbarc,lpar,lvalue
                double precision :: tint(1), zint(size(z),1), hint(1)

                lvalue  = 1
                n       = size(z)/2
                lpar    = size(par)
                nbarc   = size(ti)-1

                tint(1)     = t
                zint(:,1)   = z

                call hfunint(tint,n,zint,nbarc,ti,lpar,par,lvalue,hint)

                h = hint(1)

            end subroutine hamfunMultipleSingle

            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   Link to hfunint() in the multiple shooting case with a vector of times
            !!  \details This subroutine is called through mod_hamfun::hamfun interface.
            !!
            !!  \param[in]  t       time
            !!  \param[in]  z       state-costate
            !!  \param[in]  ti      ti = [t0 t1 ... t_nbarc-1 tf]
            !!  \param[in]  par     parameters
            !!  \param[out] h       Hamiltonian
            !!
            !!  \author Olivier Cots
            !!  \date   2009-2013
            !!  \copyright Eclipse Public License
            Subroutine hamfunMultiple(t,z,ti,par,h)
                implicit none
                double precision, intent(in),  dimension(:)     :: t
                double precision, intent(in),  dimension(:,:)   :: z
                double precision, intent(in),  dimension(:)     :: ti
                double precision, intent(in),  dimension(:)     :: par
                double precision, intent(out), dimension(:)     :: h

                !local variable
                integer          :: n,nbarc,lpar,lvalue

                lvalue  = size(t)
                n       = size(z,1)/2
                lpar    = size(par)
                nbarc   = size(ti)-1

                call hfunint(t,n,z,nbarc,ti,lpar,par,lvalue,h)

            end subroutine hamfunMultiple

      end module mod_hamfun

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!
