       module mod_hampath
            use utils
            use mod_options
            implicit none
            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   hampath interface.
            !!  \details It is used by fortran user or called by other language interface.
            !!           It makes the link between the user and the core of the program.
            interface hampath
                module procedure hampathz0
                module procedure hampathzi
            end interface
            contains

            !! hampathint(  nbarc,ti,n,zi,nparspan,lparspan,rparspan,ne,ntf, &
            !!                tif,zif,parf,dimepath,dimebif,flag,            &
            !!                iwork,dwork,der,disp,inte,inth,sapa)

            !! getbif(n,ne,dimebif,bifctab,biftoctab) not called
            !! getpath(t0,tf,m,n,ne,ntf,dimepath,lpar,parout,tis,zis,tocis,ss,det,norms,ps)

            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   Link to hampathint() in the simple shooting case
            !!  \details This subroutine is called through mod_hampath::hampath interface.
            !!
            !!  \param[in]  parspan parspan = [par0 parf]. parspan(:,i) is a parameter
            !!                      vector given to hfun and efun. The homotopy is from
            !!                      par0 to parf and of the form : (1-lambda)*par0 + lambda*parf,
            !!                      unless a fortran file with the subroutine
            !!                      parfun(lambda,lpar,par0,parf,parout), with lambda from 0 to 1, 
            !!                      is provided.
            !!  \param[in]  ti      ti = [t0 t1 ... t_nbarc-1 tf], where nbarc is the number of arcs
            !!  \param[in]  z0      initial flow
            !!  \param[in]  options hampath options
            !!  \param[out] parout  parameters at each integration step
            !!  \param[out] tiout   tiout((k-1)*(nbarc+1):k*(nbarc+1)) = ti at parout(:,k)
            !!  \param[out] ziout   zout(:,(k-1)*nbarc:k*nbarc) =  zi at parout(:,k)
            !!  \param[out] sout    arc length at each integration step
            !!  \param[out] viout   viout(:,k) = dot_c(:,k): tangent vector at parout(:,k)
            !!                      where c(:,k) = [z0 ... z_{nbarc-1} t1 ... t_{nbarc-1} tf lambda]',
            !!                      no tf if it is fixed
            !!  \param[out] dets    det(h'(c(s)),c'(s)) at each integration step, where
            !!                      h is the homotpic function
            !!  \param[out] normS   Norm of the shooting function at each integration step
            !!  \param[out] ps      <c'(s_old),c'(s)> at each integreation step
            !!  \param[out] flag    flag should be 1 (ODE integrator output) <br>
            !!                      if flag==0  then Sfmax is too small <br>
            !!                      if flag==-1 then input is not consistent <br>
            !!                      if flag==-2 then larger MaxSteps is needed <br>
            !!                      if flag==-3 then step size became too small <br>
            !!                      if flag==-4 then problem is problably stiff <br>
            !!                      if flag==-5 then |S| is greater than NormeSfunMax <br>
            !!                      if flag==-6 then the homotopic parameter do not
            !!                          evolve relatively wrt TolDenseHamX <br>
            !!                      if flag==-7 then a turning point occurs
            !!
            !!  \author Olivier Cots
            !!  \date   2009-2013
            !!  \copyright Eclipse Public License
            Subroutine hampathz0(parspan,ti,z0,options,parout,tiout,ziout,sout,viout,dets,normS,ps,flag)
                implicit none
                double precision,  dimension(:,:),              intent(in)  :: parspan
                double precision,  dimension(:),                intent(in)  :: ti
                double precision,  dimension(:),                intent(in)  :: z0
                type(TOptions),                                 intent(in)  :: options
                double precision,  dimension(:,:), allocatable, intent(out) :: parout
                double precision,  dimension(:),   allocatable, intent(out) :: tiout
                double precision,  dimension(:,:), allocatable, intent(out) :: ziout
                double precision,  dimension(:),   allocatable, intent(out) :: sout
                double precision,  dimension(:,:), allocatable, intent(out) :: viout
                double precision,  dimension(:),   allocatable, intent(out) :: dets
                double precision,  dimension(:),   allocatable, intent(out) :: normS
                double precision,  dimension(:),   allocatable, intent(out) :: ps
                integer,                                        intent(out) :: flag

                !local variable
                integer          :: n,nbarc,nparspan,lparspan,ne,ntf,dimepath,dimebif,iwork(4)
                double precision :: dwork(11),zi(size(z0,1),1),t0,tf
                character(32)    :: optName,fft,der,disp,inte,inth,sapa

                n        = size(z0,1)/2
                lparspan = size(parspan,1)
                nparspan = size(parspan,2)
                nbarc    = 1
                zi(:,1)  = z0
                t0       = ti(1)
                tf       = ti(nbarc+1)

                call getOption(options,'Derivative'    ,der)
                call getOption(options,'Display'       ,disp)
                call getOption(options,'DoSavePath'    ,sapa)
                call getOption(options,'FreeFinalTime' ,fft)
                call getOption(options,'ODE'           ,inte)
                call getOption(options,'ODEHam'        ,inth)
                
                call getOption(options,'Dispiter'           ,iwork(1))
                call getOption(options,'MaxStepsOde'        ,iwork(2))
                call getOption(options,'MaxStepsOdeHam'     ,iwork(3))
                call getOption(options,'StopAtTurningPoint' ,iwork(4))
                
                call getOption(options,'MaxSf'              ,dwork(1))
                call getOption(options,'MaxSfunNorm'        ,dwork(2))
                call getOption(options,'MaxStepSizeOde'     ,dwork(3))
                call getOption(options,'MaxStepSizeOdeHam'  ,dwork(4))
                call getOption(options,'TolOdeAbs'          ,dwork(5))
                call getOption(options,'TolOdeRel'          ,dwork(6))
                call getOption(options,'TolOdeHamAbs'       ,dwork(7))
                call getOption(options,'TolOdeHamRel'       ,dwork(8))
                call getOption(options,'TolDenseHamEnd'     ,dwork(9))
                call getOption(options,'TolDenseHamX'       ,dwork(10))
                call getOption(options,'TolScaling'         ,dwork(11))

                call initTtfNe(fft,nbarc,n,ntf,ne)
                call hampathint(nbarc,ti,n,zi,lparspan,nparspan,parspan,ne,ntf, &
                                dimepath,dimebif,flag,                          &
                                iwork,dwork,der,disp,inte,inth,sapa)

                allocate(parout(lparspan,dimepath))
                allocate(tiout((nbarc+1)*dimepath))
                allocate(ziout(2*n,nbarc*dimepath))
                allocate(sout(dimepath))
                allocate(viout(ne+1,dimepath))
                allocate(dets(dimepath))
                allocate(normS(dimepath))
                allocate(ps(dimepath))
                
                call getpath(t0,tf,nbarc,n,ne,ntf,dimepath,lparspan,parout,tiout,ziout,viout,sout,dets,normS,ps)

            end subroutine hampathz0

            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   Link to hampathint() in the simple shooting case
            !!  \details This subroutine is called through mod_hampath::hampath interface.
            !!
            !!  \param[in]  parspan parspan = [par0 parf]. parspan(:,i) is a parameter
            !!                      vector given to hfun and efun. The homotopy is from
            !!                      par0 to parf and of the form : (1-lambda)*par0 + lambda*parf,
            !!                      unless a fortran file with the subroutine
            !!                      parfun(lambda,lpar,par0,parf,parout), with lambda from 0 to 1, 
            !!                      is provided.
            !!  \param[in]  ti      ti = [t0 tf]
            !!  \param[in]  zi      zi = [z(t0) ... z(t_{nbarc-1})]
            !!  \param[in]  options hampath options
            !!  \param[out] parout  parameters at each integration step
            !!  \param[out] tiout   tiout((k-1)*(nbarc+1):k*(nbarc+1)) = ti at parout(:,k)
            !!  \param[out] ziout   zout(:,(k-1)*nbarc:k*nbarc) =  zi at parout(:,k)
            !!  \param[out] sout    arc length at each integration step
            !!  \param[out] viout   viout(:,k) = dot_c(:,k): tangent vector at parout(:,k)
            !!                      where c(:,k) = [z0 ... z_{nbarc-1} t1 ... t_{nbarc-1} tf lambda]',
            !!                      no tf if it is fixed
            !!  \param[out] dets    det(h'(c(s)),c'(s)) at each integration step, where
            !!                      h is the homotpic function
            !!  \param[out] normS   Norm of the shooting function at each integration step
            !!  \param[out] ps      <c'(s_old),c'(s)> at each integreation step
            !!  \param[out] flag    flag should be 1 (ODE integrator output) <br>
            !!                      if flag==0  then Sfmax is too small <br>
            !!                      if flag==-1 then input is not consistent <br>
            !!                      if flag==-2 then larger MaxSteps is needed <br>
            !!                      if flag==-3 then step size became too small <br>
            !!                      if flag==-4 then problem is problably stiff <br>
            !!                      if flag==-5 then |S| is greater than NormeSfunMax <br>
            !!                      if flag==-6 then the homotopic parameter do not
            !!                          evolve relatively wrt TolDenseHamX <br>
            !!                      if flag==-7 then a turning point occurs
            !!
            !!  \author Olivier Cots
            !!  \date   2009-2013
            !!  \copyright Eclipse Public License
            Subroutine hampathzi(parspan,ti,zi,options,parout,tiout,ziout,sout,viout,dets,normS,ps,flag)
                implicit none
                double precision,  dimension(:,:),              intent(in)  :: parspan
                double precision,  dimension(:),                intent(in)  :: ti
                double precision,  dimension(:,:),              intent(in)  :: zi
                type(TOptions),                                 intent(in)  :: options
                double precision,  dimension(:,:), allocatable, intent(out) :: parout
                double precision,  dimension(:),   allocatable, intent(out) :: tiout
                double precision,  dimension(:,:), allocatable, intent(out) :: ziout
                double precision,  dimension(:),   allocatable, intent(out) :: sout
                double precision,  dimension(:,:), allocatable, intent(out) :: viout
                double precision,  dimension(:),   allocatable, intent(out) :: dets
                double precision,  dimension(:),   allocatable, intent(out) :: normS
                double precision,  dimension(:),   allocatable, intent(out) :: ps
                integer,                                        intent(out) :: flag

                !local variable
                integer          :: n,nbarc,nparspan,lparspan,ne,ntf,dimepath,dimebif,iwork(4)
                double precision :: dwork(11),t0,tf
                character(32)    :: optName,fft,der,disp,inte,inth,sapa

                n        = size(zi,1)/2
                lparspan = size(parspan,1)
                nparspan = size(parspan,2)
                nbarc    = size(zi,2)
                t0       = ti(1)
                tf       = ti(nbarc+1)

                call getOption(options,'Derivative'    ,der)
                call getOption(options,'Display'       ,disp)
                call getOption(options,'DoSavePath'    ,sapa)
                call getOption(options,'FreeFinalTime' ,fft)
                call getOption(options,'ODE'           ,inte)
                call getOption(options,'ODEHam'        ,inth)
                
                call getOption(options,'Dispiter'           ,iwork(1))
                call getOption(options,'MaxStepsOde'        ,iwork(2))
                call getOption(options,'MaxStepsOdeHam'     ,iwork(3))
                call getOption(options,'StopAtTurningPoint' ,iwork(4))
                
                call getOption(options,'MaxSf'              ,dwork(1))
                call getOption(options,'MaxSfunNorm'        ,dwork(2))
                call getOption(options,'MaxStepSizeOde'     ,dwork(3))
                call getOption(options,'MaxStepSizeOdeHam'  ,dwork(4))
                call getOption(options,'TolOdeAbs'          ,dwork(5))
                call getOption(options,'TolOdeRel'          ,dwork(6))
                call getOption(options,'TolOdeHamAbs'       ,dwork(7))
                call getOption(options,'TolOdeHamRel'       ,dwork(8))
                call getOption(options,'TolDenseHamEnd'     ,dwork(9))
                call getOption(options,'TolDenseHamX'       ,dwork(10))
                call getOption(options,'TolScaling'         ,dwork(11))

                call initTtfNe(fft,nbarc,n,ntf,ne)
                call hampathint(nbarc,ti,n,zi,lparspan,nparspan,parspan,ne,ntf, &
                                dimepath,dimebif,flag,                          &
                                iwork,dwork,der,disp,inte,inth,sapa)

                allocate(parout(lparspan,dimepath))
                allocate(tiout((nbarc+1)*dimepath))
                allocate(ziout(2*n,nbarc*dimepath))
                allocate(sout(dimepath))
                allocate(viout(ne+1,dimepath))
                allocate(dets(dimepath))
                allocate(normS(dimepath))
                allocate(ps(dimepath))
                
                call getpath(t0,tf,nbarc,n,ne,ntf,dimepath,lparspan,parout,tiout,ziout,viout,sout,dets,normS,ps)

            end subroutine hampathzi

      end module mod_hampath

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!




