!% hvfun -- Vector field associated to H (needs hfun.f90)
!%
!% Usage
!%  hv = hvfun(t, z,     par) : single arc
!%  hv = hvfun(t, z, ti, par) : multiple arcs
!%
!% Inputs
!%  t    -  real row vector, t(i) = i-th time
!%  z    -  real matrix, z(:,i) = i-th state and costate
!%  ti   -  real row vector, in multiple shooting case 
!%          ti = [t0 t1 ... t_nbarc-1 tf]
!%  par  -  real row vector, parameters given to hfun
!%          par=[] if no parameters
!%
!% Outputs
!%  hv   -  real matrix, vector H at time t
!%
!% Description
!%  Computes the hviltonian vector field associated to H.
!%
        module mod_hvfun
            implicit none
            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   hvfun interface.
            !!  \details It is used by fortran user or called by other language interface.
            !!           It makes the link between the user and the core of the program.
            interface hvfun
                module procedure hvfunSimple, hvfunSimpleSingle
                module procedure hvfunMultiple, hvfunMultipleSingle
            end interface
            contains

            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   Link to hvfunint() in the simple shooting case with a scalar time
            !!  \details This subroutine is called through mod_hvfun::hvfun interface.
            !!
            !!  \param[in]  t       time
            !!  \param[in]  z       state-costate
            !!  \param[in]  par     parameters
            !!  \param[out] hv      Hamiltonian vector field at time t
            !!
            !!  \author Olivier Cots
            !!  \date   2009-2013
            !!  \copyright Eclipse Public License
            Subroutine hvfunSimpleSingle(t,z,par,hv)
                implicit none
                double precision,                   intent(in)   :: t
                double precision,  dimension(:),    intent(in)   :: z
                double precision,  dimension(:),    intent(in)   :: par
                double precision,  dimension(:),    intent(out)  :: hv

                !local variable
                integer          :: n,nbarc,lpar,lvalue
                double precision :: ti(2)
                double precision :: tint(1), zint(size(z),1)
                double precision :: hvint(size(z),1)

                lvalue  = 1
                n       = size(z)/2
                lpar    = size(par)
                nbarc   = 1
                ti(1)   = t
                ti(2)   = t+1d0

                tint(1)     = t
                zint(:,1)   = z

                call hvfunint(tint,n,zint,nbarc,ti,lpar,par,lvalue,hvint)

                hv = hvint(:,1)

            end subroutine hvfunSimpleSingle

            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   Link to hvfunint() in the simple shooting case with a vector of times
            !!  \details This subroutine is called through mod_hvfun::hvfun interface.
            !!
            !!  \param[in]  t       time
            !!  \param[in]  z       state-costate
            !!  \param[in]  par     parameters
            !!  \param[out] hv      Hamiltonian vector field at time t
            !!
            !!  \author Olivier Cots
            !!  \date   2009-2013
            !!  \copyright Eclipse Public License
            Subroutine hvfunSimple(t,z,par,hv)
                implicit none
                double precision,  dimension(:),   intent(in)   :: t
                double precision,  dimension(:,:), intent(in)   :: z
                double precision,  dimension(:),   intent(in)   :: par
                double precision,  dimension(:,:), intent(out)  :: hv

                !local variable
                integer          :: n,nbarc,lpar,lvalue
                double precision :: ti(2)

                lvalue  = size(t)
                n       = size(z,1)/2
                lpar    = size(par)
                nbarc   = 1
                ti(1)   = t(1)
                ti(2)   = t(lvalue)

                call hvfunint(t,n,z,nbarc,ti,lpar,par,lvalue,hv)

            end subroutine hvfunSimple

            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   Link to hvfunint() in the multiple shooting case with a scalar time
            !!  \details This subroutine is called through mod_hvfun::hvfun interface.
            !!
            !!  \param[in]  t       time
            !!  \param[in]  z       state-costate
            !!  \param[in]  ti      ti = [t0 t1 ... t_nbarc-1 tf]
            !!  \param[in]  par     parameters
            !!  \param[out] hv      Hamiltonian vector field at time t
            !!
            !!  \author Olivier Cots
            !!  \date   2009-2013
            !!  \copyright Eclipse Public License
            Subroutine hvfunMultipleSingle(t,z,ti,par,hv)
                implicit none
                double precision,                   intent(in)  :: t
                double precision,  dimension(:),    intent(in)  :: z
                double precision,  dimension(:),    intent(in)  :: ti
                double precision,  dimension(:),    intent(in)  :: par
                double precision,  dimension(:),    intent(out) :: hv

                !local variable
                integer          :: n,nbarc,lpar,lvalue
                double precision :: tint(1), zint(size(z),1)
                double precision :: hvint(size(z),1)

                lvalue  = 1
                n       = size(z)/2
                lpar    = size(par)
                nbarc   = size(ti)-1

                tint(1)     = t
                zint(:,1)   = z

                call hvfunint(tint,n,zint,nbarc,ti,lpar,par,lvalue,hvint)

                hv = hvint(:,1)

            end subroutine hvfunMultipleSingle

            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   Link to hvfunint() in the multiple shooting case with a vector of times
            !!  \details This subroutine is called through mod_hvfun::hvfun interface.
            !!
            !!  \param[in]  t       time
            !!  \param[in]  z       state-costate
            !!  \param[in]  ti      ti = [t0 t1 ... t_nbarc-1 tf]
            !!  \param[in]  par     parameters
            !!  \param[out] hv      Hamiltonian vector field at time t
            !!
            !!  \author Olivier Cots
            !!  \date   2009-2013
            !!  \copyright Eclipse Public License
            Subroutine hvfunMultiple(t,z,ti,par,hv)
                implicit none
                double precision, intent(in),  dimension(:)     :: t
                double precision, intent(in),  dimension(:,:)   :: z
                double precision, intent(in),  dimension(:)     :: ti
                double precision, intent(in),  dimension(:)     :: par
                double precision, intent(out), dimension(:,:)   :: hv

                !local variable
                integer          :: n,nbarc,lpar,lvalue

                lvalue  = size(t)
                n       = size(z,1)/2
                lpar    = size(par)
                nbarc   = size(ti)-1

                call hvfunint(t,n,z,nbarc,ti,lpar,par,lvalue,hv)

            end subroutine hvfunMultiple

      end module mod_hvfun

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!
