!% hampathset -- Sets options for hampath packages.
!%
!% Usage
!%  options = hampathset(name1, value1, ...)
!%
!% Or
!%  options = hampathset(oldopts, name1, value1,...) 
!%  creates a copy of oldopts with the named parameters altered
!%  with the specified values.
!%
!% Inputs
!%  name1   string, option name
!%  value1  any, option value
!%  ...
!%
!% Outputs
!%  options struct, options
!%
!% Description - options names and default values are:
!%
!%  Derivative       - Derivation method.
!%                     Names available                    ['finite',{'eqvar'}]
!%                     'finite' corresponds to finite differences.
!%                     'eqvar'  corresponds to variational equations.
!%                     Used by hampath, sjac, ssolve.
!%
!%  DispIter         - Display each mod(iter,DispIter)==0            [     1 ]
!%                     Used by hampath.
!%
!%  Display          - Display results                       [ 'off' | {'on'}]
!%                     Used by hampath, ssolve.
!%
!%  DoSavePath       - Save path in file "SavedPath(n).m" during continuation
!%                                                           [ {'off'} | 'on']
!%                     Used by hampath.
!%
!%  FreeFinalTime    - Fixed or free final time           ['true' | {'false'}]
!%                     Used by hampath, sfun, sjac, ssolve.
!%
!%  MaxSf            - Maximum final arc length for hampath          [   1e5 ]
!%                     Used by hampath.
!%
!%  MaxSfunNorm      - Maximal norm of the shooting function during homotopy.
!%                     Used by hampath                               [   1e-1]
!%
!%  MaxStepsOde      - Maximum number of steps of the intergrator    [   1e5 ]
!%                     Used by exp[d]hvfun, hampath, sfun, sjac, ssolve.
!%
!%  MaxStepsOdeHam   - Maximum number of steps of the homotopy       [   1e5 ]
!%                     Used by hampath.
!%
!%  MaxStepSizeOde   - Maximum step size                             [     0 ]
!%                     If equals 0 then the adaptative step size integrator
!%                     has no constraint on the maximal step size.
!%                     Used by exp[d]hvfun, hampath, sfun, sjac, ssolve.
!%
!%  MaxStepSizeOdeHam- Maximum step size during homotopy             [     0 ]
!%                     If equals 0 then the adaptative step size integrator
!%                     has no constraint on the maximal step size.
!%                     Used by hampath.
!%
!%  ODE              - Integrator name.
!%                     ODE names available                         ['dopri5' ]
!%                     Used by exp[d]hvfun, hampath, sfun, sjac, ssolve.
!%
!%  ODEHam           - Integrator name for homotopy.
!%                     ODE names available                         ['dopri5' ]
!%                     Used by hampath.
!%
!%  StopAtTurningPoint - Stop or not after turning point             [{0} | 1]
!%                     Used by hampath.
!%
!%  TolOdeAbs        - Absolute error tolerance of the integrator    [ 1e-14 ]
!%                     Used by exp[d]hvfun, hampath, sfun, sjac, ssolve.
!%
!%  TolOdeRel        - Relative error tolerance of the integrator    [  1e-8 ]
!%                     Used by exp[d]hvfun, hampath, sfun, sjac, ssolve.
!%
!%  TolOdeHamAbs     - Absolute error tolerance for hampath          [ 1e-14 ]
!%                     Used by hampath.
!%
!%  TolOdeHamRel     - Relative error tolerance for hampath          [  1e-8 ]
!%                     Used by hampath.
!%
!%  TolDenseHamEnd   - Absolute Dense output tolerance               [  1e-8 ]
!%                     Absolute tolerance to detect if the final homotopic
!%                     parameter has been reached.
!%                     Used by hampath.
!%
!%  TolDenseHamX     - Relative Dense output tolerance               [  1e-8 ]
!%                     The homotopy stops when the homotopic parameter do not
!%                     evolve relatively, iteration per iteration, during ten 
!%                     following steps.
!%                     Used by hampath.
!%
!%  TolScaling       - Automatic scaling between 0 and 1 for initial absolute
!%                     values greater than TolScaling                [  1e-5 ]
!%
!%  TolX             - Relative tolerance between iterates (ssolve)  [  1e-8 ]
!%                     Used by ssolve.
!%
!%
!%  See also
!%  hampathget
!%
module mod_options
    use utils
    implicit none

    !!-------------------------------------------------------!!
    !!-------------------------------------------------------!!
    !>  @ingroup modulesInterfaces
    !!  @brief   Options structure.
    !!
    !!  \author Olivier Cots
    !!  \date   2009-2013
    !!  \copyright Eclipse Public License
    type TOptions
        !>  @ingroup modulesInterfaces
        !! \brief see \ref defs
        !! \memberof mod_options::toptions
        character(32)       :: Derivative           = 'eqvar'
        !>  @ingroup modulesInterfaces
        !! \brief see \ref defs
        !! \memberof mod_options::toptions
        integer             :: Dispiter             = 1
        !>  @ingroup modulesInterfaces
        !! \brief see \ref defs
        !! \memberof mod_options::toptions
        character(32)       :: Display              = 'on'
        !>  @ingroup modulesInterfaces
        !! \brief see \ref defs
        !! \memberof mod_options::toptions
        character(32)       :: DoSavePath           = 'off'
        !>  @ingroup modulesInterfaces
        !! \brief see \ref defs
        !! \memberof mod_options::toptions
        character(32)       :: FreeFinalTime        = 'false'
        !>  @ingroup modulesInterfaces
        !! \brief see \ref defs
        !! \memberof mod_options::toptions
        double precision    :: MaxSf                = 1d5
        !>  @ingroup modulesInterfaces
        !! \brief see \ref defs
        !! \memberof mod_options::toptions
        double precision    :: MaxSfunNorm          = 1d-1
        !>  @ingroup modulesInterfaces
        !! \brief see \ref defs
        !! \memberof mod_options::toptions
        integer             :: MaxStepsOde          = 1d5
        !>  @ingroup modulesInterfaces
        !! \brief see \ref defs
        !! \memberof mod_options::toptions
        integer             :: MaxStepsOdeHam       = 1d5
        !>  @ingroup modulesInterfaces
        !! \brief see \ref defs
        !! \memberof mod_options::toptions
        double precision    :: MaxStepSizeOde       = 0d0
        !>  @ingroup modulesInterfaces
        !! \brief see \ref defs
        !! \memberof mod_options::toptions
        double precision    :: MaxStepSizeOdeHam    = 0d0
        !>  @ingroup modulesInterfaces
        !! \brief see \ref defs
        !! \memberof mod_options::toptions
        character(32)       :: ODE                  = 'dopri5'
        !>  @ingroup modulesInterfaces
        !! \brief see \ref defs
        !! \memberof mod_options::toptions
        character(32)       :: ODEHam               = 'dopri5'
        !>  @ingroup modulesInterfaces
        !! \brief see \ref defs
        !! \memberof mod_options::toptions
        integer             :: StopAtTurningPoint   = 0
        !>  @ingroup modulesInterfaces
        !! \brief see \ref defs
        !! \memberof mod_options::toptions
        double precision    :: TolOdeAbs            = 1d-14
        !>  @ingroup modulesInterfaces
        !! \brief see \ref defs
        !! \memberof mod_options::toptions
        double precision    :: TolOdeRel            = 1d-8
        !>  @ingroup modulesInterfaces
        !! \brief see \ref defs
        !! \memberof mod_options::toptions
        double precision    :: TolOdeHamAbs         = 1d-14
        !>  @ingroup modulesInterfaces
        !! \brief see \ref defs
        !! \memberof mod_options::toptions
        double precision    :: TolOdeHamRel         = 1d-8
        !>  @ingroup modulesInterfaces
        !! \brief see \ref defs
        !! \memberof mod_options::toptions
        double precision    :: TolDenseHamEnd       = 1d-8
        !>  @ingroup modulesInterfaces
        !! \brief see \ref defs
        !! \memberof mod_options::toptions
        double precision    :: TolDenseHamX         = 1d-8
        !>  @ingroup modulesInterfaces
        !! \brief see \ref defs
        !! \memberof mod_options::toptions
        double precision    :: TolScaling           = 1d-5
        !>  @ingroup modulesInterfaces
        !! \brief see \ref defs
        !! \memberof mod_options::toptions
        double precision    :: TolX                 = 1d-8
    end type TOptions

    !!-------------------------------------------------------!!
    !!-------------------------------------------------------!!
    !>  @ingroup modulesInterfaces
    !!  @brief   getOption interface.
    !!  \details It is used by fortran user or called by other language interface.
    !!           It makes the link between the user and the core of the program.
    !!
    !!  \author Olivier Cots
    !!  \date   2009-2013
    !!  \copyright Eclipse Public License
    interface getOption
        module procedure getInt, getReal, getChar
    end interface

    !!-------------------------------------------------------!!
    !!-------------------------------------------------------!!
    !>  @ingroup modulesInterfaces
    !!  @brief   setOption interface.
    !!  \details It is used by fortran user or called by other language interface.
    !!           It makes the link between the user and the core of the program.
    !!
    !!  \author Olivier Cots
    !!  \date   2009-2013
    !!  \copyright Eclipse Public License
    interface setOption
        module procedure setInt, setReal, setChar
    end interface

    contains

    !!-------------------------------------------------------!!
    !!-------------------------------------------------------!!
    !>  @ingroup modulesInterfaces
    !!  @brief  Print options.
    !!  @param[in] options options to be printed
    !!
    !!  \author Olivier Cots
    !!  \date   2009-2013
    !!  \copyright Eclipse Public License
    subroutine printOptions(options)
        type(TOptions), intent(in) :: options

        !local variables
        CHARACTER(len=120)                  :: LINE

        WRITE (LINE,'(a)')  'Options :'; call myprint(LINE,.true.);

        WRITE (LINE,'("  Derivative         = ",     a)')  options%Derivative;          call myprint(LINE,.true.)
        WRITE (LINE,'("  Dispiter           = ",  i0.1)')  options%Dispiter;            call myprint(LINE,.true.)
        WRITE (LINE,'("  Display            = ",     a)')  options%Display;             call myprint(LINE,.true.)
        WRITE (LINE,'("  DoSavePath         = ",     a)')  options%DoSavePath;          call myprint(LINE,.true.)
        WRITE (LINE,'("  FreeFinalTime      = ",     a)')  options%FreeFinalTime;       call myprint(LINE,.true.)
        WRITE (LINE,'("  MaxSf              = ",e23.15)')  options%MaxSf;               call myprint(LINE,.true.)
        WRITE (LINE,'("  MaxSfunNorm        = ",e23.15)')  options%MaxSfunNorm;         call myprint(LINE,.true.)
        WRITE (LINE,'("  MaxStepsOde        = ",  i0.1)')  options%MaxStepsOde;         call myprint(LINE,.true.)
        WRITE (LINE,'("  MaxStepsOdeHam     = ",  i0.1)')  options%MaxStepsOdeHam;      call myprint(LINE,.true.)
        WRITE (LINE,'("  MaxStepSizeOde     = ",e23.15)')  options%MaxStepSizeOde;      call myprint(LINE,.true.)
        WRITE (LINE,'("  MaxStepSizeOdeHam  = ",e23.15)')  options%MaxStepSizeOdeHam;   call myprint(LINE,.true.)
        WRITE (LINE,'("  ODE                = ",     a)')  options%ODE;                 call myprint(LINE,.true.)
        WRITE (LINE,'("  ODEHam             = ",     a)')  options%ODEHam;              call myprint(LINE,.true.)
        WRITE (LINE,'("  StopAtTurningPoint = ",  i0.1)')  options%StopAtTurningPoint;  call myprint(LINE,.true.)
        WRITE (LINE,'("  TolOdeAbs          = ",e23.15)')  options%TolOdeAbs;           call myprint(LINE,.true.)
        WRITE (LINE,'("  TolOdeRel          = ",e23.15)')  options%TolOdeRel;           call myprint(LINE,.true.)
        WRITE (LINE,'("  TolOdeHamAbs       = ",e23.15)')  options%TolOdeHamAbs;        call myprint(LINE,.true.)
        WRITE (LINE,'("  TolOdeHamRel       = ",e23.15)')  options%TolOdeHamRel;        call myprint(LINE,.true.)
        WRITE (LINE,'("  TolDenseHamEnd     = ",e23.15)')  options%TolDenseHamEnd;      call myprint(LINE,.true.)
        WRITE (LINE,'("  TolDenseHamX       = ",e23.15)')  options%TolDenseHamX;        call myprint(LINE,.true.)
        WRITE (LINE,'("  TolScaling         = ",e23.15)')  options%TolScaling;          call myprint(LINE,.true.)
        WRITE (LINE,'("  TolX               = ",e23.15)')  options%TolX;                call myprint(LINE,.true.)
        !!!
        WRITE (LINE,'(a)')  ''; call myprint(LINE,.true.);

    end subroutine printOptions

    !!-------------------------------------------------------!!
    !!-------------------------------------------------------!!
    !>  @ingroup modulesInterfaces
    !!  @brief  Get integer option.
    !!  @param[in]  options     stucture containing all options
    !!  @param[in]  optName     name of the option to get
    !!  @param[out] optValue    value of the option
    !!
    !!  \author Olivier Cots
    !!  \date   2009-2013
    !!  \copyright Eclipse Public License
    subroutine getInt(options,optName,optValue)
        type(TOptions), intent(in)  :: options
        character(LEN=*)  , intent(in)  :: optName
        integer,        intent(out) :: optValue

        CHARACTER(len=120)  :: LINE
        select case (trim(optName))
            case ('Dispiter')
                optValue = options%Dispiter
            case ('MaxStepsOde')
                optValue = options%MaxStepsOde
            case ('MaxStepsOdeHam')
                optValue = options%MaxStepsOdeHam
            case ('StopAtTurningPoint')
                optValue = options%StopAtTurningPoint
            case default
            WRITE (LINE,*) 'Invalid option: ', trim(optName), ' or value option: ', optValue, '!'
            CALL printandstop(LINE)
        end select

    end subroutine getInt

    !!-------------------------------------------------------!!
    !!-------------------------------------------------------!!
    !>  @ingroup modulesInterfaces
    !!  @brief  Get real option.
    !!  @param[in]  options     stucture containing all options
    !!  @param[in]  optName     name of the option to get
    !!  @param[out] optValue    value of the option
    !!
    !!  \author Olivier Cots
    !!  \date   2009-2013
    !!  \copyright Eclipse Public License
    subroutine getReal(options,optName,optValue)
        type(TOptions),     intent(in)  :: options
        character(LEN=*),     intent(in)  :: optName
        double precision,   intent(out) :: optValue

        CHARACTER(len=120)  :: LINE
        select case (trim(optName))
            case ('MaxSf')
                optValue = options%MaxSf
            case ('MaxSfunNorm')
                optValue = options%MaxSfunNorm
            case ('MaxStepSizeOde')
                optValue = options%MaxStepSizeOde
            case ('MaxStepSizeOdeHam')
                optValue = options%MaxStepSizeOdeHam
            case ('TolOdeAbs')
                optValue = options%TolOdeAbs
            case ('TolOdeRel')
                optValue = options%TolOdeRel
            case ('TolOdeHamAbs')
                optValue = options%TolOdeHamAbs
            case ('TolOdeHamRel')
                optValue = options%TolOdeHamRel
            case ('TolDenseHamEnd')
                optValue = options%TolDenseHamEnd
            case ('TolDenseHamX')
                optValue = options%TolDenseHamX
            case ('TolScaling')
                optValue = options%TolScaling
            case ('TolX')
                optValue = options%TolX
            case default
            WRITE (LINE,*) 'Invalid option: ', trim(optName), ' or value option: ', optValue, '!'
            CALL printandstop(LINE)
        end select

    end subroutine getReal

    !!-------------------------------------------------------!!
    !!-------------------------------------------------------!!
    !>  @ingroup modulesInterfaces
    !!  @brief  Get string option.
    !!  @param[in]  options     stucture containing all options
    !!  @param[in]  optName     name of the option to get
    !!  @param[out] optValue    value of the option
    !!
    !!  \author Olivier Cots
    !!  \date   2009-2013
    !!  \copyright Eclipse Public License
    subroutine getChar(options,optName,optValue)
        type(TOptions), intent(in)  :: options
        character(LEN=*)  , intent(in)  :: optName
        character(LEN=*)  , intent(out) :: optValue

        CHARACTER(len=120)  :: LINE
        select case (trim(optName))
            case ('Derivative')
                optValue = options%Derivative
            case ('Display')
                optValue = options%Display
            case ('DoSavePath')
                optValue = options%DoSavePath
            case ('FreeFinalTime')
                optValue = options%FreeFinalTime
            case ('ODE')
                optValue = options%ODE
            case ('ODEHam')
                optValue = options%ODEHam
            case default
            WRITE (LINE,*) 'Invalid option: ', trim(optName), ' or value option: ', optValue, '!'
            CALL printandstop(LINE)
        end select

    end subroutine getChar

    !!-------------------------------------------------------!!
    !!-------------------------------------------------------!!
    !>  @ingroup modulesInterfaces
    !!  @brief  Set integer option.
    !!  @param[in,out]   options     stucture containing all options
    !!  @param[in]      optName     name of the option to set
    !!  @param[in]      optValue    value of the option
    !!
    !!  \author Olivier Cots
    !!  \date   2009-2013
    !!  \copyright Eclipse Public License
    subroutine setInt(options,optName,optValue)
        type(TOptions),   intent(inout)   :: options
        character(LEN=*), intent(in)      :: optName
        integer,          intent(in)      :: optValue

        CHARACTER(len=120)  :: LINE
        select case (trim(optName))
            case ('Dispiter')
                  options%Dispiter              = optValue
            case ('MaxStepsOde')
                  options%MaxStepsOde           = optValue
            case ('MaxStepsOdeHam')
                  options%MaxStepsOdeHam        = optValue
            case ('StopAtTurningPoint')
                  options%StopAtTurningPoint    = optValue
            case default
            WRITE (LINE,*) 'Invalid option: ', trim(optName), ' or value option: ', optValue, '!'
            CALL printandstop(LINE)
        end select

    end subroutine setInt

    !!-------------------------------------------------------!!
    !!-------------------------------------------------------!!
    !>  @ingroup modulesInterfaces
    !!  @brief  Set real option.
    !!  @param[in,out]   options     stucture containing all options
    !!  @param[in]      optName     name of the option to set
    !!  @param[in]      optValue    value of the option
    !!
    !!  \author Olivier Cots
    !!  \date   2009-2013
    !!  \copyright Eclipse Public License
    subroutine setReal(options,optName,optValue)
        type(TOptions),     intent(inout)   :: options
        character(LEN=*),   intent(in)      :: optName
        double precision,   intent(in)      :: optValue

        CHARACTER(len=120)  :: LINE
        select case (trim(optName))
            case ('MaxSf')
                  options%MaxSf             = optValue
            case ('MaxSfunNorm')
                  options%MaxSfunNorm       = optValue
            case ('MaxStepSizeOde')
                  options%MaxStepSizeOde    = optValue
            case ('MaxStepSizeOdeHam')
                  options%MaxStepSizeOdeHam = optValue
            case ('TolOdeAbs')
                  options%TolOdeAbs         = optValue
            case ('TolOdeRel')
                  options%TolOdeRel         = optValue
            case ('TolOdeHamAbs')
                  options%TolOdeHamAbs      = optValue
            case ('TolOdeHamRel')
                  options%TolOdeHamRel      = optValue
            case ('TolDenseHamEnd')
                  options%TolDenseHamEnd    = optValue
            case ('TolDenseHamX')
                  options%TolDenseHamX      = optValue
            case ('TolScaling')
                  options%TolScaling        = optValue
            case ('TolX')
                  options%TolX              = optValue
            case default
            WRITE (LINE,*) 'Invalid option: ', trim(optName), ' or value option: ', optValue, '!'
            CALL printandstop(LINE)
        end select

    end subroutine setReal

    !!-------------------------------------------------------!!
    !!-------------------------------------------------------!!
    !>  @ingroup modulesInterfaces
    !!  @brief  Set string option.
    !!  @param[in,out]   options     stucture containing all options
    !!  @param[in]      optName     name of the option to set
    !!  @param[in]      optValue    value of the option
    !!
    !!  \author Olivier Cots
    !!  \date   2009-2013
    !!  \copyright Eclipse Public License
    subroutine setChar(options,optName,optValue)
        type(TOptions), intent(inout)   :: options
        character(LEN=*)  , intent(in)      :: optName
        character(LEN=*)  , intent(in)      :: optValue

        CHARACTER(len=120)  :: LINE
        select case (trim(optName))
            case ('Derivative')
                  options%Derivative    = trim(optValue)
            case ('Display')
                  options%Display       = trim(optValue)
            case ('DoSavePath')
                  options%DoSavePath    = trim(optValue)
            case ('FreeFinalTime')
                  options%FreeFinalTime = trim(optValue)
            case ('ODE')
                  options%ODE           = trim(optValue)
            case ('ODEHam')
                  options%ODEHam        = trim(optValue)
            case default
            WRITE (LINE,*) 'Invalid option: ', trim(optName), ' or value option: ', optValue, '!'
            CALL printandstop(LINE)
        end select

    end subroutine setChar

end module mod_options

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!
