       module mod_sfun
            use utils
            use mod_options
            implicit none
            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   sfun interface.
            !!  \details It is used by fortran user or called by other language interface.
            !!           It makes the link between the user and the core of the program.
            interface sfun
                module procedure sfunz0
                module procedure sfunzi
            end interface
            contains

            !! sfunint(nbarc,ti,n,zi,lpar,par,s,flag,at,rt,inte,ms,mss,tsc)

            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   Link to sfunint() in the simple shooting case
            !!  \details This subroutine is called through mod_sfun::sfun interface.
            !!
            !!  \param[in]  ti      ti = [t0 tf]
            !!  \param[in]  z0      initial flow
            !!  \param[in]  options hampath options
            !!  \param[in]  par     parameters given to hfun
            !!  \param[out] s       shooting value
            !!  \param[out] flag    flag should be 1 (ODE integrator output)
            !!
            !!  \author Olivier Cots
            !!  \date   2009-2013
            !!  \copyright Eclipse Public License
            Subroutine sfunz0(ti, z0, options, par, s, flag)
                implicit none
                double precision,  dimension(:), intent(in)  :: ti
                double precision,  dimension(:), intent(in)  :: z0
                type(TOptions),                  intent(in)  :: options
                double precision,  dimension(:), intent(in)  :: par
                double precision,  dimension(:), intent(out) :: s
                integer,                         intent(out) :: flag

                !local variable
                integer          :: n,nbarc,lpar,ms,ne,ntf
                double precision :: at,rt,mss,tsc,zi(size(z0,1),1)
                character(32)    :: inte,optName,fft

                n       = size(z0,1)/2
                lpar    = size(par)
                nbarc   = 1
                zi(:,1) = z0

                call getOption(options,'TolOdeAbs'     ,at)
                call getOption(options,'TolOdeRel'     ,rt)
                call getOption(options,'ODE'           ,inte)
                call getOption(options,'MaxStepsOde'   ,ms)
                call getOption(options,'MaxStepSizeOde',mss)
                call getOption(options,'TolScaling'    ,tsc)
                call getOption(options,'FreeFinalTime' ,fft)

                call initTtfNe(fft,nbarc,n,ntf,ne)
                call sfunint(nbarc,ti,n,zi,lpar,par,ne,s,flag,at,rt,inte,ms,mss,tsc)

            end subroutine sfunz0

            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   Link to sfunint() in the multiple shooting case
            !!  \details This subroutine is called through mod_sfun::sfun interface.
            !!
            !!  \param[in]  ti      ti = [t0 t1 ... t_{nbarc-1} tf]
            !!  \param[in]  zi      zi = [z(t0) ... z(t_{nbarc-1})]
            !!  \param[in]  options hampath options
            !!  \param[in]  par     parameters given to hfun
            !!  \param[out] s       shooting value
            !!  \param[out] flag    flag should be 1 (ODE integrator output)
            !!
            !!  \author Olivier Cots
            !!  \date   2009-2013
            !!  \copyright Eclipse Public License
            Subroutine sfunzi(ti, zi, options, par, s, flag)
                implicit none
                double precision,  dimension(:),   intent(in)  :: ti
                double precision,  dimension(:,:), intent(in)  :: zi
                type(TOptions),                    intent(in)  :: options
                double precision,  dimension(:),   intent(in)  :: par
                double precision,  dimension(:),   intent(out) :: s
                integer,                           intent(out) :: flag

                !local variable
                integer          :: n,nbarc,lpar,ms,ne,ntf
                double precision :: at,rt,mss,tsc
                character(32)    :: inte,optName,fft

                n       = size(zi,1)/2
                lpar    = size(par)
                nbarc   = size(zi,2)

                call getOption(options,'TolOdeAbs'     ,at)
                call getOption(options,'TolOdeRel'     ,rt)
                call getOption(options,'ODE'           ,inte)
                call getOption(options,'MaxStepsOde'   ,ms)
                call getOption(options,'MaxStepSizeOde',mss)
                call getOption(options,'TolScaling'    ,tsc)
                call getOption(options,'FreeFinalTime' ,fft)

                call initTtfNe(fft,nbarc,n,ntf,ne)
                call sfunint(nbarc,ti,n,zi,lpar,par,ne,s,flag,at,rt,inte,ms,mss,tsc)

            end subroutine sfunzi

      end module mod_sfun

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!


