       module mod_sjac
            use utils
            use mod_options
            implicit none
            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   sjac interface.
            !!  \details It is used by fortran user or called by other language interface.
            !!           It makes the link between the user and the core of the program.
            interface sjac
                module procedure sjaczzero, sjaczzeroipar
                module procedure sjaczi, sjacziipar
            end interface
            contains

            !! sjacint(nbarc,ti,n,zi,lpar,par,lindexpar,indexpar,nl,ne,ntf,fjac,flag,at,rt,inte,ms,der,mss,tsc)

            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   Link to sjacint() in the simple shooting case with no indexpar
            !!  \details This subroutine is called through mod_sjac::sjac interface.
            !!
            !!  \param[in]  ti      ti = [t0 tf]
            !!  \param[in]  z0      initial flow
            !!  \param[in]  options hampath options
            !!  \param[in]  par     parameters given to hfun
            !!  \param[out] fjac    Jacobian of the shooting function
            !!  \param[out] flag    flag should be 1 (ODE integrator output)
            !!
            !!  \author Olivier Cots
            !!  \date   2009-2013
            !!  \copyright Eclipse Public License
            Subroutine sjaczzero(ti, z0, options, par, fjac, flag)
                implicit none
                double precision,  dimension(:),   intent(in)  :: ti
                double precision,  dimension(:),   intent(in)  :: z0
                type(TOptions),                    intent(in)  :: options
                double precision,  dimension(:),   intent(in)  :: par
                double precision,  dimension(:,:), intent(out) :: fjac
                integer,                           intent(out) :: flag

                !local variable
                integer          :: n,nbarc,lpar,ms,ne,ntf,lindexpar,nl
                double precision :: at,rt,mss,tsc,zi(size(z0,1),1),ipardble(1)
                character(32)    :: inte,optName,fft,der

                n         = size(z0,1)/2
                lpar      = size(par)
                nbarc     = 1
                zi(:,1)   = z0
                lindexpar = 0
                ipardble  = 0d0
                nl        = 0

                call getOption(options,'Derivative'    ,der)
                call getOption(options,'TolOdeAbs'     ,at)
                call getOption(options,'TolOdeRel'     ,rt)
                call getOption(options,'ODE'           ,inte)
                call getOption(options,'MaxStepsOde'   ,ms)
                call getOption(options,'MaxStepSizeOde',mss)
                call getOption(options,'TolScaling'    ,tsc)
                call getOption(options,'FreeFinalTime' ,fft)

                call initTtfNe(fft,nbarc,n,ntf,ne)
                call sjacint(nbarc,ti,n,zi,lpar,par,lindexpar,ipardble,nl,ne,ntf,fjac,flag, &
                                at,rt,inte,ms,der,mss,tsc)

            end subroutine sjaczzero

            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   Link to sjacint() in the multiple shooting case with no indexpar
            !!  \details This subroutine is called through mod_sjac::sjac interface.
            !!
            !!  \param[in]  ti      ti = [t0 t1 ... t_{nbarc-1} tf]
            !!  \param[in]  zi      zi = [z(t0) ... z(t_{nbarc-1})]
            !!  \param[in]  options hampath options
            !!  \param[in]  par     parameters given to hfun
            !!  \param[out] fjac    Jacobian of the shooting function
            !!  \param[out] flag    flag should be 1 (ODE integrator output)
            !!
            !!  \author Olivier Cots
            !!  \date   2009-2013
            !!  \copyright Eclipse Public License
            Subroutine sjaczi(ti, zi, options, par, fjac, flag)
                implicit none
                double precision,  dimension(:),   intent(in)  :: ti
                double precision,  dimension(:,:), intent(in)  :: zi
                type(TOptions),                    intent(in)  :: options
                double precision,  dimension(:),   intent(in)  :: par
                double precision,  dimension(:,:), intent(out) :: fjac
                integer,                           intent(out) :: flag

                !local variable
                integer          :: n,nbarc,lpar,ms,ne,ntf,lindexpar,nl
                double precision :: at,rt,mss,tsc,ipardble(1)
                character(32)    :: inte,optName,fft,der

                n         = size(zi,1)/2
                lpar      = size(par)
                nbarc     = size(zi,2)
                lindexpar = 0
                ipardble  = 0d0
                nl        = 0

                call getOption(options,'Derivative'    ,der)
                call getOption(options,'TolOdeAbs'     ,at)
                call getOption(options,'TolOdeRel'     ,rt)
                call getOption(options,'ODE'           ,inte)
                call getOption(options,'MaxStepsOde'   ,ms)
                call getOption(options,'MaxStepSizeOde',mss)
                call getOption(options,'TolScaling'    ,tsc)
                call getOption(options,'FreeFinalTime' ,fft)

                call initTtfNe(fft,nbarc,n,ntf,ne)
                call sjacint(nbarc,ti,n,zi,lpar,par,lindexpar,ipardble,nl,ne,ntf,fjac,flag, &
                                at,rt,inte,ms,der,mss,tsc)

            end subroutine sjaczi

            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   Link to sjacint() in the simple shooting case with indexpar
            !!  \details This subroutine is called through mod_sjac::sjac interface.
            !!
            !!  \param[in]  ti      ti = [t0 tf]
            !!  \param[in]  z0      initial flow
            !!  \param[in]  options hampath options
            !!  \param[in]  par     parameters given to hfun
            !!  \param[in]  indexpar    index of parameters for which the derivative
            !!                          is computed
            !!  \param[out] fjac    Jacobian of the homotopic function
            !!  \param[out] flag    flag should be 1 (ODE integrator output)
            !!
            !!  \author Olivier Cots
            !!  \date   2009-2013
            !!  \copyright Eclipse Public License
            Subroutine sjaczzeroipar(ti, z0, options, par, indexpar, fjac, flag)
                implicit none
                double precision,  dimension(:),   intent(in)  :: ti
                double precision,  dimension(:),   intent(in)  :: z0
                type(TOptions),                    intent(in)  :: options
                double precision,  dimension(:),   intent(in)  :: par
                integer,           dimension(:),   intent(in)  :: indexpar
                double precision,  dimension(:,:), intent(out) :: fjac
                integer,                           intent(out) :: flag

                !local variable
                integer          :: n,nbarc,lpar,ms,ne,ntf,lindexpar,nl,i
                double precision :: at,rt,mss,tsc,zi(size(z0,1),1),ipardble(size(indexpar))
                character(32)    :: inte,optName,fft,der

                n         = size(z0,1)/2
                lpar      = size(par)
                nbarc     = 1
                zi(:,1)   = z0
                lindexpar = size(indexpar)
                if(lindexpar.eq.0)then
                    nl    = 0
                else
                    nl    = 1
                end if
                do i=1,lindexpar
                        ipardble(i) = dble(indexpar(i))
                end do

                call getOption(options,'Derivative'    ,der)
                call getOption(options,'TolOdeAbs'     ,at)
                call getOption(options,'TolOdeRel'     ,rt)
                call getOption(options,'ODE'           ,inte)
                call getOption(options,'MaxStepsOde'   ,ms)
                call getOption(options,'MaxStepSizeOde',mss)
                call getOption(options,'TolScaling'    ,tsc)
                call getOption(options,'FreeFinalTime' ,fft)

                call initTtfNe(fft,nbarc,n,ntf,ne)
                call sjacint(nbarc,ti,n,zi,lpar,par,lindexpar,ipardble,nl,ne,ntf,fjac,flag, &
                                at,rt,inte,ms,der,mss,tsc)

            end subroutine sjaczzeroipar

            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   Link to sjacint() in the multiple shooting case with indexpar
            !!  \details This subroutine is called through mod_sjac::sjac interface.
            !!
            !!  \param[in]  ti      ti = [t0 t1 ... t_{nbarc-1} tf]
            !!  \param[in]  zi      zi = [z(t0) ... z(t_{nbarc-1})]
            !!  \param[in]  options hampath options
            !!  \param[in]  par     parameters given to hfun
            !!  \param[in]  indexpar    index of parameters for which the derivative
            !!                          is computed
            !!  \param[out] fjac    Jacobian of the homotopic function
            !!  \param[out] flag    flag should be 1 (ODE integrator output)
            !!
            !!  \author Olivier Cots
            !!  \date   2009-2013
            !!  \copyright Eclipse Public License
            Subroutine sjacziipar(ti, zi, options, par, indexpar, fjac, flag)
                implicit none
                double precision,  dimension(:),   intent(in)  :: ti
                double precision,  dimension(:,:), intent(in)  :: zi
                type(TOptions),                    intent(in)  :: options
                double precision,  dimension(:),   intent(in)  :: par
                integer,           dimension(:),   intent(in)  :: indexpar
                double precision,  dimension(:,:), intent(out) :: fjac
                integer,                           intent(out) :: flag

                !local variable
                integer          :: n,nbarc,lpar,ms,ne,ntf,lindexpar,nl,i
                double precision :: at,rt,mss,tsc,ipardble(size(indexpar))
                character(32)    :: inte,optName,fft,der

                n         = size(zi,1)/2
                lpar      = size(par)
                nbarc     = size(zi,2)
                lindexpar = size(indexpar)
                if(lindexpar.eq.0)then
                    nl    = 0
                else
                    nl    = 1
                end if
                do i=1,lindexpar
                        ipardble(i) = dble(indexpar(i))
                end do

                call getOption(options,'Derivative'    ,der)
                call getOption(options,'TolOdeAbs'     ,at)
                call getOption(options,'TolOdeRel'     ,rt)
                call getOption(options,'ODE'           ,inte)
                call getOption(options,'MaxStepsOde'   ,ms)
                call getOption(options,'MaxStepSizeOde',mss)
                call getOption(options,'TolScaling'    ,tsc)
                call getOption(options,'FreeFinalTime' ,fft)

                call initTtfNe(fft,nbarc,n,ntf,ne)
                call sjacint(nbarc,ti,n,zi,lpar,par,lindexpar,ipardble,nl,ne,ntf,fjac,flag, &
                                at,rt,inte,ms,der,mss,tsc)

            end subroutine sjacziipar

      end module mod_sjac

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!

