       module mod_ssolve
            use utils
            use mod_options
            implicit none
            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   ssolve interface.
            !!  \details It is used by fortran user or called by other language interface.
            !!           It makes the link between the user and the core of the program.
            interface ssolve
                module procedure ssolvez0
                module procedure ssolvezi
            end interface
            contains

            !! ssolveint(nbarc,ti,n,zi,lpar,par,tisol,zisol,ne,ntf,ssol,nfev,njev,  &
            !!               flag,at,rt,xt,inte,ms,disp,der,mss,tsc)

            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   Link to ssolveint() in the simple shooting case
            !!  \details This subroutine is called through mod_ssolve::ssolve interface.
            !!
            !!  \param[in]  ti0     initial guess for ti = [t0 tf]
            !!  \param[in]  z00     initial guess for the initial flow
            !!  \param[in]  options hampath options
            !!  \param[in]  par     parameters given to hfun
            !!  \param[out] tisol   ti solution
            !!  \param[out] z0sol   z0 solution
            !!  \param[out] ssol    value of sfun at tisol, zisol
            !!  \param[out] nfev    number of evaluations of sfun
            !!  \param[out] njev    number of evaluations of sjac
            !!  \param[out] flag    solver output: should be 1
            !!
            !!  \author Olivier Cots
            !!  \date   2009-2013
            !!  \copyright Eclipse Public License
            Subroutine ssolvez0(ti0, z00, options, par, tisol, z0sol, ssol, nfev, njev, flag)
                implicit none
                double precision,  dimension(:), intent(in)  :: ti0, tisol
                double precision,  dimension(:), intent(in)  :: z00, z0sol
                type(TOptions),                  intent(in)  :: options
                double precision,  dimension(:), intent(in)  :: par
                double precision,  dimension(:), intent(out) :: ssol
                integer,                         intent(out) :: flag, nfev, njev

                !local variable
                integer          :: n,nbarc,lpar,ms,ne,ntf
                double precision :: at,rt,mss,tsc,zi0(size(z00,1),1),xt
                character(32)    :: inte,optName,fft,disp,der

                n        = size(z00,1)/2
                lpar     = size(par)
                nbarc    = 1
                zi0(:,1) = z00

                call getOption(options,'Derivative'    ,der)
                call getOption(options,'Display'       ,disp)
                call getOption(options,'FreeFinalTime' ,fft)
                call getOption(options,'TolOdeAbs'     ,at)
                call getOption(options,'TolOdeRel'     ,rt)
                call getOption(options,'ODE'           ,inte)
                call getOption(options,'MaxStepsOde'   ,ms)
                call getOption(options,'MaxStepSizeOde',mss)
                call getOption(options,'TolScaling'    ,tsc)
                call getOption(options,'TolX'          ,xt)

                call initTtfNe(fft,nbarc,n,ntf,ne)
                call ssolveint(nbarc,ti0,n,z00,lpar,par,tisol,z0sol,ne,ntf,ssol,nfev,njev,flag, &
                                at,rt,xt,inte,ms,disp,der,mss,tsc)

            end subroutine ssolvez0

            !!-------------------------------------------------------!!
            !!-------------------------------------------------------!!
            !>  \ingroup modulesInterfaces
            !!  \brief   Link to ssolveint() in the multiple shooting case
            !!  \details This subroutine is called through mod_ssolve::ssolve interface.
            !!
            !!  \param[in]  ti0     initial guess for ti = [t0 t1 .. t_{nbarc-1} tf]
            !!  \param[in]  zi0     initial guess for zi = [z(t0) ... z(t_{nbarc-1})]
            !!  \param[in]  options hampath options
            !!  \param[in]  par     parameters given to hfun
            !!  \param[out] tisol   ti solution
            !!  \param[out] zisol   zi solution
            !!  \param[out] ssol    value of sfun at tisol, zisol
            !!  \param[out] nfev    number of evaluations of sfun
            !!  \param[out] njev    number of evaluations of sjac
            !!  \param[out] flag    solver output: should be 1
            !!
            !!  \author Olivier Cots
            !!  \date   2009-2013
            !!  \copyright Eclipse Public License
            Subroutine ssolvezi(ti0, zi0, options, par, tisol, zisol, ssol, nfev, njev, flag)
                implicit none
                double precision,  dimension(:),    intent(in)  :: ti0, tisol
                double precision,  dimension(:,:),  intent(in)  :: zi0, zisol
                type(TOptions),                     intent(in)  :: options
                double precision,  dimension(:),    intent(in)  :: par
                double precision,  dimension(:),    intent(out) :: ssol
                integer,                            intent(out) :: flag, nfev, njev

                !local variable
                integer          :: n,nbarc,lpar,ms,ne,ntf
                double precision :: at,rt,mss,tsc,xt
                character(32)    :: inte,optName,fft,disp,der

                n        = size(zi0,1)/2
                lpar     = size(par)
                nbarc    = size(zi0,2)

                call getOption(options,'Derivative'    ,der)
                call getOption(options,'Display'       ,disp)
                call getOption(options,'FreeFinalTime' ,fft)
                call getOption(options,'TolOdeAbs'     ,at)
                call getOption(options,'TolOdeRel'     ,rt)
                call getOption(options,'ODE'           ,inte)
                call getOption(options,'MaxStepsOde'   ,ms)
                call getOption(options,'MaxStepSizeOde',mss)
                call getOption(options,'TolScaling'    ,tsc)
                call getOption(options,'TolX'          ,xt)

                call initTtfNe(fft,nbarc,n,ntf,ne)
                call ssolveint(nbarc,ti0,n,zi0,lpar,par,tisol,zisol,ne,ntf,ssol,nfev,njev,flag, &
                                at,rt,xt,inte,ms,disp,der,mss,tsc)

            end subroutine ssolvezi

      end module mod_ssolve

!!******************************************************************************************!!
! Written on 2009-2013                                                                      !!
! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
!!******************************************************************************************!!


