function [ parout, tiout, ziout, sout, viout, dets, normS, ps, flag ] = hampath(parspan,ti,zi,options)
%-------------------------------------------------------------------------------------------
%
%    hampath (needs efun.f90 and hfun.f90)
%
%    Description
%
%        Interface of the homotopic method.
%
%    Options used
%        Derivative, DispIter, Display, DoSavePath, FreeFinalTime, MaxSf, MaxSfunNorm,
%        MaxStepsOde, MaxStepsOdeHam, MaxStepSizeOde, MaxStepSizeOdeHam, ODE, ODEHam,
%        StopAtTurningPoint, TolOdeAbs, TolOdeRel, TolOdeHamAbs, TolOdeHamRel, 
%        TolDenseHamEnd, TolDenseHamX, TolScaling
%
%-------------------------------------------------------------------------------------------
%
%    Matlab / Octave Usage
%
%        [parout,tiout,ziout,sout,viout,...
%                        dets,normS,ps,flag] = hampath(parspan,ti,zi,options)
%
%    Inputs
%
%        parspan - real matrix, parspan = [par0 parf]. parspan(:,i) is a parameter vector 
%                    given to hfun and efun. The homotopy is from par0 to parf and of
%                    the form : (1-lambda)*par0 + lambda*parf, unless a fortran file with 
%                    the subroutine parfun(lambda,lpar,par0,parf,parout), with lambda 
%                    from 0 to 1, is provided.
%        ti      - real row vector,
%                    ti = [t0 tf]                   : simple shooting
%                    ti = [t0 t1 ... t_nbarc-1 tf] : multiple shooting
%        zi      - real matrix, zi = [z(t0) ... z(t_{nbarc-1})]
%        options - struct, hampathset options
%
%
%    Outputs
%
%        parout  - real matrix, parameters at each integration step.
%        tiout   - real vector, tiout((k-1)*(nbarc+1):k*(nbarc+1)) =  ti at parout(:,k)
%        ziout   - real matrix, zout(:,(k-1)*nbarc:k*nbarc) =  zi at parout(:,k)
%        sout    - real vector, arc length at each integration step
%        viout   - real matrix, viout(:,k) = dot_c(:,k) : tangent vector at parout(:,k)
%                    where c(:,k) = [z0 ... z_{nbarc-1} t1 ... t_{nbarc-1} tf lambda]',
%                    no tf if it is fixed
%        dets    - real vector, det(h'(c(s)),c'(s)) at each integration step, where
%                    h is the homotpic function
%        normS   - real vector, norm of the shooting function at each integration step
%        ps      - real vector, <c'(s_old),c'(s)> at each integ. step
%        flag    - integer, flag should be 1 (ODE integrator output)
%                    if flag==0  then Sfmax is too small
%                    if flag==-1 then input is not consistent
%                    if flag==-2 then larger MaxSteps is needed
%                    if flag==-3 then step size became too small
%                    if flag==-4 then problem is problably stiff
%                    if flag==-5 then |S| is greater than NormeSfunMax
%                    if flag==-6 then the homotopic parameter do not
%                            evolve relatively wrt TolDenseHamX
%                    if flag==-7 then a turning point occurs
%
%
%-------------------------------------------------------------------------------------------
if(nargout~=9)
error('wrong number of outputs: try help');
end

nrhs0min = 4;
if(nargin<nrhs0min)
error('wrong syntax: try help');
end

if(size(parspan,2)==2)
if(abs(parspan(:,1)-parspan(:,2))==0)
parout = parspan(:,1);
tiout  = ti;
ziout  = zi;
sout   = -1;
viout  = [];
bifs   = {};
dets   = -1;
normS  = -1;
ps     = -1;
flag   = -1;
fprintf('\n');
fprintf('Warning: no continuation performed because parspan is not valid.');
fprintf('\n');
return
end
end

str=num2str(clock); str=strrep(str,' ',''); str=strrep(str,'.',''); 
strFinal = [str int2str(ceil(1e3*rand(1)))]; nomfilein=[ 'hampathIn' strFinal '.txt'];
while(exist(nomfilein,'file'))
    strFinal = [str int2str(ceil(1e3*rand(1)))]; nomfilein=[ 'hampathIn' strFinal '.txt'];
end;
nomfileout=[ 'hampathOu' strFinal '.m']; nomfun='hampath'; nominterface='matlab'; nomfunint=[ 'hampathOu' strFinal];

%!Description of input file
%!options
%!n
%!nbarc
%!lpar
%!ti(1,nbarc+1)
%!zi(2*n,nbarc)
%!parspan(lpar,2)

%Create file in
[nn,ntz] = size(zi);        if(mod(nn,2)~=0)    error('Variable zi must have an even number of rows!');     end; n = nn/2;
[m,nti]  = size(ti);        if(m>1)             error('Variable ti must be a row vector!');                 end; nbarc = nti - 1;
                            if(ntz~=nti-1)      error('Variables ti must have one column more than zi!');   end;
[lpar,m] = size(parspan);   if(m~=2)            error('Variable parspan must have two columns!');           end;

fid = fopen(nomfilein,'w');
fprintf(fid,hampathget(options));
fprintf(fid,'%i\n%i\n%i\n',nbarc,n,lpar);
fclose(fid);
save(nomfilein,'-ascii','-double','-append','ti','zi','parspan');

%Call main routine
p = mfilename('fullpath'); [pathstr,name,ext] = fileparts(p);
gfortranMatlab(1); flag = system([pathstr '/mainProg ' nomfilein ' ' nomfileout ' ' nomfun ' ' nominterface ' ' nomfunint]); gfortranMatlab(2);

%Get results from file out
try
[ parout, tiout, ziout, sout, viout, dets, normS, ps, flag ] = eval(nomfunint);
catch err
    if(exist(nomfilein,'file'))  flagAux = system(['rm ' nomfilein]);  end;
    if(exist(nomfileout,'file')) flagAux = system(['rm ' nomfileout]); end;
    error('Problem during execution!');
end;

flagAux = system(['rm ' nomfilein]);
flagAux = system(['rm ' nomfileout]);

return;

%!!******************************************************************************************!!
%! Written on 2009-2013                                                                      !!
%! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
%!!******************************************************************************************!!



