function value = hampathget(options, name)
% hampathget -- Gets hampath package options.
%
% Usage
%  value = hampathget(options, name)
%
% Inputs
%  options - struct, options
%  name    - string, option name
%
% Outputs
%  value   - any, option value
%
% Description - options names and default values are:
%
%  Derivative       - Derivation method.
%                     Names available                    ['finite',{'eqvar'}]
%                     'finite' corresponds to finite differences.
%                     'eqvar'  corresponds to variational equations.
%                     Used by hampath, sjac, ssolve.
%
%  DispIter         - Display each mod(iter,DispIter)==0            [     1 ]
%                     Used by hampath.
%
%  Display          - Display results                       [ 'off' | {'on'}]
%                     Used by hampath, ssolve.
%
%  DoSavePath       - Save path in file "SavedPath(n).m" during continuation
%                                                           [ 'off' | {'on'}]
%                     Used by hampath.
%
%  FreeFinalTime    - Fixed or free final time           ['true' | {'false'}]
%                     Used by hampath, sfun, sjac, ssolve.
%
%  MaxSf            - Maximum final arc length for hampath          [   1e5 ]
%                     Used by hampath.
%
%  MaxSfunNorm      - Maximal norm of the shooting function during homotopy.
%                     Used by hampath                               [   1e-1]
%
%  MaxStepsOde      - Maximum number of steps of the intergrator    [   1e5 ]
%                     Used by exp[d]hvfun, hampath, sfun, sjac, ssolve.
%
%  MaxStepsOdeHam   - Maximum number of steps of the homotopy       [   1e5 ]
%                     Used by hampath.
%
%  MaxStepSizeOde   - Maximum step size                             [     0 ]
%                     If equals 0 then the adaptative step size integrator
%                     has no constraint on the maximal step size.
%                     Used by exp[d]hvfun, hampath, sfun, sjac, ssolve.
%
%  MaxStepSizeOdeHam- Maximum step size during homotopy             [     0 ]
%                     If equals 0 then the adaptative step size integrator
%                     has no constraint on the maximal step size.
%                     Used by hampath.
%
%  ODE              - Integrator name.
%                     ODE names available                         ['dopri5' ]
%                     Used by exp[d]hvfun, hampath, sfun, sjac, ssolve.
%
%  ODEHam           - Integrator name for homotopy.
%                     ODE names available                         ['dopri5' ]
%                     Used by hampath.
%
%  StopAtTurningPoint - Stop or not after turning point             [{0} | 1]
%                     Used by hampath.
%
%  TolOdeAbs        - Absolute error tolerance of the integrator    [ 1e-14 ]
%                     Used by exp[d]hvfun, hampath, sfun, sjac, ssolve.
%
%  TolOdeRel        - Relative error tolerance of the integrator    [  1e-8 ]
%                     Used by exp[d]hvfun, hampath, sfun, sjac, ssolve.
%
%  TolOdeHamAbs     - Absolute error tolerance for hampath          [ 1e-14 ]
%                     Used by hampath.
%
%  TolOdeHamRel     - Relative error tolerance for hampath          [  1e-8 ]
%                     Used by hampath.
%
%  TolDenseHamEnd   - Absolute Dense output tolerance               [  1e-8 ]
%                     Absolute tolerance to detect if the final homotopic
%                     parameter has been reached.
%                     Used by hampath.
%
%  TolDenseHamX     - Relative Dense output tolerance               [  1e-8 ]
%                     The homotopy stops when the homotopic parameter do not
%                     evolve relatively, iteration per iteration, during ten 
%                     following steps.
%                     Used by hampath.
%
%  TolScaling       - Automatic scaling between 0 and 1 for initial absolute
%                     values greater than TolScaling                [  1e-5 ]
%
%  TolX             - Relative tolerance between iterates (ssolve)  [  1e-8 ]
%                     Used by ssolve.
%
%
%  See also
%    hampathset
%

%Derivative
%DispIter
%Display
%DoSavePath
%FreeFinalTime
%MaxSf
%MaxSfunNorm
%MaxStepsOde
%MaxStepsOdeHam
%MaxStepSizeOde
%MaxStepSizeOdeHam
%ODE
%ODEHam
%StopAtTurningPoint
%TolOdeAbs
%TolOdeRel
%TolOdeHamAbs
%TolOdeHamRel
%TolDenseHamEnd
%TolDenseHamX
%TolScaling
%TolX
if(nargout~=1)
error('wrong number of outputs: try help');
end

if(nargin==1)

    value = [   options.Derivative                  '\n'    ...
                int2str(options.DispIter)           '\n'    ...
                options.Display                     '\n'    ...
                options.DoSavePath                  '\n'    ...
                options.FreeFinalTime               '\n'    ...
                num2str(options.MaxSf)              '\n'    ...
                num2str(options.MaxSfunNorm)        '\n'    ...
                int2str(options.MaxStepsOde)        '\n'    ...
                int2str(options.MaxStepsOdeHam)     '\n'    ...
                num2str(options.MaxStepSizeOde)     '\n'    ...
                num2str(options.MaxStepSizeOdeHam)  '\n'    ...
                options.ODE                         '\n'    ...
                options.ODEHam                      '\n'    ...
                int2str(options.StopAtTurningPoint) '\n'    ...
                num2str(options.TolOdeAbs)          '\n'    ...
                num2str(options.TolOdeRel)          '\n'    ...
                num2str(options.TolOdeHamAbs)       '\n'    ...
                num2str(options.TolOdeHamRel)       '\n'    ...
                num2str(options.TolDenseHamEnd)     '\n'    ...
                num2str(options.TolDenseHamX)       '\n'    ...
                num2str(options.TolScaling)         '\n'    ...
                num2str(options.TolX)               '\n' ];

else

    switch lower(name)
        case 'derivative' 
            value = options.Derivative;
        case 'dispiter' 
            value = options.DispIter;
        case 'display' 
            value = options.Display;
        case 'dosavepath' 
            value = options.DoSavePath;
        case 'freefinaltime' 
            value = options.FreeFinalTime;
        case 'maxsf' 
            value = options.MaxSf;
        case 'maxsfunnorm' 
            value = options.MaxSfunNorm;
        case 'maxstepsode' 
            value = options.MaxStepsOde;
        case 'maxstepsodeham' 
            value = options.MaxStepsOdeHam;
        case 'maxstepsizeode' 
            value = options.MaxStepSizeOde;
        case 'maxstepsizeodeham' 
            value = options.MaxStepSizeOdeHam;
        case 'ode' 
            value = options.ODE;
        case 'odeham' 
            value = options.ODEHam;
        case 'stopatturningpoint' 
            value = options.StopAtTurningPoint;
        case 'tolodeabs' 
            value = options.TolOdeAbs;
        case 'toloderel' 
            value = options.TolOdeRel;
        case 'tolodehamabs' 
            value = options.TolOdeHamAbs;
        case 'tolodehamrel' 
            value = options.TolOdeHamRel;
        case 'toldensehamend' 
            value = options.TolDenseHamEnd;
        case 'toldensehamx' 
            value = options.TolDenseHamX;
        case 'tolscaling'
            value = options.TolScaling;
        case 'tolx'
            value = options.TolX;
    otherwise
        error(sprintf('Unrecognized property name ''%s''.', name));
    end;

end;

%!!******************************************************************************************!!
%! Written on 2009-2013                                                                      !!
%! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
%!!******************************************************************************************!!

