function options = hampathset(varargin)
% hampathset -- Sets options for hampath packages.
%
% Usage
%  options = hampathset(name1, value1, ...)
%
% Or
%  options = hampathset(oldopts, name1, value1,...) 
%  creates a copy of oldopts with the named parameters altered
%  with the specified values.
%
% Inputs
%  name1   string, option name
%  value1  any, option value
%  ...
%
% Outputs
%  options struct, options
%
% Description - options names and default values are:
%
%  Derivative       - Derivation method.
%                     Names available                    ['finite',{'eqvar'}]
%                     'finite' corresponds to finite differences.
%                     'eqvar'  corresponds to variational equations.
%                     Used by hampath, sjac, ssolve.
%
%  DispIter         - Display each mod(iter,DispIter)==0            [     1 ]
%                     Used by hampath.
%
%  Display          - Display results                       [ 'off' | {'on'}]
%                     Used by hampath, ssolve.
%
%  DoSavePath       - Save path in file "SavedPath(n).m" during continuation
%                                                           [ {'off'} | 'on']
%                     Used by hampath.
%
%  FreeFinalTime    - Fixed or free final time           ['true' | {'false'}]
%                     Used by hampath, sfun, sjac, ssolve.
%
%  MaxSf            - Maximum final arc length for hampath          [   1e5 ]
%                     Used by hampath.
%
%  MaxSfunNorm      - Maximal norm of the shooting function during homotopy.
%                     Used by hampath                               [   1e-1]
%
%  MaxStepsOde      - Maximum number of steps of the intergrator    [   1e5 ]
%                     Used by exp[d]hvfun, hampath, sfun, sjac, ssolve.
%
%  MaxStepsOdeHam   - Maximum number of steps of the homotopy       [   1e5 ]
%                     Used by hampath.
%
%  MaxStepSizeOde   - Maximum step size                             [     0 ]
%                     If equals 0 then the adaptative step size integrator
%                     has no constraint on the maximal step size.
%                     Used by exp[d]hvfun, hampath, sfun, sjac, ssolve.
%
%  MaxStepSizeOdeHam- Maximum step size during homotopy             [     0 ]
%                     If equals 0 then the adaptative step size integrator
%                     has no constraint on the maximal step size.
%                     Used by hampath.
%
%  ODE              - Integrator name.
%                     ODE names available                         ['dopri5' ]
%                     Used by exp[d]hvfun, hampath, sfun, sjac, ssolve.
%
%  ODEHam           - Integrator name for homotopy.
%                     ODE names available                         ['dopri5' ]
%                     Used by hampath.
%
%  StopAtTurningPoint - Stop or not after turning point             [{0} | 1]
%                     Used by hampath.
%
%  TolOdeAbs        - Absolute error tolerance of the integrator    [ 1e-14 ]
%                     Used by exp[d]hvfun, hampath, sfun, sjac, ssolve.
%
%  TolOdeRel        - Relative error tolerance of the integrator    [  1e-8 ]
%                     Used by exp[d]hvfun, hampath, sfun, sjac, ssolve.
%
%  TolOdeHamAbs     - Absolute error tolerance for hampath          [ 1e-14 ]
%                     Used by hampath.
%
%  TolOdeHamRel     - Relative error tolerance for hampath          [  1e-8 ]
%                     Used by hampath.
%
%  TolDenseHamEnd   - Absolute Dense output tolerance               [  1e-8 ]
%                     Absolute tolerance to detect if the final homotopic
%                     parameter has been reached.
%                     Used by hampath.
%
%  TolDenseHamX     - Relative Dense output tolerance               [  1e-8 ]
%                     The homotopy stops when the homotopic parameter do not
%                     evolve relatively, iteration per iteration, during ten 
%                     following steps.
%                     Used by hampath.
%
%  TolScaling       - Automatic scaling between 0 and 1 for initial absolute
%                     values greater than TolScaling                [  1e-5 ]
%
%  TolX             - Relative tolerance between iterates (ssolve)  [  1e-8 ]
%                     Used by ssolve.
%
%
%  See also
%  hampathget
%
if(nargout~=1)
error('wrong number of outputs: try help');
end

if (mod(nargin, 2) ~= 0)
options = varargin{1};
n = (nargin-1) / 2;
for i=1:nargin-1
varargin{i}=varargin{i+1};
end
else
options.Derivative          = 'eqvar';
options.DispIter            = 1;
options.Display             = 'on';
options.DoSavePath          = 'off';
options.FreeFinalTime       = 'false';
options.MaxSf               = 1e5;
options.MaxSfunNorm         = 1e-1;
options.MaxStepsOde         = 1e5;
options.MaxStepsOdeHam      = 1e5;
options.MaxStepSizeOde      = 0.0;
options.MaxStepSizeOdeHam   = 0.0;
options.ODE                 = 'dopri5';
options.ODEHam              = 'dopri5';
options.StopAtTurningPoint  = 0;
options.TolOdeAbs           = 1e-14;
options.TolOdeRel           = 1e-8;
options.TolOdeHamAbs        = 1e-14;
options.TolOdeHamRel        = 1e-8;
options.TolDenseHamEnd      = 1e-8;
options.TolDenseHamX        = 1e-8;
options.TolScaling          = 1e-5;
options.TolX                = 1e-8;
n = nargin / 2;
end;

for i = 1:n
  switch lower(varargin{1+2*(i-1)})

    case 'derivative' 
        options.Derivative = varargin{2+2*(i-1)};
    case 'dispiter' 
        options.DispIter = varargin{2+2*(i-1)};
    case 'display' 
        options.Display = varargin{2+2*(i-1)};
    case 'dosavepath' 
        options.DoSavePath = varargin{2+2*(i-1)};
    case 'freefinaltime' 
        options.FreeFinalTime = varargin{2+2*(i-1)};
    case 'maxsf' 
        options.MaxSf = varargin{2+2*(i-1)};
    case 'maxsfunnorm' 
        options.MaxSfunNorm = varargin{2+2*(i-1)};
    case 'maxstepsode' 
        options.MaxStepsOde = varargin{2+2*(i-1)};
    case 'maxstepsodeham' 
        options.MaxStepsOdeHam = varargin{2+2*(i-1)};
    case 'maxstepsizeode' 
        options.MaxStepSizeOde = varargin{2+2*(i-1)};
    case 'maxstepsizeodeham' 
        options.MaxStepSizeOdeHam = varargin{2+2*(i-1)};
    case 'ode' 
        options.ODE = varargin{2+2*(i-1)};
    case 'odeham' 
        options.ODEHam = varargin{2+2*(i-1)};
    case 'stopatturningpoint' 
        options.StopAtTurningPoint = varargin{2+2*(i-1)};
    case 'tolodeabs' 
        options.TolOdeAbs = varargin{2+2*(i-1)};
    case 'toloderel' 
        options.TolOdeRel = varargin{2+2*(i-1)};
    case 'tolodehamabs' 
        options.TolOdeHamAbs = varargin{2+2*(i-1)};
    case 'tolodehamrel' 
        options.TolOdeHamRel = varargin{2+2*(i-1)};
    case 'toldensehamend' 
        options.TolDenseHamEnd = varargin{2+2*(i-1)};
    case 'toldensehamx' 
        options.TolDenseHamX = varargin{2+2*(i-1)};
    case 'tolscaling'
        options.TolScaling = varargin{2+2*(i-1)};
    case 'tolx'
        options.TolX = varargin{2+2*(i-1)};

    otherwise
      error(sprintf('Unrecognized property name ''%s''.', ...
                  varargin{1+2*(i-1)}));
  end;
end;

%!!******************************************************************************************!!
%! Written on 2009-2013                                                                      !!
%! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
%!!******************************************************************************************!!

