function [ tisol, zisol, ssol, nfev, njev, flag] = ssolve(ti0,zi0,options,par)
%-------------------------------------------------------------------------------------------
%
%    ssolve (needs efun.f90 and hfun.f90)
%
%    Description
%
%        Interface of the Fortran non linear solver (hybrj) to solve the optimal
%        control problem described by the Fortran subroutines.
%
%    Options used
%        Derivative, Display, FreeFinalTime, MaxStepsOde, MaxStepSizeOde, ODE,
%        TolOdeAbs, TolOdeRel, TolX, TolScaling
%
%-------------------------------------------------------------------------------------------
%
%    Matlab / Octave Usage
%
%        [tisol,zisol,ssol,nfev,njev,flag] = ssolve(ti0,zi0,options,par)
%
%    Inputs
%
%        ti0     - real row vector, intial guess, 
%                    ti0 = [t0 tf]                    : simple shooting
%                    ti0 = [t0 t1 .. t_{nbarc-1} tf] : multiple shooting
%        zi0     - real matrix, initial guess, 
%                    zi0 = [z(t0) ... z(t_{nbarc-1})]
%        options - struct vector, hampathset options
%        par     - real row vector, par in hfun and efun par=[] if no parameters
%
%    Outputs
%
%        tisol   - real row vector, ti solution
%        zisol   - real matrix, zi solution
%        ssol    - real vector, value of sfun at tisol, zisol
%        nfev    - integer, number of evaluations of sfun
%        njev    - integer, number of evaluations of sjac
%        flag    - integer, solver output (should be 1)
%
%-------------------------------------------------------------------------------------------
if(nargout~=6)
error('wrong number of outputs: try help');
end

nrhs0min = 4;
if(nargin<nrhs0min || nargout~=6)
error('wrong syntax: try help');
end

str=num2str(clock); str=strrep(str,' ',''); str=strrep(str,'.',''); 
strFinal = [str int2str(ceil(1e3*rand(1)))]; nomfilein=[ 'ssolveIn' strFinal '.txt'];
while(exist(nomfilein,'file'))
    strFinal = [str int2str(ceil(1e3*rand(1)))]; nomfilein=[ 'ssolveIn' strFinal '.txt'];
end;
nomfileout=[ 'ssolveOu' strFinal '.m']; nomfun='ssolve'; nominterface='matlab'; nomfunint=[ 'ssolveOu' strFinal];

%!Description of input file
%!options
%!n
%!nbarc
%!lpar
%!ti0(1,nbarc+1)
%!zi0(2*n,nbarc)
%!par(lpar,1)

%Create file in
[nn,ntz] = size(zi0);    if(mod(nn,2)~=0)    error('Variable zi0 must have an even number of rows!');      end; n = nn/2;
[m,nti]  = size(ti0);    if(m>1)             error('Variable ti0 must be a row vector!');                  end; nbarc = nti - 1;
                         if(ntz~=nti-1)      error('Variables ti0 must have one column more than zi0!');   end;
[lpar,m] = size(par);    if(m>1)             error('Variable par must be a column vector!');               end;

fid = fopen(nomfilein,'w');
fprintf(fid,hampathget(options));
fprintf(fid,'%i\n%i\n%i\n',nbarc,n,lpar);
fclose(fid);
save(nomfilein,'-ascii','-double','-append','ti0','zi0','par');

%Call main routine
p = mfilename('fullpath'); [pathstr,name,ext] = fileparts(p);
gfortranMatlab(1); flag = system([pathstr '/mainProg ' nomfilein ' ' nomfileout ' ' nomfun ' ' nominterface ' ' nomfunint]); gfortranMatlab(2);

%Get results from file out
%faire un try catch pour toujours supprimer les fichiers meme s'il y a eu une erreur.
%Ecrire quand meme le message d'erreur et stopper l'execution
try
    [ tisol, zisol, ssol, nfev, njev, flag] = eval(nomfunint);
catch err
    if(exist(nomfilein,'file'))  flagAux = system(['rm ' nomfilein]);  end;
    if(exist(nomfileout,'file')) flagAux = system(['rm ' nomfileout]); end;
    error('Problem during execution!');
end;

flagAux = system(['rm ' nomfilein]);
flagAux = system(['rm ' nomfileout]);

return;

%!!******************************************************************************************!!
%! Written on 2009-2013                                                                      !!
%! by Oivier Cots - Math. Institute, Bourgogne Univ / ENSEEIHT-IRIT / INRIA Sophia-Antipolis !!
%!!******************************************************************************************!!


